@@ -10,10 +10,8 @@
 #include "allegro5/allegro_audio.h"
 #include "allegro5/internal/aintern.h"
 #include "allegro5/internal/aintern_audio.h"
-#include "allegro5/internal/aintern_exitfunc.h"
 #include "allegro5/internal/aintern_system.h"
 #include "acodec.h"
-#include "helper.h"
 
 #ifndef ALLEGRO_CFG_ACODEC_FLAC
    #error configuration problem, ALLEGRO_CFG_ACODEC_FLAC not set
@@ -420,8 +418,6 @@ static size_t flac_stream_update(ALLEGRO_AUDIO_STREAM *stream, void *data,
 static void flac_stream_close(ALLEGRO_AUDIO_STREAM *stream)
 {
    FLACFILE *ff = stream->extra;
-   _al_acodec_stop_feed_thread(stream);
-
    al_fclose(ff->fh);
    flac_close(ff);
 }
