@@ -8,7 +8,6 @@
 #include "allegro5/allegro_audio.h"
 #include "allegro5/internal/aintern_audio.h"
 #include "acodec.h"
-#include "helper.h"
 
 ALLEGRO_DEBUG_CHANNEL("wav")
 
@@ -190,7 +189,7 @@ static bool wav_stream_seek(ALLEGRO_AUDIO_STREAM * stream, double time)
    if (time >= wavfile->loop_end)
       return false;
    cpos += cpos % align;
-   return al_fseek(wavfile->f, wavfile->dpos + cpos, ALLEGRO_SEEK_SET);
+   return (al_fseek(wavfile->f, wavfile->dpos + cpos, ALLEGRO_SEEK_SET) != -1);
 }
 
 
@@ -266,9 +265,13 @@ static size_t wav_stream_update(ALLEGRO_AUDIO_STREAM *stream, void *data,
 static void wav_stream_close(ALLEGRO_AUDIO_STREAM *stream)
 {
    WAVFILE *wavfile = (WAVFILE *) stream->extra;
+   ALLEGRO_EVENT quit_event;
+
+   quit_event.type = _KCM_STREAM_FEEDER_QUIT_EVENT_TYPE;
+   al_emit_user_event(al_get_audio_stream_event_source(stream), &quit_event, NULL);
+   al_join_thread(stream->feed_thread, NULL);
+   al_destroy_thread(stream->feed_thread);
 
-   _al_acodec_stop_feed_thread(stream);
-   
    al_fclose(wavfile->f);
    wav_close(wavfile);
    stream->extra = NULL;
@@ -395,9 +398,9 @@ bool _al_save_wav(const char *filename, ALLEGRO_SAMPLE *spl)
    ALLEGRO_FILE *pf = al_fopen(filename, "wb");
 
    if (pf) {
-      bool rvsave = _al_save_wav_f(pf, spl);
-      bool rvclose = al_fclose(pf);
-      return rvsave && rvclose;
+      bool rv = _al_save_wav_f(pf, spl);
+      al_fclose(pf);
+      return rv;
    }
 
    return false;   
