@@ -18,8 +18,7 @@ typedef enum ALLEGRO_AUDIO_DRIVER_ENUM
    ALLEGRO_AUDIO_DRIVER_DSOUND     = 0x20003,
    ALLEGRO_AUDIO_DRIVER_OSS        = 0x20004,
    ALLEGRO_AUDIO_DRIVER_AQUEUE     = 0x20005,
-   ALLEGRO_AUDIO_DRIVER_PULSEAUDIO = 0x20006,
-   ALLEGRO_AUDIO_DRIVER_OPENSL     = 0x20007
+   ALLEGRO_AUDIO_DRIVER_PULSEAUDIO = 0x20006
 } ALLEGRO_AUDIO_DRIVER_ENUM;
 
 typedef struct ALLEGRO_AUDIO_DRIVER ALLEGRO_AUDIO_DRIVER;
@@ -42,18 +41,12 @@ struct ALLEGRO_AUDIO_DRIVER {
 
    unsigned int   (*get_voice_position)(const ALLEGRO_VOICE*);
    int            (*set_voice_position)(ALLEGRO_VOICE*, unsigned int);
-   
-   
-   int            (*allocate_recorder)(ALLEGRO_AUDIO_RECORDER *);
-   void           (*deallocate_recorder)(ALLEGRO_AUDIO_RECORDER *);
 };
 
 extern ALLEGRO_AUDIO_DRIVER *_al_kcm_driver;
 
-const void *_al_voice_update(ALLEGRO_VOICE *voice, ALLEGRO_MUTEX *mutex,
-   unsigned int *samples);
-bool _al_kcm_set_voice_playing(ALLEGRO_VOICE *voice, ALLEGRO_MUTEX *mutex,
-   bool val);
+const void *_al_voice_update(ALLEGRO_VOICE *voice, unsigned int *samples);
+bool _al_kcm_set_voice_playing(ALLEGRO_VOICE *voice, bool val);
 
 /* A voice structure that you'd attach a mixer or sample to. Ideally there
  * would be one ALLEGRO_VOICE per system/hardware voice.
@@ -223,14 +216,14 @@ struct ALLEGRO_AUDIO_STREAM {
 
    void                 *main_buffer;
                         /* Pointer to a single buffer big enough to hold all
-                         * the fragments. Each fragment has additional samples
-                         * at the start for linear/cubic interpolation.
+                         * the fragments. Each fragment has one additional
+                         * sample at the start for linear interpolation.
                          */
 
    void                 **pending_bufs;
    void                 **used_bufs;
                         /* Arrays of offsets into the main_buffer.
-                         * The arrays are each 'buf_count' long.
+                         * The arrays are each 'buf_count + 1' long.
                          *
                          * 'pending_bufs' holds pointers to fragments supplied
                          * by the user which are yet to be handed off to the
@@ -248,11 +241,6 @@ struct ALLEGRO_AUDIO_STREAM {
                           * played.
                           */
 
-   uint64_t              consumed_fragments;
-                         /* Number of complete fragment buffers consumed since
-                          * the stream was started.
-                          */
-
    ALLEGRO_THREAD        *feed_thread;
    volatile bool         quit_feed_thread;
    unload_feeder_t       unload_feeder;
@@ -315,6 +303,7 @@ typedef enum {
 extern void _al_set_error(int error, char* string);
 
 /* Supposedly internal */
+ALLEGRO_KCM_AUDIO_FUNC(int, _al_kcm_get_silence, (ALLEGRO_AUDIO_DEPTH depth));
 ALLEGRO_KCM_AUDIO_FUNC(void*, _al_kcm_feed_stream, (ALLEGRO_THREAD *self, void *vstream));
 
 /* Helper to emit an event that the stream has got a buffer ready to be refilled. */
@@ -333,50 +322,6 @@ ALLEGRO_KCM_AUDIO_FUNC(void, _al_kcm_shutdown_default_mixer, (void));
 ALLEGRO_KCM_AUDIO_FUNC(ALLEGRO_CHANNEL_CONF, _al_count_to_channel_conf, (int num_channels));
 ALLEGRO_KCM_AUDIO_FUNC(ALLEGRO_AUDIO_DEPTH, _al_word_size_to_depth_conf, (int word_size));
 
-ALLEGRO_KCM_AUDIO_FUNC(void, _al_emit_audio_event, (int event_type));
-
-
-/*
- * Recording
- */
- 
-struct ALLEGRO_AUDIO_RECORDER {
-  ALLEGRO_EVENT_SOURCE source;
-  
-  ALLEGRO_THREAD           *thread;
-  ALLEGRO_MUTEX            *mutex;
-  ALLEGRO_COND             *cond;
-                           /* recording is done in its own thread as
-                              implemented by the driver */
-  
-  ALLEGRO_AUDIO_DEPTH      depth;
-  ALLEGRO_CHANNEL_CONF     chan_conf;
-  unsigned int             frequency;
-
-  void                     **fragments;
-                           /* the buffers to record into */
-
-  unsigned int             fragment_count;
-                           /* the number of fragments */
-
-  unsigned int             samples;
-                           /* the number of samples returned at every FRAGMENT event */
-                           
-  size_t                   fragment_size;
-                           /* size in bytes of each fragument */
-
-  unsigned int             sample_size;
-                           /* the size in bytes of each sample */
-  
-  volatile bool            is_recording; 
-                           /* true if the driver should actively be updating
-                              the buffer */
-                              
-  void                     *extra;
-                           /* custom data for the driver to use as needed */
-};
-
-
 #endif
 
 /* vim: set sts=3 sw=3 et: */
