@@ -9,12 +9,10 @@
 #include <math.h>
 #include <stdio.h>
 
-#include "allegro5/allegro.h"
 #include "allegro5/allegro_audio.h"
-#include "allegro5/internal/aintern.h"
-#include "allegro5/internal/aintern_exitfunc.h"
 #include "allegro5/internal/aintern_audio.h"
 #include "allegro5/internal/aintern_audio_cfg.h"
+#include "allegro5/internal/aintern.h"
 
 ALLEGRO_DEBUG_CHANNEL("audio")
 
@@ -28,9 +26,6 @@ ALLEGRO_AUDIO_DRIVER *_al_kcm_driver = NULL;
 #if defined(ALLEGRO_CFG_KCM_OPENAL)
    extern struct ALLEGRO_AUDIO_DRIVER _al_kcm_openal_driver;
 #endif
-#if defined(ALLEGRO_CFG_KCM_OPENSL)
-   extern struct ALLEGRO_AUDIO_DRIVER _al_kcm_opensl_driver;
-#endif
 #if defined(ALLEGRO_CFG_KCM_ALSA)
    extern struct ALLEGRO_AUDIO_DRIVER _al_kcm_alsa_driver;
 #endif
@@ -46,9 +41,6 @@ ALLEGRO_AUDIO_DRIVER *_al_kcm_driver = NULL;
 #if defined(ALLEGRO_CFG_KCM_PULSEAUDIO)
    extern struct ALLEGRO_AUDIO_DRIVER _al_kcm_pulseaudio_driver;
 #endif
-#if defined(ALLEGRO_SDL)
-   extern struct ALLEGRO_AUDIO_DRIVER _al_kcm_sdl_driver;
-#endif
 
 /* Channel configuration helpers */
 
@@ -122,47 +114,18 @@ ALLEGRO_AUDIO_DEPTH _al_word_size_to_depth_conf(int word_size)
    }
 }
 
-/* Function: al_fill_silence
- */
-void al_fill_silence(void *buf, unsigned int samples,
-   ALLEGRO_AUDIO_DEPTH depth, ALLEGRO_CHANNEL_CONF chan_conf)
+/* Returns a silent sample frame. */
+int _al_kcm_get_silence(ALLEGRO_AUDIO_DEPTH depth)
 {
-   size_t bytes = samples * al_get_audio_depth_size(depth) *
-      al_get_channel_count(chan_conf);
-
    switch (depth) {
-      case ALLEGRO_AUDIO_DEPTH_INT8:
-      case ALLEGRO_AUDIO_DEPTH_INT16:
-      case ALLEGRO_AUDIO_DEPTH_INT24:
-      case ALLEGRO_AUDIO_DEPTH_FLOAT32:
-         memset(buf, 0, bytes);
-         break;
       case ALLEGRO_AUDIO_DEPTH_UINT8:
-         memset(buf, 0x80, bytes);
-         break;
-      case ALLEGRO_AUDIO_DEPTH_UINT16:
-         {
-            uint16_t *buffer = buf;
-            size_t n = bytes / sizeof(uint16_t);
-            size_t i;
-
-            for (i = 0; i < n; i++)
-               buffer[i] = 0x8000;
-         }
-         break;
-      case ALLEGRO_AUDIO_DEPTH_UINT24:
-         {
-            uint32_t *buffer = buf;
-            size_t n = bytes / sizeof(uint32_t);
-            size_t i;
-
-            for (i = 0; i < n; i++)
-               buffer[i] = 0x800000;
-         }
-         break;
+         return 0x80;
+      case ALLEGRO_AUDIO_DEPTH_INT16:
+         return 0x8000;
+      case ALLEGRO_AUDIO_DEPTH_INT24:
+         return 0x800000;
       default:
-         ASSERT(false);
-         break;
+         return 0;
    }
 }
 
@@ -171,6 +134,9 @@ static ALLEGRO_AUDIO_DRIVER_ENUM get_config_audio_driver(void)
    ALLEGRO_CONFIG *config = al_get_system_config();
    const char *value;
 
+   if (!config)
+      return ALLEGRO_AUDIO_DRIVER_AUTODETECT;
+
    value = al_get_config_value(config, "audio", "driver");
    if (!value || value[0] == '\0')
       return ALLEGRO_AUDIO_DRIVER_AUTODETECT;
@@ -180,9 +146,6 @@ static ALLEGRO_AUDIO_DRIVER_ENUM get_config_audio_driver(void)
 
    if (0 == _al_stricmp(value, "OPENAL"))
       return ALLEGRO_AUDIO_DRIVER_OPENAL;
-   
-   if (0 == _al_stricmp(value, "OPENSL"))
-      return ALLEGRO_AUDIO_DRIVER_OPENSL;
 
    if (0 == _al_stricmp(value, "OSS"))
       return ALLEGRO_AUDIO_DRIVER_OSS;
@@ -196,50 +159,6 @@ static ALLEGRO_AUDIO_DRIVER_ENUM get_config_audio_driver(void)
    return ALLEGRO_AUDIO_DRIVER_AUTODETECT;
 }
 
-/* Function: al_get_num_audio_output_devices
- */
-int al_get_num_audio_output_devices()
-{
-   if (_al_kcm_driver) {
-      if (_al_kcm_driver->get_output_devices) {
-         _AL_LIST* audio_devices = _al_kcm_driver->get_output_devices();
-         return _al_list_size(audio_devices);
-      }
-      else {
-         return -1;
-      }
-   }
-
-   return 0;
-}
-
-/* Function: al_get_audio_output_device
- */
-const ALLEGRO_AUDIO_DEVICE* al_get_audio_output_device(int index)
-{
-   if (_al_kcm_driver) {
-      if (_al_kcm_driver->get_output_devices) {
-         _AL_LIST* audio_devices = _al_kcm_driver->get_output_devices();
-
-         if (index >= 0 && index < (int)_al_list_size(audio_devices)) {
-            return _al_list_item_data(_al_list_at(audio_devices, index));
-         }
-      }
-      else {
-         return NULL;
-      }
-   }
-
-   return NULL;
-}
-
-/* Function: al_get_audio_device_name
- */
-const char* al_get_audio_device_name(const ALLEGRO_AUDIO_DEVICE * device)
-{
-   return device ? device->name : NULL;
-}
-
 static bool do_install_audio(ALLEGRO_AUDIO_DRIVER_ENUM mode)
 {
    bool retVal;
@@ -256,11 +175,6 @@ static bool do_install_audio(ALLEGRO_AUDIO_DRIVER_ENUM mode)
 
    switch (mode) {
       case ALLEGRO_AUDIO_DRIVER_AUTODETECT:
-#if defined(ALLEGRO_SDL)
-         retVal = do_install_audio(ALLEGRO_AUDIO_DRIVER_SDL);
-         if (retVal)
-            return retVal;
-#endif
 #if defined(ALLEGRO_CFG_KCM_AQUEUE)
          retVal = do_install_audio(ALLEGRO_AUDIO_DRIVER_AQUEUE);
          if (retVal)
@@ -297,12 +211,6 @@ static bool do_install_audio(ALLEGRO_AUDIO_DRIVER_ENUM mode)
          if (retVal)
             return retVal;
 #endif
-#if defined(ALLEGRO_CFG_KCM_OPENSL)
-         retVal = do_install_audio(ALLEGRO_AUDIO_DRIVER_OPENSL);
-         if (retVal)
-            return retVal;
-#endif
-
          _al_set_error(ALLEGRO_INVALID_PARAM, "No audio driver can be used.");
          _al_kcm_driver = NULL;
          return false;
@@ -333,19 +241,6 @@ static bool do_install_audio(ALLEGRO_AUDIO_DRIVER_ENUM mode)
             return false;
          #endif
 
-      case ALLEGRO_AUDIO_DRIVER_OPENSL:
-         #if defined(ALLEGRO_CFG_KCM_OPENSL)
-            if (_al_kcm_opensl_driver.open() == 0) {
-               ALLEGRO_INFO("Using OpenSL driver\n"); 
-               _al_kcm_driver = &_al_kcm_opensl_driver;
-               return true;
-            }
-            return false;
-         #else
-            _al_set_error(ALLEGRO_INVALID_PARAM, "OpenSL not available on this platform");
-            return false;
-         #endif
-
       case ALLEGRO_AUDIO_DRIVER_ALSA:
          #if defined(ALLEGRO_CFG_KCM_ALSA)
             if (_al_kcm_alsa_driver.open() == 0) {
@@ -398,19 +293,6 @@ static bool do_install_audio(ALLEGRO_AUDIO_DRIVER_ENUM mode)
             return false;
          #endif
 
-      case ALLEGRO_AUDIO_DRIVER_SDL:
-         #if defined(ALLEGRO_SDL)
-            if (_al_kcm_sdl_driver.open() == 0) {
-               ALLEGRO_INFO("Using SDL driver\n");
-               _al_kcm_driver = &_al_kcm_sdl_driver;
-               return true;
-            }
-            return false;
-         #else
-            _al_set_error(ALLEGRO_INVALID_PARAM, "SDL not available on this platform");
-            return false;
-         #endif
-
       default:
          _al_set_error(ALLEGRO_INVALID_PARAM, "Invalid audio driver");
          return false;
@@ -421,8 +303,6 @@ static bool do_install_audio(ALLEGRO_AUDIO_DRIVER_ENUM mode)
  */
 bool al_install_audio(void)
 {
-   bool ret;
-
    if (_al_kcm_driver)
       return true;
 
@@ -432,8 +312,7 @@ bool al_install_audio(void)
    _al_kcm_init_destructors();
    _al_add_exit_func(al_uninstall_audio, "al_uninstall_audio");
 
-   ret = do_install_audio(ALLEGRO_AUDIO_DRIVER_AUTODETECT);
-   return ret;
+   return do_install_audio(ALLEGRO_AUDIO_DRIVER_AUTODETECT);
 }
 
 /* Function: al_uninstall_audio
