@@ -406,7 +406,7 @@ bool al_set_sample_instance_speed(ALLEGRO_SAMPLE_INSTANCE *spl, float val)
 {
    ASSERT(spl);
 
-   if (fabsf(val) < (1.0f/64.0f)) {
+   if (fabs(val) < (1.0f/64.0f)) {
       _al_set_error(ALLEGRO_INVALID_PARAM,
          "Attempted to set zero speed");
       return false;
@@ -542,9 +542,13 @@ bool al_set_sample_instance_playing(ALLEGRO_SAMPLE_INSTANCE *spl, bool val)
 {
    ASSERT(spl);
 
-   if (!spl->parent.u.ptr || !spl->spl_data.buffer.ptr) {
-      spl->is_playing = val;
-      return true;
+   if (!spl->parent.u.ptr) {
+      _al_set_error(ALLEGRO_INVALID_OBJECT, "Sample has no parent");
+      return false;
+   }
+   if (!spl->spl_data.buffer.ptr) {
+      _al_set_error(ALLEGRO_INVALID_OBJECT, "Sample has no data");
+      return false;
    }
 
    if (spl->parent.is_voice) {
