@@ -121,10 +121,10 @@ ALLEGRO_AUDIO_STREAM *al_create_audio_stream(size_t fragment_count,
    }
 
    for (i = 0; i < fragment_count; i++) {
-      char *buffer = (char *)stream->main_buffer
-         + i * (MAX_LAG * bytes_per_sample + bytes_per_frag_buf);
-      al_fill_silence(buffer, MAX_LAG, depth, chan_conf);
-      stream->used_bufs[i] = buffer + MAX_LAG * bytes_per_sample;
+      stream->pending_bufs[i] =
+         (char *) stream->main_buffer
+         + i * (MAX_LAG*bytes_per_sample + bytes_per_frag_buf)
+         + MAX_LAG*bytes_per_sample;
    }
 
    al_init_user_event_source(&stream->spl.es);
@@ -214,7 +214,7 @@ unsigned int al_get_available_audio_stream_fragments(
    unsigned int i;
    ASSERT(stream);
 
-   for (i = 0; i < stream->buf_count && stream->used_bufs[i]; i++)
+   for (i = 0; stream->used_bufs[i] && i < stream->buf_count; i++)
       ;
    return i;
 }
@@ -302,25 +302,6 @@ bool al_get_audio_stream_attached(const ALLEGRO_AUDIO_STREAM *stream)
    return (stream->spl.parent.u.ptr != NULL);
 }
 
-/* Function: al_get_audio_stream_played_samples
-*/
-uint64_t al_get_audio_stream_played_samples(const ALLEGRO_AUDIO_STREAM *stream)
-{
-   uint64_t result;
-   ASSERT(stream);
-
-   maybe_lock_mutex(stream->spl.mutex);
-   if (stream->spl.spl_data.buffer.ptr) {
-      result = stream->consumed_fragments * stream->spl.spl_data.len +
-         stream->spl.pos;
-   }
-   else {
-      result = 0;
-   }
-   maybe_unlock_mutex(stream->spl.mutex);
-
-   return result;
-}
 
 /* Function: al_get_audio_stream_fragment
 */
@@ -338,7 +319,7 @@ void *al_get_audio_stream_fragment(const ALLEGRO_AUDIO_STREAM *stream)
    }
    else {
       fragment = stream->used_bufs[0];
-      for (i = 0; i < stream->buf_count-1 && stream->used_bufs[i]; i++) {
+      for (i = 0; stream->used_bufs[i] && i < stream->buf_count-1; i++) {
          stream->used_bufs[i] = stream->used_bufs[i+1];
       }
       stream->used_bufs[i] = NULL;
@@ -480,83 +461,35 @@ bool al_set_audio_stream_playmode(ALLEGRO_AUDIO_STREAM *stream,
 }
 
 
-static void reset_stopped_stream(ALLEGRO_AUDIO_STREAM *stream)
-{
-   const int bytes_per_sample =
-      al_get_channel_count(stream->spl.spl_data.chan_conf) *
-      al_get_audio_depth_size(stream->spl.spl_data.depth);
-   const int fragment_buffer_size =
-      bytes_per_sample * (stream->spl.spl_data.len + MAX_LAG);
-   size_t i, n;
-
-   /* Write silence to the "invisible" part in between fragment buffers to
-    * avoid interpolation artifacts.  It's tempting to zero the complete
-    * memory block in one go but some of the buffers might be getting
-    * refilled.  So they are currently "owned" by the library user and
-    * should not be overwritten.  But zeroing the parts not visible to the
-    * user should be OK.
-    */
-   for (i = 0; i < stream->buf_count; ++i) {
-      al_fill_silence((char *)stream->main_buffer + i * fragment_buffer_size,
-         MAX_LAG, stream->spl.spl_data.depth, stream->spl.spl_data.chan_conf);
-   }
-
-   /* Get the current number of entries in the used_buf list. */
-   for (n = 0; n < stream->buf_count && stream->used_bufs[n]; n++)
-      ;
-
-   /* Move everything from pending_bufs to used_bufs. */
-   i = 0;
-   while (i < stream->buf_count &&
-         n < stream->buf_count &&
-         stream->pending_bufs[i])
-   {
-      stream->used_bufs[n] = stream->pending_bufs[i];
-      stream->pending_bufs[i] = NULL;
-      n++;
-      i++;
-   }
-
-   /* No fragment buffer is currently playing. */
-   stream->spl.spl_data.buffer.ptr = NULL;
-   stream->spl.pos = stream->spl.spl_data.len;
-   stream->spl.pos_bresenham_error = 0;
-   stream->consumed_fragments = 0;
-}
-
-
 /* Function: al_set_audio_stream_playing
  */
 bool al_set_audio_stream_playing(ALLEGRO_AUDIO_STREAM *stream, bool val)
 {
-   bool rc = true;
    ASSERT(stream);
 
    if (stream->spl.parent.u.ptr && stream->spl.parent.is_voice) {
       ALLEGRO_VOICE *voice = stream->spl.parent.u.voice;
-      if (val != stream->spl.is_playing) {
-         rc = _al_kcm_set_voice_playing(voice, voice->mutex, val);
+      bool rc;
+
+      if (val == stream->spl.is_playing) {
+         return true;
       }
+
+      rc = _al_kcm_set_voice_playing(voice, val);
+      if (rc) {
+         stream->spl.is_playing = val;
+      }
+      return rc;
    }
 
-   maybe_lock_mutex(stream->spl.mutex);
+   stream->spl.is_playing = val;
 
-   stream->spl.is_playing = rc && val;
-
-   if (stream->spl.is_playing) {
-      /* We usually emit fragment events when a fragment is used up, but if the
-       * stream has zero pending fragments to begin with then no events would
-       * ever be emitted.
-       */
-      _al_kcm_emit_stream_events(stream);
+   if (!val) {
+      maybe_lock_mutex(stream->spl.mutex);
+      stream->spl.pos = stream->spl.spl_data.len;
+      maybe_unlock_mutex(stream->spl.mutex);
    }
-   else if (!val) {
-      reset_stopped_stream(stream);
-   }
-
-   maybe_unlock_mutex(stream->spl.mutex);
-
-   return rc;
+   return true;
 }
 
 
@@ -582,7 +515,7 @@ bool al_set_audio_stream_fragment(ALLEGRO_AUDIO_STREAM *stream, void *val)
 
    maybe_lock_mutex(stream->spl.mutex);
 
-   for (i = 0; i < stream->buf_count && stream->pending_bufs[i] ; i++)
+   for (i = 0; stream->pending_bufs[i] && i < stream->buf_count; i++)
       ;
    if (i < stream->buf_count) {
       stream->pending_bufs[i] = val;
@@ -618,7 +551,7 @@ bool _al_kcm_refill_stream(ALLEGRO_AUDIO_STREAM *stream)
        * completed buffer into the used array to be refilled.
        */
       for (i = 0;
-            i < stream->buf_count-1 && stream->pending_bufs[i];
+            stream->pending_bufs[i] && i < stream->buf_count-1;
             i++) {
          stream->pending_bufs[i] = stream->pending_bufs[i+1];
       }
@@ -647,8 +580,6 @@ bool _al_kcm_refill_stream(ALLEGRO_AUDIO_STREAM *stream)
          (char *) new_buf - bytes_per_sample * MAX_LAG,
          (char *) old_buf + bytes_per_sample * (spl->pos-MAX_LAG),
          bytes_per_sample * MAX_LAG);
-
-      stream->consumed_fragments++;
    }
 
    stream->spl.pos = 0;
