@@ -14,16 +14,13 @@
 
 #include <string.h>
 #include "allegro5/allegro.h"
-#include "allegro5/allegro_font.h"
 #include "allegro5/internal/aintern.h"
-#include "allegro5/internal/aintern_bitmap.h"
-#include "allegro5/internal/aintern_exitfunc.h"
 #include "allegro5/internal/aintern_vector.h"
+#include "allegro5/allegro_font.h"
+#include "allegro5/internal/aintern_bitmap.h"
 
 #include "font.h"
 
-ALLEGRO_DEBUG_CHANNEL("font")
-
 
 typedef struct
 {
@@ -32,8 +29,6 @@ typedef struct
 } FONT_HANDLER;
 
 
-/* globals */
-static bool font_inited = false;
 static _AL_VECTOR font_handlers;
 
 
@@ -245,23 +240,6 @@ static void color_destroy(ALLEGRO_FONT* f)
 }
 
 
-static int color_get_font_ranges(ALLEGRO_FONT *font, int ranges_count,
-   int *ranges)
-{
-   ALLEGRO_FONT_COLOR_DATA *cf = font->data;
-   int i = 0;
-   while (cf) {
-      if (i < ranges_count) {
-         ranges[i * 2 + 0] = cf->begin;
-         ranges[i * 2 + 1] = cf->end - 1;
-      }
-      i++;
-      cf = cf->next;
-   }
-   return i;
-}
-
-
 /********
  * vtable declarations
  ********/
@@ -276,35 +254,24 @@ ALLEGRO_FONT_VTABLE _al_font_vtable_color = {
     color_render,
     color_destroy,
     color_get_text_dimensions,
-    color_get_font_ranges,
 };
 
 
 static void font_shutdown(void)
 {
-    if (!font_inited)
-       return;
-
     while (!_al_vector_is_empty(&font_handlers)) {
        FONT_HANDLER *h = _al_vector_ref_back(&font_handlers);
        al_ustr_free(h->extension);
        _al_vector_delete_at(&font_handlers, _al_vector_size(&font_handlers)-1);
     }
     _al_vector_free(&font_handlers);
-
-    font_inited = false;
 }
 
 
 /* Function: al_init_font_addon
  */
-bool al_init_font_addon(void)
+void al_init_font_addon(void)
 {
-   if (font_inited) {
-      ALLEGRO_WARN("Font addon already initialised.\n");
-      return true;
-   }
-
    _al_vector_init(&font_handlers, sizeof(FONT_HANDLER));
 
    al_register_font_loader(".bmp", _al_load_bitmap_font);
@@ -314,9 +281,6 @@ bool al_init_font_addon(void)
    al_register_font_loader(".tga", _al_load_bitmap_font);
 
    _al_add_exit_func(font_shutdown, "font_shutdown");
-
-   font_inited = true;
-   return font_inited;
 }
 
 
@@ -378,11 +342,6 @@ ALLEGRO_FONT *al_load_font(char const *filename, int size, int flags)
 
    ASSERT(filename);
 
-   if (!font_inited) {
-      ALLEGRO_ERROR("Font addon not initialised.\n");
-      return NULL;
-   }
-
    ext = strrchr(filename, '.');
    if (!ext)
       return NULL;
