@@ -89,24 +89,20 @@ static int check_if_png(ALLEGRO_FILE *fp)
 /* really_load_png:
  *  Worker routine, used by load_png and load_memory_png.
  */
-static ALLEGRO_BITMAP *really_load_png(png_structp png_ptr, png_infop info_ptr,
-   int flags)
+static ALLEGRO_BITMAP *really_load_png(png_structp png_ptr, png_infop info_ptr)
 {
    ALLEGRO_BITMAP *bmp;
-   png_uint_32 width, height, rowbytes, real_rowbytes;
+   png_uint_32 width, height, rowbytes;
    int bit_depth, color_type, interlace_type;
    double image_gamma, screen_gamma;
    int intent;
    int bpp;
    int number_passes, pass;
-   int num_trans = 0;
    PalEntry pal[256];
-   png_bytep trans;
    ALLEGRO_LOCKED_REGION *lock;
    unsigned char *buf;
-   unsigned char *dest;
-   bool premul = !(flags & ALLEGRO_NO_PREMULTIPLIED_ALPHA);
-   bool index_only;
+   unsigned char *rgba;
+   bool premul = !(al_get_new_bitmap_flags() & ALLEGRO_NO_PREMULTIPLIED_ALPHA);
 
    ALLEGRO_ASSERT(png_ptr && info_ptr);
 
@@ -128,12 +124,9 @@ static ALLEGRO_BITMAP *really_load_png(png_structp png_ptr, png_infop info_ptr,
       png_set_expand(png_ptr);
 
    /* Adds a full alpha channel if there is transparency information
-    * in a tRNS chunk.
-    */
+    * in a tRNS chunk. */
    if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)) {
-      if (!(color_type & PNG_COLOR_MASK_PALETTE))
-         png_set_tRNS_to_alpha(png_ptr);
-      png_get_tRNS(png_ptr, info_ptr, &trans, &num_trans, NULL);
+      png_set_tRNS_to_alpha(png_ptr);
    }
 
    /* Convert 16-bits per colour component to 8-bits per colour component. */
@@ -208,122 +201,73 @@ static ALLEGRO_BITMAP *really_load_png(png_structp png_ptr, png_infop info_ptr,
       return NULL;
    }
 
-   // TODO: can this be different from rowbytes?
-   real_rowbytes = ((bpp + 7) / 8) * width;
-   if (interlace_type == PNG_INTERLACE_ADAM7)
-      buf = al_malloc(real_rowbytes * height);
-   else
-      buf = al_malloc(real_rowbytes);
+   buf = al_malloc(((bpp + 7) / 8) * width);
 
-   if (bpp == 8 && (color_type & PNG_COLOR_MASK_PALETTE) &&
-      (flags & ALLEGRO_KEEP_INDEX))
-   {
-      lock = al_lock_bitmap(bmp, ALLEGRO_PIXEL_FORMAT_SINGLE_CHANNEL_8,
-         ALLEGRO_LOCK_WRITEONLY);
-      index_only = true;
-   }
-   else {
-      lock = al_lock_bitmap(bmp, ALLEGRO_PIXEL_FORMAT_ABGR_8888_LE,
-         ALLEGRO_LOCK_WRITEONLY);
-      index_only = false;
-   }
+   lock = al_lock_bitmap(bmp, ALLEGRO_PIXEL_FORMAT_ABGR_8888_LE,
+      ALLEGRO_LOCK_WRITEONLY);
 
-   /* Read the image, one line at a time (easier to debug!) */
+   /* Read the image, one line at a line (easier to debug!) */
    for (pass = 0; pass < number_passes; pass++) {
       png_uint_32 y;
       unsigned int i;
       unsigned char *ptr;
-      dest = lock->data;
-
+      rgba = lock->data;
       for (y = 0; y < height; y++) {
-         unsigned char *dest_row_start = dest;
-         /* For interlaced pictures, the row needs to be initialized with
-          * the contents of the previous pass.
-          */
-         if (interlace_type == PNG_INTERLACE_ADAM7)
-            ptr = buf + y * real_rowbytes;
-         else
-            ptr = buf;
-         png_read_row(png_ptr, NULL, ptr);
-   
-         switch (bpp) {
-            case 8:
-               if (index_only) {
-                  for (i = 0; i < width; i++) {
-                     *(dest++) = *(ptr++);
-                  }
-               }
-               else if (color_type & PNG_COLOR_MASK_PALETTE) {
-                  for (i = 0; i < width; i++) {
-                     int pix = ptr[0];
-                     ptr++;
-                     dest[0] = pal[pix].r;
-                     dest[1] = pal[pix].g;
-                     dest[2] = pal[pix].b;
-                     if (pix < num_trans) {
-                        int a = trans[pix];
-                        dest[3] = a;
-                        if (premul) {
-                           dest[0] = dest[0] * a / 255;
-                           dest[1] = dest[1] * a / 255;
-                           dest[2] = dest[2] * a / 255;
-                        }
-                     } else {
-                        dest[3] = 255;
-                     }
-                     dest += 4;
-                  }
-               }
-               else {
-                  for (i = 0; i < width; i++) {
-                     int pix = ptr[0];
-                     ptr++;
-                     *(dest++) = pix;
-                     *(dest++) = pix;
-                     *(dest++) = pix;
-                     *(dest++) = 255;
-                  }
-               }
-               break;
-
-            case 24:
-               for (i = 0; i < width; i++) {
-                  uint32_t pix = _AL_READ3BYTES(ptr);
-                  ptr += 3;
-                  *(dest++) = pix & 0xff;
-                  *(dest++) = (pix >> 8) & 0xff;
-                  *(dest++) = (pix >> 16) & 0xff;
-                  *(dest++) = 255;
-               }
-               break;
-
-            case 32:
-               for (i = 0; i < width; i++) {
-                  uint32_t pix = *(uint32_t*)ptr;
-                  int r = pix & 0xff;
-                  int g = (pix >> 8) & 0xff;
-                  int b = (pix >> 16) & 0xff;
-                  int a = (pix >> 24) & 0xff;
-                  ptr += 4;
-
-                  if (premul) {
-                     r = r * a / 255;
-                     g = g * a / 255;
-                     b = b * a / 255;
-                  }
-
-                  *(dest++) = r;
-                  *(dest++) = g;
-                  *(dest++) = b;
-                  *(dest++) = a;
-               }
-               break;
-
-            default:
-               ALLEGRO_ASSERT(bpp == 8 || bpp == 24 || bpp == 32);
-               break;
+         unsigned char *rgba_row_start = rgba;
+         png_read_row(png_ptr, buf, NULL);
+         ptr = buf;
+         if (bpp == 8 && (color_type & PNG_COLOR_MASK_PALETTE)) {
+            for (i = 0; i < width; i++) {
+               int pix = ptr[0];
+               ptr++;
+               *(rgba++) = pal[pix].r;
+               *(rgba++) = pal[pix].g;
+               *(rgba++) = pal[pix].b;
+               *(rgba++) = 255;
+            }
          }
-         dest = dest_row_start + lock->pitch;
+         else if (bpp == 8) {
+            for (i = 0; i < width; i++) {
+               int pix = ptr[0];
+               ptr++;
+               *(rgba++) = pix;
+               *(rgba++) = pix;
+               *(rgba++) = pix;
+               *(rgba++) = 255;
+            }
+         }
+         else if (bpp == 24) {
+            for (i = 0; i < width; i++) {
+               uint32_t pix = READ3BYTES(ptr);
+               ptr += 3;
+               *(rgba++) = pix & 0xff;
+               *(rgba++) = (pix >> 8) & 0xff;
+               *(rgba++) = (pix >> 16) & 0xff;
+               *(rgba++) = 255;
+            }
+         }
+         else {
+            for (i = 0; i < width; i++) {
+               uint32_t pix = bmp_read32(ptr);
+               int r = pix & 0xff;
+               int g = (pix >> 8) & 0xff;
+               int b = (pix >> 16) & 0xff;
+               int a = (pix >> 24) & 0xff;
+               ptr += 4;
+
+               if (premul) {
+                  r = r * a / 255;
+                  g = g * a / 255;
+                  b = b * a / 255;
+               }
+
+               *(rgba++) = r;
+               *(rgba++) = g;
+               *(rgba++) = b;
+               *(rgba++) = a;
+            }
+         }
+         rgba = rgba_row_start + lock->pitch;
       }
    }
 
@@ -340,7 +284,7 @@ static ALLEGRO_BITMAP *really_load_png(png_structp png_ptr, png_infop info_ptr,
 
 /* Load a PNG file from disk, doing colour coversion if required.
  */
-ALLEGRO_BITMAP *_al_load_png_f(ALLEGRO_FILE *fp, int flags)
+ALLEGRO_BITMAP *_al_load_png_f(ALLEGRO_FILE *fp)
 {
    jmp_buf jmpbuf;
    ALLEGRO_BITMAP *bmp;
@@ -392,7 +336,7 @@ ALLEGRO_BITMAP *_al_load_png_f(ALLEGRO_FILE *fp, int flags)
    png_set_sig_bytes(png_ptr, PNG_BYTES_TO_CHECK);
 
    /* Really load the image now. */
-   bmp = really_load_png(png_ptr, info_ptr, flags);
+   bmp = really_load_png(png_ptr, info_ptr);
 
    /* Clean up after the read, and free any memory allocated. */
    png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp) NULL);
@@ -404,7 +348,7 @@ ALLEGRO_BITMAP *_al_load_png_f(ALLEGRO_FILE *fp, int flags)
 
 
 
-ALLEGRO_BITMAP *_al_load_png(const char *filename, int flags)
+ALLEGRO_BITMAP *_al_load_png(const char *filename)
 {
    ALLEGRO_FILE *fp;
    ALLEGRO_BITMAP *bmp;
@@ -415,7 +359,7 @@ ALLEGRO_BITMAP *_al_load_png(const char *filename, int flags)
    if (!fp)
       return NULL;
 
-   bmp = _al_load_png_f(fp, flags);
+   bmp = _al_load_png_f(fp);
 
    al_fclose(fp);
 
@@ -563,8 +507,7 @@ bool _al_save_png_f(ALLEGRO_FILE *fp, ALLEGRO_BITMAP *bmp)
 bool _al_save_png(const char *filename, ALLEGRO_BITMAP *bmp)
 {
    ALLEGRO_FILE *fp;
-   bool retsave;
-   bool retclose;
+   bool result;
 
    ALLEGRO_ASSERT(filename);
    ALLEGRO_ASSERT(bmp);
@@ -575,10 +518,10 @@ bool _al_save_png(const char *filename, ALLEGRO_BITMAP *bmp)
       return false;
    }
 
-   retsave = _al_save_png_f(fp, bmp);
-   retclose = al_fclose(fp);
+   result = _al_save_png_f(fp, bmp);
+   al_fclose(fp);
 
-   return retsave && retclose;
+   return result;
 }
 
 /* vim: set sts=3 sw=3 et: */
