@@ -23,9 +23,6 @@
 
 ALLEGRO_DEBUG_CHANNEL("win_dialog")
 
-#define WM_SHOW_POPUP (WM_APP + 42)
-#define WM_HIDE_MENU (WM_APP + 43)
-#define WM_SHOW_MENU (WM_APP + 44)
 
 /* Reference count for shared resources. */
 static int wlog_count = 0;
@@ -40,16 +37,6 @@ static wchar_t* wlog_edit_control = L"EDIT";
 static bool wlog_unicode = false;
 
 
-bool _al_init_native_dialog_addon(void)
-{
-   return true;
-}
-
-void _al_shutdown_native_dialog_addon(void)
-{
-}
-
-
 static bool select_folder(ALLEGRO_DISPLAY_WIN *win_display,
    ALLEGRO_NATIVE_DIALOG *fd)
 {
@@ -644,203 +631,4 @@ void _al_append_native_text_log(ALLEGRO_NATIVE_DIALOG *textlog)
    }
 }
 
-static bool menu_callback(ALLEGRO_DISPLAY *display, UINT msg, WPARAM wParam, LPARAM lParam,
-                             void *userdata)
-{
-   (void) userdata;
-
-   if (msg == WM_COMMAND && lParam == 0) {
-      const int id = LOWORD(wParam);
-      ALLEGRO_MENU *menu = _al_find_parent_menu_by_id(display, id);
-      if (menu && (al_get_menu_item_flags(menu, id) & ALLEGRO_MENU_ITEM_CHECKBOX)) {
-         /* Toggle the checkbox, since Windows doesn't do that automatically. */
-         al_toggle_menu_item_flags(menu, id, ALLEGRO_MENU_ITEM_CHECKED);
-      }
-      _al_emit_menu_event(display, id);
-      return true;
-   }
-   else if (msg == WM_SYSCOMMAND) {
-      if ((wParam & 0xfff0) == SC_KEYMENU && al_get_display_menu(display) != NULL) {
-         /* Allow the ALT key to open the menu
-          * XXX: do we even want to do this? Should it be optional?
-          */
-         DefWindowProc(al_get_win_window_handle(display), msg, wParam, lParam);
-         return true;
-      }
-   }
-   else if (msg == WM_SHOW_POPUP) {
-      ALLEGRO_MENU *menu = (ALLEGRO_MENU *) lParam;
-      HWND hwnd = al_get_win_window_handle(display);
-      POINT pos;
-      GetCursorPos(&pos);      
-      SetForegroundWindow(hwnd);
-      TrackPopupMenuEx((HMENU) menu->extra1, TPM_LEFTALIGN | TPM_TOPALIGN | TPM_RIGHTBUTTON, pos.x, pos.y, hwnd, NULL);
-
-      return true;
-   }
-   else if (msg == WM_HIDE_MENU) {
-      SetMenu(al_get_win_window_handle(display), NULL);
-      return true;
-   }
-   else if (msg == WM_SHOW_MENU) {
-      ALLEGRO_MENU *menu = (ALLEGRO_MENU *) lParam;
-      SetMenu(al_get_win_window_handle(display), (HMENU) menu->extra1);
-      return true;
-   }
-   else if (msg == WM_MENUSELECT) {
-      /* XXX: could use this as a way to indicate the popup menu was canceled */
-   }
-
-   return false;
-}
-
-static void init_menu_info(MENUITEMINFO *info, ALLEGRO_MENU_ITEM *menu)
-{
-   memset(info, 0, sizeof(*info));
-
-   info->cbSize = sizeof(*info);   
-   info->fMask = MIIM_FTYPE | MIIM_STATE | MIIM_ID | MIIM_SUBMENU | MIIM_STRING | MIIM_CHECKMARKS;
-   info->wID = menu->id;
-
-   if (!menu->caption) {
-      info->fType = MFT_SEPARATOR;
-   }
-   else {
-      info->fType = MFT_STRING;
-      info->dwTypeData = (LPSTR) al_cstr(menu->caption);
-      info->cch = al_ustr_size(menu->caption);
-   }
-
-   if (menu->flags & ALLEGRO_MENU_ITEM_CHECKED) {
-      info->fState |= MFS_CHECKED;
-   }
-
-   if (menu->flags & ALLEGRO_MENU_ITEM_DISABLED) {
-      info->fState |= MFS_DISABLED;
-   }
-
-   if (menu->icon) {
-      /* convert ALLEGRO_BITMAP to HBITMAP (could be moved into a public function) */
-      const int h = al_get_bitmap_height(menu->icon), w = al_get_bitmap_width(menu->icon);
-      HDC hdc;
-      HBITMAP hbmp;
-      BITMAPINFO bi;
-      uint8_t *data = NULL;
-      ALLEGRO_LOCKED_REGION *lock;
-
-      ZeroMemory(&bi, sizeof(BITMAPINFO));
-      bi.bmiHeader.biSize = sizeof(BITMAPINFO);
-      bi.bmiHeader.biWidth = w;
-      bi.bmiHeader.biHeight = -h;
-      bi.bmiHeader.biPlanes = 1;
-      bi.bmiHeader.biBitCount = 32;
-      bi.bmiHeader.biCompression = BI_RGB;
-
-      hdc = GetDC(menu->parent->display ? al_get_win_window_handle(menu->parent->display) : NULL);
-      
-      hbmp = CreateDIBSection(hdc, (BITMAPINFO *)&bi, DIB_RGB_COLORS, (void **)&data, NULL, 0);
-
-      lock = al_lock_bitmap(menu->icon, ALLEGRO_PIXEL_FORMAT_ARGB_8888, ALLEGRO_LOCK_READONLY);
-      memcpy(data, lock->data, w * h * 4);
-      al_unlock_bitmap(menu->icon);
-      
-      info->hbmpUnchecked = hbmp;
-      menu->extra2 = hbmp;
-
-      ReleaseDC(menu->parent->display ? al_get_win_window_handle(menu->parent->display) : NULL, hdc);
-   }
-
-   if (menu->popup) {
-      info->hSubMenu = (HMENU) menu->popup->extra1;
-   }
-
-}
-
-bool _al_init_menu(ALLEGRO_MENU *menu)
-{
-   menu->extra1 = CreateMenu();
-   return menu->extra1 != NULL;
-}
-
-bool _al_init_popup_menu(ALLEGRO_MENU *menu)
-{
-   menu->extra1 = CreatePopupMenu();
-   return menu->extra1 != NULL;
-}
-
-bool _al_insert_menu_item_at(ALLEGRO_MENU_ITEM *item, int i)
-{
-   MENUITEMINFO info;
-   init_menu_info(&info, item);
-  
-   InsertMenuItem((HMENU) item->parent->extra1, i, TRUE, &info);
-
-   return true;
-}
-
-bool _al_update_menu_item_at(ALLEGRO_MENU_ITEM *item, int i)
-{
-   MENUITEMINFO info;
-
-   init_menu_info(&info, item);
-   SetMenuItemInfo((HMENU) item->parent->extra1, i, TRUE, &info);
-
-   if (item->parent->display)
-      DrawMenuBar(al_get_win_window_handle(item->parent->display));
-
-   return true;
-}
-
-bool _al_destroy_menu_item_at(ALLEGRO_MENU_ITEM *item, int i)
-{
-   DeleteMenu((HMENU) item->parent->extra1, i, MF_BYPOSITION);
-
-   if (item->parent->display)
-      DrawMenuBar(al_get_win_window_handle(item->parent->display));
-
-   return true;
-}
-
-bool _al_show_display_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *menu)
-{
-   HWND hwnd = al_get_win_window_handle(display);
-   if (!hwnd) return false;
-
-   ASSERT(menu->extra1);
-   
-   /* Note that duplicate callbacks are automatically filtered out, so it's safe
-      to call this many times. */
-   al_win_add_window_callback(display, menu_callback, NULL);
-
-   PostMessage(al_get_win_window_handle(display), WM_SHOW_MENU, 0, (LPARAM) menu);
-   
-   return true;
-}
-
-bool _al_hide_display_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *menu)
-{
-   HWND hwnd = al_get_win_window_handle(display);
-   if (!hwnd) return false;
-
-   /* Must be run from the main thread to avoid a crash. */
-   PostMessage(al_get_win_window_handle(display), WM_HIDE_MENU, 0, (LPARAM) menu);
-
-   (void) menu;
-   
-   return true;
-}
-
-bool _al_show_popup_menu(ALLEGRO_DISPLAY *display, ALLEGRO_MENU *menu)
-{
-   /* The popup request must come from the main thread. */
-
-   if (!display)
-      return false;
-
-   al_win_add_window_callback(display, menu_callback, NULL);
-   PostMessage(al_get_win_window_handle(display), WM_SHOW_POPUP, 0, (LPARAM) menu);
-
-   return true;
-}
-
 /* vim: set sts=3 sw=3 et: */
