@@ -52,30 +52,11 @@
 
 /*
  * Make an estimate of the scale of the current transformation. 
- * We do this by computing the determinants of the 2D section of the transformation matrix.
  */
 static float get_scale(void)
 {
-#define DET2D(T) (fabs((T)->m[0][0] * (T)->m[1][1] - (T)->m[0][1] * (T)->m[1][0]))
-
    const ALLEGRO_TRANSFORM* t = al_get_current_transform();
-   float scale_sq = DET2D(t);
-   ALLEGRO_DISPLAY* d = al_get_current_display();
-   if (d) {
-      ALLEGRO_BITMAP* b = al_get_target_bitmap();
-      ALLEGRO_TRANSFORM* p = al_get_projection_transform(d);
-      /*
-       * Sub-bitmaps are wonky when it comes to this. Right now they grab the
-       * projection from the parent bitmap (e.g. see _al_ogl_set_target_bitmap).
-       */
-      if (al_is_sub_bitmap(b))
-         b = al_get_parent_bitmap(b);
-      /* Divide by 4.0f as the screen coordinates range from -1 to 1 on both axes. */
-      scale_sq *= DET2D(p) * al_get_bitmap_width(b) * al_get_bitmap_height(b) / 4.0f;
-   }
-   return sqrtf(scale_sq);
-
-#undef DET2D
+   return (hypotf(t->m[0][0], t->m[0][1]) + hypotf(t->m[1][0], t->m[1][1])) / 2;
 }
 
 /* Function: al_draw_line
@@ -152,7 +133,7 @@ void al_draw_triangle(float x1, float y1, float x2, float y2,
       /*
        * If the triangle is very flat, draw it as a line
        */
-      if(fabsf(cross) < 0.0001f) {
+      if(fabs(cross) < 0.0001f) {
          float tx, ty, lx, ly;
          float len;
          /*
@@ -523,7 +504,7 @@ void al_draw_pieslice(float cx, float cy, float r, float start_theta,
    }
    
    if (thickness <= 0) {
-      num_segments = fabsf(delta_theta / (2 * ALLEGRO_PI) * ALLEGRO_PRIM_QUALITY * scale * sqrtf(r));
+      num_segments = fabs(delta_theta / (2 * ALLEGRO_PI) * ALLEGRO_PRIM_QUALITY * scale * sqrtf(r));
 
       if (num_segments < 2)
          num_segments = 2;
@@ -552,7 +533,7 @@ void al_draw_pieslice(float cx, float cy, float r, float start_theta,
       float midpoint_dir_y = sinf(midangle);
       float side_dir_x = cosf(start_theta);
       float side_dir_y = sinf(start_theta);
-      float sine_half_delta = fabsf(side_dir_x * midpoint_dir_y - side_dir_y * midpoint_dir_x); /* Cross product */
+      float sine_half_delta = fabs(side_dir_x * midpoint_dir_y - side_dir_y * midpoint_dir_x); /* Cross product */
       float connect_len = ht / sine_half_delta;
       bool blunt_tip = connect_len > 2 * thickness;
       
@@ -672,7 +653,7 @@ void al_draw_filled_pieslice(float cx, float cy, float r, float start_theta,
    
    ASSERT(r >= 0);
    
-   num_segments = fabsf(delta_theta / (2 * ALLEGRO_PI) * ALLEGRO_PRIM_QUALITY * scale * sqrtf(r));
+   num_segments = fabs(delta_theta / (2 * ALLEGRO_PI) * ALLEGRO_PRIM_QUALITY * scale * sqrtf(r));
 
    if (num_segments < 2)
       num_segments = 2;
@@ -804,7 +785,7 @@ void al_draw_elliptical_arc(float cx, float cy, float rx, float ry, float start_
 
    ASSERT(rx >= 0 && ry >= 0);
    if (thickness > 0) {
-      int num_segments = fabsf(delta_theta / (2 * ALLEGRO_PI) * ALLEGRO_PRIM_QUALITY * scale * sqrtf((rx + ry) / 2.0f));
+      int num_segments = fabs(delta_theta / (2 * ALLEGRO_PI) * ALLEGRO_PRIM_QUALITY * scale * sqrtf((rx + ry) / 2.0f));
       int ii;
 
       if (num_segments < 2)
@@ -823,7 +804,7 @@ void al_draw_elliptical_arc(float cx, float cy, float rx, float ry, float start_
       
       al_draw_prim(vertex_cache, 0, 0, 0, 2 * num_segments, ALLEGRO_PRIM_TRIANGLE_STRIP);
    } else {
-      int num_segments = fabsf(delta_theta / (2 * ALLEGRO_PI) * ALLEGRO_PRIM_QUALITY * scale * sqrtf((rx + ry) / 2.0f));
+      int num_segments = fabs(delta_theta / (2 * ALLEGRO_PI) * ALLEGRO_PRIM_QUALITY * scale * sqrtf((rx + ry) / 2.0f));
       int ii;
 
       if (num_segments < 2)
