@@ -366,27 +366,6 @@ static int draw_prim_raw(ALLEGRO_BITMAP* target, ALLEGRO_BITMAP* texture,
    IDirect3DDevice9_SetSamplerState(device, 0, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP);
    IDirect3DDevice9_SetSamplerState(device, 0, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP);
 
-   if (texture) {
-      if (texture->flags & ALLEGRO_MIN_LINEAR) {
-         IDirect3DDevice9_SetSamplerState(device, 0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
-      }
-      else {
-         IDirect3DDevice9_SetSamplerState(device, 0, D3DSAMP_MINFILTER, D3DTEXF_POINT);
-      }
-      if (texture->flags & ALLEGRO_MAG_LINEAR) {
-         IDirect3DDevice9_SetSamplerState(device, 0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
-      }
-      else {
-         IDirect3DDevice9_SetSamplerState(device, 0, D3DSAMP_MAGFILTER, D3DTEXF_POINT);
-      }
-      if (texture->flags & ALLEGRO_MIPMAP) {
-         IDirect3DDevice9_SetSamplerState(device, 0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
-      }
-      else {
-         IDirect3DDevice9_SetSamplerState(device, 0, D3DSAMP_MIPFILTER, D3DTEXF_NONE);
-      }
-   }
-
    /* Convert vertices for legacy cards */
    if(legacy_card) {
       al_lock_mutex(d3d_mutex);
@@ -584,8 +563,6 @@ void _al_set_d3d_decl(ALLEGRO_DISPLAY* display, ALLEGRO_VERTEX_DECL* ret)
                case ALLEGRO_PRIM_SHORT_2:
                   type = D3DDECLTYPE_SHORT2;
                break;
-               default:
-                  ASSERT(0);
             }
             d3delements[idx].Stream = 0;
             d3delements[idx].Offset = e->offset;
@@ -603,13 +580,12 @@ void _al_set_d3d_decl(ALLEGRO_DISPLAY* display, ALLEGRO_VERTEX_DECL* ret)
             int type = 0;
             switch(e->storage) {
                case ALLEGRO_PRIM_FLOAT_2:
+               case ALLEGRO_PRIM_FLOAT_3:
                   type = D3DDECLTYPE_FLOAT2;
                break;
                case ALLEGRO_PRIM_SHORT_2:
                   type = D3DDECLTYPE_SHORT2;
                break;
-               default:
-                  ASSERT(0);
             }
             d3delements[idx].Stream = 0;
             d3delements[idx].Offset = e->offset;
