@@ -64,6 +64,7 @@ typedef struct ALLEGRO_TTF_FONT_DATA
 {
    FT_Face face;
    int flags;
+   bool no_premultiply_alpha;
    _AL_VECTOR glyph_ranges;  /* sorted array of of ALLEGRO_TTF_GLYPH_RANGE */
 
    _AL_VECTOR page_bitmaps;  /* of ALLEGRO_BITMAP pointers */
@@ -84,7 +85,7 @@ typedef struct ALLEGRO_TTF_FONT_DATA
 
 
 /* globals */
-static bool ttf_inited;
+static bool inited;
 static FT_Library ft;
 static ALLEGRO_FONT_VTABLE vt;
 
@@ -174,6 +175,16 @@ static ALLEGRO_BITMAP *push_new_page(ALLEGRO_TTF_FONT_DATA *data)
     back = _al_vector_alloc_back(&data->page_bitmaps);
     *back = page;
 
+    /* Sometimes OpenGL will partly sample texels from the border of
+     * glyphs. So we better clear the texture to transparency.
+     * XXX This is very slow and avoidable with some effort.
+     */
+    al_store_state(&state, ALLEGRO_STATE_TARGET_BITMAP);
+    al_hold_bitmap_drawing(false);
+    al_set_target_bitmap(*back);
+    al_clear_to_color(al_map_rgba_f(0, 0, 0, 0));
+    al_restore_state(&state);
+
     data->page_pos_x = 0;
     data->page_pos_y = 0;
     data->page_line_height = 0;
@@ -183,8 +194,7 @@ static ALLEGRO_BITMAP *push_new_page(ALLEGRO_TTF_FONT_DATA *data)
 
 
 static unsigned char *alloc_glyph_region(ALLEGRO_TTF_FONT_DATA *data,
-   int ft_index, int w, int h, bool new, ALLEGRO_TTF_GLYPH_DATA *glyph,
-   bool lock_more)
+   int w, int h, bool new, ALLEGRO_TTF_GLYPH_DATA *glyph, bool lock_more)
 {
    ALLEGRO_BITMAP *page;
    bool relock;
@@ -203,8 +213,7 @@ static unsigned char *alloc_glyph_region(ALLEGRO_TTF_FONT_DATA *data,
    w4 = align4(w);
    h4 = align4(h);
 
-   ALLEGRO_DEBUG("Glyph %d: %dx%d (%dx%d)%s\n",
-      ft_index, w, h, w4, h4, new ? " new" : "");
+   ALLEGRO_DEBUG("glyph %dx%d (%dx%d)%s\n", w, h, w4, h4, new ? " new" : "");
 
    if (data->page_pos_x + w4 > al_get_bitmap_width(page)) {
       data->page_pos_y += data->page_line_height;
@@ -215,7 +224,7 @@ static unsigned char *alloc_glyph_region(ALLEGRO_TTF_FONT_DATA *data,
    }
 
    if (data->page_pos_y + h4 > al_get_bitmap_height(page)) {
-      return alloc_glyph_region(data, ft_index, w, h, true, glyph, lock_more);
+      return alloc_glyph_region(data, w, h, true, glyph, lock_more);
    }
 
    glyph->page_bitmap = page;
@@ -232,7 +241,7 @@ static unsigned char *alloc_glyph_region(ALLEGRO_TTF_FONT_DATA *data,
 
    if (relock) {
       char *ptr;
-      int i;
+      int n;
       unlock_current_page(data);
 
       data->lock_rect.x = glyph->region.x;
@@ -254,18 +263,17 @@ static unsigned char *alloc_glyph_region(ALLEGRO_TTF_FONT_DATA *data,
          data->lock_rect.w, data->lock_rect.h,
          ALLEGRO_PIXEL_FORMAT_ABGR_8888_LE, ALLEGRO_LOCK_WRITEONLY);
 
-      if (!data->page_lr) {
-         return NULL;
-      }
-
       /* Clear the data so we don't get garbage when using filtering
        * FIXME We could clear just the border but I'm not convinced that
        * would be faster (yet)
        */
-      for (i = 0; i < data->lock_rect.h; i++) {
-          ptr = (char *)(data->page_lr->data) + (i * data->page_lr->pitch);
-          memset(ptr, 0, data->lock_rect.w * 4);
+      ptr = data->page_lr->data;
+      n = data->lock_rect.h * data->page_lr->pitch;
+      if (n < 0) {
+         ptr += n - data->page_lr->pitch;
+         n = -n;
       }
+      memset(ptr, 0, n);
    }
 
    ASSERT(data->page_lr);
@@ -288,7 +296,7 @@ static void copy_glyph_mono(ALLEGRO_TTF_FONT_DATA *font_data, FT_Face face,
       unsigned char *dptr = glyph_data + pitch * y;
       int bit = 0;
 
-      if (font_data->flags & ALLEGRO_NO_PREMULTIPLIED_ALPHA) {
+      if (font_data->no_premultiply_alpha) {
          for (x = 0; x < face->glyph->bitmap.width; x++) {
             unsigned char set = ((*ptr >> (7-bit)) & 1) ? 255 : 0;
             *dptr++ = 255;
@@ -328,7 +336,7 @@ static void copy_glyph_color(ALLEGRO_TTF_FONT_DATA *font_data, FT_Face face,
       unsigned char const *ptr = face->glyph->bitmap.buffer + face->glyph->bitmap.pitch * y;
       unsigned char *dptr = glyph_data + pitch * y;
 
-      if (font_data->flags & ALLEGRO_NO_PREMULTIPLIED_ALPHA) {
+      if (font_data->no_premultiply_alpha) {
          for (x = 0; x < face->glyph->bitmap.width; x++) {
             unsigned char c = *ptr;
             *dptr++ = 255;
@@ -393,19 +401,14 @@ static void cache_glyph(ALLEGRO_TTF_FONT_DATA *font_data, FT_Face face,
        /* Mark this glyph so we won't try to cache it next time. */
        glyph->region.x = -1;
        glyph->region.y = -1;
-       ALLEGRO_DEBUG("Glyph %d has zero size.\n", ft_index);
        return;
     }
 
     /* Each glyph has a 1-pixel border all around. Note: The border is kept
      * even against the outer bitmap edge, to ensure consistent rendering.
      */
-    glyph_data = alloc_glyph_region(font_data, ft_index,
-       w + 2, h + 2, false, glyph, lock_more);
-
-    if (glyph_data == NULL) {
-       return;
-    }
+    glyph_data = alloc_glyph_region(font_data, w + 2, h + 2, false, glyph,
+      lock_more);
 
     if (font_data->flags & ALLEGRO_TTF_MONOCHROME)
        copy_glyph_mono(font_data, face, glyph_data);
@@ -441,15 +444,6 @@ static int render_glyph(ALLEGRO_FONT const *f,
    FT_Face face = data->face;
    ALLEGRO_TTF_GLYPH_DATA *glyph = get_glyph(data, ft_index);
    int advance = 0;
-   ALLEGRO_DISPLAY *display;
-   ALLEGRO_TRANSFORM old_projection_transform;
-
-   /* Workabout for bug 3484535 */
-   display = al_get_current_display();
-   if (display) {
-      al_copy_transform(&old_projection_transform,
-         al_get_projection_transform(display));
-   }
 
    /* We don't try to cache all glyphs in a pre-pass before drawing them.
     * While that would indeed save us making separate texture uploads, it
@@ -459,11 +453,6 @@ static int render_glyph(ALLEGRO_FONT const *f,
     */
    cache_glyph(data, face, ft_index, glyph, false);
 
-   /* Workabout for bug 3484535 */
-   if (display) {
-      al_set_projection_transform(display, &old_projection_transform);
-   }
-
    advance += get_kerning(data, face, prev_ft_index, ft_index);
 
    if (glyph->page_bitmap) {
@@ -474,9 +463,6 @@ static int render_glyph(ALLEGRO_FONT const *f,
          xpos + glyph->offset_x + advance,
          ypos + glyph->offset_y, 0);
    }
-   else if (glyph->region.x > 0) {
-      ALLEGRO_ERROR("Glyph %d not on any page.\n", ft_index);
-   }
 
    advance += glyph->advance;
 
@@ -724,7 +710,6 @@ ALLEGRO_FONT *al_load_ttf_font_f(ALLEGRO_FILE *file,
     return al_load_ttf_font_stretch_f(file, filename, 0, size, flags);
 }
 
-
 /* Function: al_load_ttf_font_stretch_f
  */
 ALLEGRO_FONT *al_load_ttf_font_stretch_f(ALLEGRO_FILE *file,
@@ -807,6 +792,8 @@ ALLEGRO_FONT *al_load_ttf_font_stretch_f(ALLEGRO_FILE *file,
 
     data->face = face;
     data->flags = flags;
+    data->no_premultiply_alpha =
+       (al_get_new_bitmap_flags() & ALLEGRO_NO_PREMULTIPLIED_ALPHA);
 
     _al_vector_init(&data->glyph_ranges, sizeof(ALLEGRO_TTF_GLYPH_RANGE));
     _al_vector_init(&data->page_bitmaps, sizeof(ALLEGRO_BITMAP*));
@@ -830,11 +817,10 @@ ALLEGRO_FONT *al_load_ttf_font(char const *filename, int size, int flags)
    return al_load_ttf_font_stretch(filename, 0, size, flags);
 }
 
-
 /* Function: al_load_ttf_font_stretch
  */
 ALLEGRO_FONT *al_load_ttf_font_stretch(char const *filename, int w, int h,
-   int flags)
+    int flags)
 {
    ALLEGRO_FILE *f;
    ALLEGRO_FONT *font;
@@ -854,45 +840,12 @@ ALLEGRO_FONT *al_load_ttf_font_stretch(char const *filename, int w, int h,
 }
 
 
-static int ttf_get_font_ranges(ALLEGRO_FONT *font, int ranges_count,
-   int *ranges)
-{
-   ALLEGRO_TTF_FONT_DATA *data = font->data;
-   FT_UInt g;
-   FT_ULong unicode = FT_Get_First_Char(data->face, &g);
-   int i = 0;
-   if (i < ranges_count) {
-      ranges[i * 2 + 0] = unicode;
-      ranges[i * 2 + 1] = unicode;
-   }
-   while (g) {
-      FT_ULong unicode2 = FT_Get_Next_Char(data->face, unicode, &g);
-      if (unicode + 1 != unicode2) {
-         if (i < ranges_count) {
-            ranges[i * 2 + 1] = unicode;
-            if (i + 1 < ranges_count) {
-               ranges[(i + 1) * 2 + 0] = unicode2;
-            }
-         }
-         i++;
-      }
-      if (i < ranges_count) {
-         ranges[i * 2 + 1] = unicode2;
-      }
-      unicode = unicode2;
-   }
-   return i;
-}
-
-
 /* Function: al_init_ttf_addon
  */
 bool al_init_ttf_addon(void)
 {
-   if (ttf_inited) {
-      ALLEGRO_WARN("TTF addon already initialised.\n");
-      return true;
-   }
+   if (inited) return false;
+   inited = true;
 
    FT_Init_FreeType(&ft);
    vt.font_height = ttf_font_height;
@@ -904,16 +857,13 @@ bool al_init_ttf_addon(void)
    vt.render = ttf_render;
    vt.destroy = ttf_destroy;
    vt.get_text_dimensions = ttf_get_text_dimensions;
-   vt.get_font_ranges = ttf_get_font_ranges;
 
    al_register_font_loader(".ttf", al_load_ttf_font);
-
    /* Can't fail right now - in the future we might dynamically load
     * the FreeType DLL here and/or initialize FreeType (which both
     * could fail and would cause a false return).
     */
-   ttf_inited = true;
-   return ttf_inited;
+   return true;
 }
 
 
@@ -921,16 +871,12 @@ bool al_init_ttf_addon(void)
  */
 void al_shutdown_ttf_addon(void)
 {
-   if (!ttf_inited) {
-      ALLEGRO_ERROR("TTF addon not initialised.\n");
-      return;
-   }
+   if (!inited) return;
+   inited = false;
 
    al_register_font_loader(".ttf", NULL);
 
    FT_Done_FreeType(ft);
-
-   ttf_inited = false;
 }
 
 
