@@ -42,13 +42,11 @@ static void draw(ALLEGRO_JOYSTICK *curr_joy)
 
    if (curr_joy) {
       al_get_joystick_state(curr_joy, &joystate);
-      for (i = 0; i < al_get_joystick_num_sticks(curr_joy); i++) {
-         al_draw_filled_circle(
-               x+joystate.stick[i].axis[0]*20 + i * 80,
-               y+joystate.stick[i].axis[1]*20,
-               20, al_map_rgb(255, 255, 255)
+      al_draw_filled_circle(
+            x+joystate.stick[0].axis[0]*20,
+            y+joystate.stick[0].axis[1]*20,
+            20, al_map_rgb(255, 255, 255)
             );
-      }
       for (i = 0; i < al_get_joystick_num_buttons(curr_joy); i++) {
          if (joystate.button[i]) {
             al_draw_filled_circle(
@@ -61,21 +59,20 @@ static void draw(ALLEGRO_JOYSTICK *curr_joy)
    al_flip_display();
 }
 
-int main(int argc, char **argv)
+int main(void)
 {
    int num_joysticks;
    ALLEGRO_EVENT_QUEUE *queue;
    ALLEGRO_JOYSTICK *curr_joy;
    ALLEGRO_DISPLAY *display;
 
-   (void)argc;
-   (void)argv;
-
    if (!al_init()) {
       abort_example("Could not init Allegro.\n");
+      return 1;
    }
    if (!al_install_joystick()) {
       abort_example("Could not init joysticks.\n");
+      return 1;
    }
    al_install_keyboard();
    al_init_primitives_addon();
@@ -85,6 +82,7 @@ int main(int argc, char **argv)
    display = al_create_display(640, 480);
    if (!display) {
       abort_example("Could not create display.\n");
+      return 1;
    }
 
    queue = al_create_event_queue();
@@ -124,6 +122,7 @@ int main(int argc, char **argv)
          }
       }
       else if (event.type == ALLEGRO_EVENT_JOYSTICK_CONFIGURATION) {
+         int old = al_get_num_joysticks();
          al_reconfigure_joysticks();
          num_joysticks = al_get_num_joysticks();
          log_printf("after reconfiguration num joysticks = %d\n",
@@ -132,7 +131,9 @@ int main(int argc, char **argv)
             log_printf("current joystick is: %s\n",
                al_get_joystick_active(curr_joy) ? "active" : "inactive");
          }
-         curr_joy = al_get_joystick(0);
+         if (old < num_joysticks) {
+            curr_joy = al_get_joystick(0);
+         }
       }
       else if (event.type == ALLEGRO_EVENT_JOYSTICK_AXIS) {
          log_printf("axis event from %p, stick %d, axis %d\n", event.joystick.id, event.joystick.stick, event.joystick.axis);
@@ -140,11 +141,7 @@ int main(int argc, char **argv)
       else if (event.type == ALLEGRO_EVENT_JOYSTICK_BUTTON_DOWN) {
          log_printf("button down event %d from %p\n",
             event.joystick.button, event.joystick.id);
-      } 
-      else if (event.type == ALLEGRO_EVENT_JOYSTICK_BUTTON_UP) {
-         log_printf("button up event %d from %p\n",
-            event.joystick.button, event.joystick.id);
-      } 
+      }
 
       draw(curr_joy);
    }
