@@ -54,32 +54,25 @@
 #endif
 
 #define ALLEGRO_VERSION          5
-#define ALLEGRO_SUB_VERSION      2
-#define ALLEGRO_WIP_VERSION      1
-
-#ifdef ALLEGRO_UNSTABLE
-   #define ALLEGRO_UNSTABLE_BIT  1 << 31
-#else
-   #define ALLEGRO_UNSTABLE_BIT  0
-#endif
+#define ALLEGRO_SUB_VERSION      0
+#define ALLEGRO_WIP_VERSION      8
 
 /* Not sure we need it, but since ALLEGRO_VERSION_STR contains it:
- * 0 = GIT
+ * 0 = SVN
  * 1 = first release
  * 2... = hotfixes?
  *
  * Note x.y.z (= x.y.z.0) has release number 1, and x.y.z.1 has release
  * number 2, just to confuse you.
  */
-#define ALLEGRO_RELEASE_NUMBER   2
+#define ALLEGRO_RELEASE_NUMBER   1
 
-#define ALLEGRO_VERSION_STR      "5.2.1"
-#define ALLEGRO_DATE_STR         "2016"
-#define ALLEGRO_DATE             20160813    /* yyyymmdd */
+#define ALLEGRO_VERSION_STR      "5.0.8"
+#define ALLEGRO_DATE_STR         "2012"
+#define ALLEGRO_DATE             20121118    /* yyyymmdd */
 #define ALLEGRO_VERSION_INT \
     ((ALLEGRO_VERSION << 24) | (ALLEGRO_SUB_VERSION << 16) | \
-    (ALLEGRO_WIP_VERSION << 8) | ALLEGRO_RELEASE_NUMBER | \
-    ALLEGRO_UNSTABLE_BIT)
+    (ALLEGRO_WIP_VERSION << 8) | ALLEGRO_RELEASE_NUMBER)
 
 AL_FUNC(uint32_t, al_get_allegro_version, (void));
 AL_FUNC(int, al_run_main, (int argc, char **argv, int (*)(int, char **)));
@@ -94,6 +87,13 @@ AL_FUNC(int, al_run_main, (int argc, char **argv, int (*)(int, char **)));
 
 #define AL_ID(a,b,c,d)     (((a)<<24) | ((b)<<16) | ((c)<<8) | (d))
 
+typedef struct _AL_DRIVER_INFO      /* info about a hardware driver */
+{
+   int id;                          /* integer ID */
+   void *driver;                    /* the driver structure */
+   int autodetect;                  /* set to allow autodetection */
+} _AL_DRIVER_INFO;
+
        
 
 #ifdef __cplusplus
