@@ -3,10 +3,6 @@
 
 #include "allegro5/internal/aintern_float.h"
 
-#ifdef __cplusplus
-   extern "C" {
-#endif
-
 
 #define _AL_MAP_RGBA(_color, _r, _g, _b, _a)                                  \
    do {                                                                       \
@@ -222,18 +218,10 @@
          case ALLEGRO_PIXEL_FORMAT_RGBA_4444: {                               \
             uint16_t _gp_pixel = *(uint16_t *)(data);                         \
             _AL_MAP_RGBA(color,                                               \
-               _al_rgb_scale_4[(_gp_pixel & 0xF000) >> 12],                   \
-               _al_rgb_scale_4[(_gp_pixel & 0x0F00) >> 8],                    \
-               _al_rgb_scale_4[(_gp_pixel & 0x00F0) >> 4],                    \
-               _al_rgb_scale_4[(_gp_pixel & 0x000F)]);                        \
-            if (advance)                                                      \
-               data += 2;                                                     \
-            break;                                                            \
-         }                                                                    \
-                                                                              \
-         case ALLEGRO_PIXEL_FORMAT_SINGLE_CHANNEL_8: {                        \
-            uint8_t c = *(uint8_t *)(data);                                   \
-            _AL_MAP_RGBA(color, c, c, c, 255);                                \
+               _al_rgb_scale_4[(_gp_pixel & 0xF000) >> 12],                      \
+               _al_rgb_scale_4[(_gp_pixel & 0x0F00) >> 8],                       \
+               _al_rgb_scale_4[(_gp_pixel & 0x00F0) >> 4],                       \
+               _al_rgb_scale_4[(_gp_pixel & 0x000F)]);                           \
             if (advance)                                                      \
                data += 2;                                                     \
             break;                                                            \
@@ -248,13 +236,13 @@
          case ALLEGRO_PIXEL_FORMAT_ANY_24_NO_ALPHA:                           \
          case ALLEGRO_PIXEL_FORMAT_ANY_32_NO_ALPHA:                           \
          case ALLEGRO_PIXEL_FORMAT_ANY_32_WITH_ALPHA:                         \
-            ALLEGRO_ERROR("INLINE_GET got fake pixel format: %d\n", format); \
+            ALLEGRO_ERROR("INLINE_GET got fake _gp_pixel format: %d\n", format); \
             abort();                                                          \
             break;                                                            \
                                                                               \
          case ALLEGRO_NUM_PIXEL_FORMATS:                                      \
          default:                                                             \
-            ALLEGRO_ERROR("INLINE_GET got non pixel format: %d\n", format); \
+            ALLEGRO_ERROR("INLINE_GET got non _gp_pixel format: %d\n", format); \
             abort();                                                          \
             break;                                                            \
       }                                                                       \
@@ -439,14 +427,6 @@
                data += 2;                                                     \
             break;                                                            \
                                                                               \
-         case ALLEGRO_PIXEL_FORMAT_SINGLE_CHANNEL_8: {                        \
-            uint8_t c = color.r;                                              \
-            *(uint8_t *)data = c;                                             \
-            if (advance)                                                      \
-               data += 3;                                                     \
-            break;                                                            \
-         }                                                                    \
-                                                                              \
          case ALLEGRO_PIXEL_FORMAT_ANY:                                       \
          case ALLEGRO_PIXEL_FORMAT_ANY_NO_ALPHA:                              \
          case ALLEGRO_PIXEL_FORMAT_ANY_WITH_ALPHA:                            \
@@ -472,17 +452,8 @@ AL_ARRAY(int, _al_rgb_scale_4);
 AL_ARRAY(int, _al_rgb_scale_5);
 AL_ARRAY(int, _al_rgb_scale_6);
 AL_ARRAY(float, _al_u8_to_float);
-
-void _al_init_pixels(void);
-bool _al_pixel_format_has_alpha(int format);
-bool _al_pixel_format_is_real(int format);
-int _al_get_real_pixel_format(ALLEGRO_DISPLAY *display, int format);
-char const *_al_pixel_format_name(ALLEGRO_PIXEL_FORMAT format);
-
-
-#ifdef __cplusplus
-   }
-#endif
+AL_FUNC(char const *, _al_format_name, (ALLEGRO_PIXEL_FORMAT format));
+AL_FUNC(void, _al_init_pixels, (void));
 
 #endif
 
