@@ -245,16 +245,15 @@ ALLEGRO_DISPLAY_INTERFACE *_al_display_gp2xwiz_opengl_driver(void)
    gp2xwiz_vt->update_display_region = gp2xwiz_update_display_region_ogl;
    gp2xwiz_vt->acknowledge_resize = gp2xwiz_acknowledge_resize_ogl;
    gp2xwiz_vt->create_bitmap = _al_ogl_create_bitmap;
+   gp2xwiz_vt->create_sub_bitmap = _al_ogl_create_sub_bitmap;
    gp2xwiz_vt->get_backbuffer = _al_ogl_get_backbuffer;
    gp2xwiz_vt->set_target_bitmap = _al_ogl_set_target_bitmap;
    gp2xwiz_vt->is_compatible_bitmap = gp2xwiz_is_compatible_bitmap_ogl;
    gp2xwiz_vt->resize_display = gp2xwiz_resize_display_ogl;
-   gp2xwiz_vt->set_icons = NULL;
+   gp2xwiz_vt->set_icon = NULL;
    gp2xwiz_vt->set_window_title = NULL;
    gp2xwiz_vt->set_window_position = NULL;
    gp2xwiz_vt->get_window_position = gp2xwiz_get_window_position_ogl;
-   gp2xwiz_vt->set_window_constraints = NULL;
-   gp2xwiz_vt->get_window_constraints = NULL;
    gp2xwiz_vt->set_display_flag = NULL;
    gp2xwiz_vt->wait_for_vsync = gp2xwiz_wait_for_vsync_ogl;
 
