@@ -16,24 +16,24 @@
  */
 
 
-#include "allegro.h"
+#include "allegro5/allegro.h"
 
 
 /* list the available drivers */
-_DRIVER_INFO _linux_gfx_driver_list[] =
+_AL_DRIVER_INFO _linux_gfx_driver_list[] =
 {
 #if (defined ALLEGRO_LINUX_FBCON) && (!defined ALLEGRO_WITH_MODULES)
-   {  GFX_FBCON,    &gfx_fbcon,    TRUE  },
+   {  GFX_FBCON,    &gfx_fbcon,    true  },
 #endif
 #ifdef ALLEGRO_LINUX_VBEAF
-   {  GFX_VBEAF,    &gfx_vbeaf,    TRUE  },
+   {  GFX_VBEAF,    &gfx_vbeaf,    true  },
 #endif
 #if (defined ALLEGRO_LINUX_VGA) && (!defined ALLEGRO_WITH_MODULES)
-   {  GFX_VGA,      &gfx_vga,      TRUE  },
-/* {  GFX_MODEX,    &gfx_modex,    TRUE  }, */
+   {  GFX_VGA,      &gfx_vga,      true  },
+   {  GFX_MODEX,    &gfx_modex,    true  },
 #endif
 #if (defined ALLEGRO_LINUX_SVGALIB) && (!defined ALLEGRO_WITH_MODULES)
-   {  GFX_SVGALIB,  &gfx_svgalib,  FALSE },
+   {  GFX_SVGALIB,  &gfx_svgalib,  false },
 #endif
    {  0,            NULL,          0     }
 };
