@@ -16,24 +16,25 @@
  */
 
 
-#include "allegro.h"
-#include "allegro/internal/aintern.h"
-#include "allegro/platform/aintunix.h"
+#include "allegro5/allegro.h"
+#include "allegro5/internal/aintern.h"
+#include "allegro5/platform/aintunix.h"
 
 /* list the available drivers */
-_DRIVER_INFO _linux_mouse_driver_list[] =
+_AL_DRIVER_INFO _al_linux_mouse_driver_list[] =
 {
-   {  MOUSEDRV_LINUX_IPS2,     &mousedrv_linux_ips2,     TRUE  },
-   {  MOUSEDRV_LINUX_PS2,      &mousedrv_linux_ps2,      TRUE  },
-   {  MOUSEDRV_LINUX_MS,       &mousedrv_linux_ms,       TRUE  },
-   {  MOUSEDRV_LINUX_IMS,      &mousedrv_linux_ims,      TRUE  },
-   {  MOUSEDRV_LINUX_GPMDATA,  &mousedrv_linux_gpmdata,  TRUE  },
+   /* These drivers have not been updated for the new mouse API.
+    * They may be updated as required, although the evdev driver
+    * should be fine on modern kernels --pw
+    */
+/* {  MOUSEDRV_LINUX_GPMDATA,  &mousedrv_linux_gpmdata,  true  },*/
+/* {  MOUSEDRV_LINUX_MS,       &mousedrv_linux_ms,       true  },*/
+/* {  MOUSEDRV_LINUX_IMS,      &mousedrv_linux_ims,      true  },*/
+/* {  MOUSEDRV_LINUX_PS2,      &mousedrv_linux_ps2,      true  },*/
+/* {  MOUSEDRV_LINUX_IPS2,     &mousedrv_linux_ips2,     true  },*/
 #ifdef ALLEGRO_HAVE_LINUX_INPUT_H
-   {  MOUSEDRV_LINUX_EVDEV,    &mousedrv_linux_evdev,    TRUE  },
+   {  AL_MOUSEDRV_LINUX_EVDEV, &_al_mousedrv_linux_evdev, true  },
 #endif
-#ifdef ALLEGRO_LINUX_TSLIB
-   {  MOUSEDRV_LINUX_TSLIB,    &mousedrv_linux_tslib,    TRUE  },
-#endif
-   {  MOUSEDRV_NONE,           &mousedrv_none,           TRUE  },
    {  0,                       NULL,                     0     }
 };
+
