@@ -1,5 +1,5 @@
 /*         ______   ___    ___ 
- *        /\  _  \ /\_ \  /\_ \
+ *        /\  _  \ /\_ \  /\_ \ 
  *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
  *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
  *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
@@ -11,34 +11,35 @@
  *      OpenGL extensions management subsystem
  *
  *      By Elias Pschernig and Milan Mimica.
- *
  *      Based on AllegroGL extensions management.
  */
 
-#include <ctype.h>
-#include <math.h>
-#include <stdio.h>
-#include <string.h>
+/* Title: OpenGL extensions
+ */
+
 
 #include "allegro5/allegro.h"
 #include "allegro5/allegro_opengl.h"
+#include "allegro5/display.h"
 #include "allegro5/opengl/gl_ext.h"
 #include "allegro5/internal/aintern.h"
 #include "allegro5/internal/aintern_opengl.h"
 #include "allegro5/internal/aintern_display.h"
 #include "allegro5/internal/aintern_system.h"
+#include <stdio.h>
+#include <math.h>
 
 /* We need some driver specific details not worth of a vtable entry. */
 #if defined ALLEGRO_WINDOWS
    #include "../win/wgl.h"
 #elif defined ALLEGRO_UNIX && !defined ALLEGRO_EXCLUDE_GLX
-   #include "allegro5/internal/aintern_xdisplay.h"
-   #include "allegro5/internal/aintern_xsystem.h"
+   #include "allegro5/internal/aintern_xglx.h"
 #endif
 
-#if defined __APPLE__ && !defined ALLEGRO_IPHONE
+#include <string.h>
+#if defined ALLEGRO_MACOSX
 #include <OpenGL/glu.h>
-#elif !defined ALLEGRO_CFG_OPENGLES
+#elif !defined ALLEGRO_GP2XWIZ && !defined ALLEGRO_IPHONE
 #include <GL/glu.h>
 #endif
 
@@ -59,7 +60,7 @@ ALLEGRO_DEBUG_CHANNEL("opengl")
     */
    #if defined ALLEGRO_GLXGETPROCADDRESSARB
       #define alXGetProcAddress glXGetProcAddressARB
-   #elif defined ALLEGRO_RASPBERRYPI
+   #elif defined ALLEGRO_GP2XWIZ
       #define alXGetProcAddress eglGetProcAddress
    #else
       #define alXGetProcAddress glXGetProcAddress
@@ -92,6 +93,7 @@ ALLEGRO_DEBUG_CHANNEL("opengl")
 
 
 
+#ifndef ALLEGRO_IPHONE
 static uint32_t parse_opengl_version(const char *s)
 {
    char *p = (char *) s;
@@ -99,15 +101,6 @@ static uint32_t parse_opengl_version(const char *s)
    int n;
    uint32_t ver;
 
-   /* For OpenGL ES version strings have the form:
-    * "OpenGL ES-<profile> <major>.<minor>"
-    * So for example: "OpenGL ES-CM 2.0". We simply skip any non-digit
-    * characters and then parse it like for normal OpenGL.
-    */
-   while (*p && !isdigit(*p)) {
-      p++;
-   }
-
    /* e.g. "4.0.0 Vendor blah blah" */
    for (n = 0; n < 4; n++) {
       char *end;
@@ -127,33 +120,34 @@ static uint32_t parse_opengl_version(const char *s)
    ALLEGRO_DEBUG("Parsed '%s' as 0x%08x\n", s, ver);
    return ver;
 }
+#endif
 
 
 
 /* Reads version info out of glGetString(GL_VERSION) */
 static uint32_t _al_ogl_version(void)
 {
-   char const *value = al_get_config_value(al_get_system_config(), "opengl",
-      "force_opengl_version");
-   if (value) {
-      uint32_t v = parse_opengl_version(value);
-      ALLEGRO_INFO("OpenGL version forced to %d.%d.%d.%d.\n",
-         (v >> 24) & 0xff,
-         (v >> 16) & 0xff,
-         (v >> 8) & 0xff,
-         (v & 0xff));
-      return v;
-   }
-
+#ifndef ALLEGRO_IPHONE
+   ALLEGRO_CONFIG *cfg;
    const char *str;
 
+   cfg = al_get_system_config();
+   if (cfg) {
+      char const *value = al_get_config_value(cfg,
+	 "opengl", "force_opengl_version");
+      if (value) {
+         uint32_t v = parse_opengl_version(value);
+         ALLEGRO_INFO("OpenGL version forced to %d.%d.%d.%d.\n",
+            (v >> 24) & 0xff,
+            (v >> 16) & 0xff,
+            (v >> 8) & 0xff,
+            (v & 0xff));
+         return v;
+      }
+   }
+
    str = (const char *)glGetString(GL_VERSION);
    if (str) {
-      #ifdef ALLEGRO_CFG_OPENGLES
-      char *str2 = strstr(str, "ES ");
-      if (str2)
-         str = str2 + 3;
-      #endif
       return parse_opengl_version(str);
    }
    else {
@@ -162,7 +156,24 @@ static uint32_t _al_ogl_version(void)
        */
       return _ALLEGRO_OPENGL_VERSION_1_0;
    }
+#else
+   /* XXX this is asking for trouble, and should be documented */
+   const char *s = (char *)glGetString(GL_VERSION);
+   if (strstr(s, "2.0"))
+      return _ALLEGRO_OPENGL_VERSION_2_0;
+   else if (strstr(s, "1.1"))
+      return _ALLEGRO_OPENGL_VERSION_1_1;
+   else
+      return _ALLEGRO_OPENGL_VERSION_0;
+#endif
+}
 
+
+
+static bool _al_ogl_version_3_only(int flags)
+{
+   const int mask = ALLEGRO_OPENGL_3_0 | ALLEGRO_OPENGL_FORWARD_COMPATIBLE;
+   return (flags & mask) == mask;
 }
 
 
@@ -194,7 +205,6 @@ static void print_extensions(char const *extension)
 
 
 
-#if !defined ALLEGRO_CFG_OPENGLES
 /* Print all extensions the OpenGL 3.0 way. */ 
 static void print_extensions_3_0(void)
 {
@@ -207,7 +217,6 @@ static void print_extensions_3_0(void)
       ALLEGRO_DEBUG("%s\n", name);
    }
 }
-#endif
 
 
 
@@ -233,7 +242,7 @@ uint32_t al_get_opengl_version(void)
  */
 int al_get_opengl_variant(void)
 {
-#if defined ALLEGRO_CFG_OPENGLES
+#if defined ALLEGRO_IPHONE || defined ALLEGRO_GP2XWIZ
    return ALLEGRO_OPENGL_ES;
 #else
    return ALLEGRO_DESKTOP_OPENGL;
@@ -268,36 +277,6 @@ static ALLEGRO_OGL_EXT_API *create_extension_api_table(void)
 
 
 
-typedef void (*VOID_FPTR)(void);
-/* GCC 4.8.2 and possibly others are really slow at optimizing the 100's of the
- * if statements in the load_extensions function below, so we extract them to
- * this function.
- */
-static VOID_FPTR load_extension(const char* name)
-{
-   VOID_FPTR fptr = NULL;
-#ifdef ALLEGRO_WINDOWS
-   fptr = (VOID_FPTR)wglGetProcAddress(name);
-#elif defined ALLEGRO_UNIX
-   fptr = (VOID_FPTR)alXGetProcAddress((const GLubyte*)name);
-#elif defined ALLEGRO_MACOSX
-   CFStringRef cfstr = CFStringCreateWithCStringNoCopy(NULL, name,
-      kCFStringEncodingUTF8, kCFAllocatorNull);
-   if (cfstr) {
-      fptr = (VOID_FPTR)CFBundleGetFunctionPointerForName(opengl_bundle_ref, cfstr);
-      CFRelease(cfstr);
-   }
-#elif defined ALLEGRO_SDL
-   fptr = SDL_GL_GetProcAddress(name);
-#endif
-   if (fptr) {
-      ALLEGRO_DEBUG("%s successfully loaded (%p)\n", name, fptr);
-   }
-   return fptr;
-}
-
-
-
 /* Load the extension API addresses into the table.
  * Should only be done on context creation.
  */
@@ -317,15 +296,17 @@ static void load_extensions(ALLEGRO_OGL_EXT_API *ext)
 
 #ifdef ALLEGRO_WINDOWS
 
-   #define AGL_API(type, name, args)                                           \
-      ext->name = (_ALLEGRO_gl##name##_t)load_extension("gl" #name);
+   #define AGL_API(type, name, args)                                 \
+      ext->name = (_ALLEGRO_gl##name##_t)wglGetProcAddress("gl" #name); \
+      if (ext->name) { ALLEGRO_DEBUG("gl" #name " successfully loaded\n"); }
 
       #include "allegro5/opengl/GLext/gl_ext_api.h"
 
    #undef AGL_API
 
-   #define AGL_API(type, name, args)                                           \
-      ext->name = (_ALLEGRO_wgl##name##_t)load_extension("wgl" #name);
+   #define AGL_API(type, name, args)                                  \
+      ext->name = (_ALLEGRO_wgl##name##_t)wglGetProcAddress("wgl" #name); \
+      if (ext->name) { ALLEGRO_DEBUG("wgl" #name " successfully loaded\n"); }
 
       #include "allegro5/opengl/GLext/wgl_ext_api.h"
 
@@ -333,15 +314,17 @@ static void load_extensions(ALLEGRO_OGL_EXT_API *ext)
 
 #elif defined ALLEGRO_UNIX
 
-   #define AGL_API(type, name, args)                                           \
-      ext->name = (_ALLEGRO_gl##name##_t)load_extension("gl" #name);
+   #define AGL_API(type, name, args)                                               \
+      ext->name = (_ALLEGRO_gl##name##_t)alXGetProcAddress((const GLubyte*)"gl" #name);  \
+      if (ext->name) { ALLEGRO_DEBUG("gl" #name " successfully loaded (%p)\n", ext->name); }
 
       #include "allegro5/opengl/GLext/gl_ext_api.h"
 
    #undef AGL_API
 
-   #define AGL_API(type, name, args)                                           \
-      ext->name = (_ALLEGRO_glX##name##_t)load_extension("glX" #name);
+   #define AGL_API(type, name, args)                                               \
+      ext->name = (_ALLEGRO_glX##name##_t)alXGetProcAddress((const GLubyte*)"glX" #name); \
+      if (ext->name) { ALLEGRO_DEBUG("glX" #name " successfully loaded\n"); }
 
       #include "allegro5/opengl/GLext/glx_ext_api.h"
 
@@ -349,17 +332,9 @@ static void load_extensions(ALLEGRO_OGL_EXT_API *ext)
 
 #elif defined ALLEGRO_MACOSX
 
-#define AGL_API(type, name, args)                                              \
-      ext->name = (_ALLEGRO_gl##name##_t)load_extension("gl" # name);
-
-      #include "allegro5/opengl/GLext/gl_ext_api.h"
-
-   #undef AGL_API
-
-#elif defined ALLEGRO_SDL
-
-#define AGL_API(type, name, args)                                              \
-      ext->name = (_ALLEGRO_gl##name##_t)load_extension("gl" # name);
+#define AGL_API(type, name, args)                                                                 \
+      ext->name = (_ALLEGRO_gl##name##_t)CFBundleGetFunctionPointerForName(opengl_bundle_ref, CFSTR("gl" # name)); \
+      if (ext->name) { ALLEGRO_DEBUG("gl" #name " successfully loaded\n"); }
 
       #include "allegro5/opengl/GLext/gl_ext_api.h"
 
@@ -456,12 +431,14 @@ static bool _ogl_is_extension_supported(const char *extension,
 {
    int ret = 0;
    GLubyte const *ext_str;
-#if !defined ALLEGRO_CFG_OPENGLES
    int v = al_get_opengl_version();
-#endif
    (void)disp;
 
-#if !defined ALLEGRO_CFG_OPENGLES
+#if defined ALLEGRO_GP2XWIZ
+   return false;
+#endif
+
+#ifndef ALLEGRO_IPHONE
    if (disp->flags & ALLEGRO_OPENGL_3_0 || v >= _ALLEGRO_OPENGL_VERSION_3_0) {
       int i;
       GLint ext_cnt;
@@ -525,6 +502,7 @@ static bool _ogl_is_extension_supported(const char *extension,
 static bool _ogl_is_extension_with_version_supported(
    const char *extension, ALLEGRO_DISPLAY *disp, uint32_t ver)
 {
+   ALLEGRO_CONFIG *cfg;
    char const *value;
 
   /* For testing purposes, any OpenGL extension can be disable in
@@ -535,12 +513,15 @@ static bool _ogl_is_extension_with_version_supported(
     * GL_EXT_framebuffer_object=0
     * 
     */
-   value = al_get_config_value(al_get_system_config(),
-      "opengl_disabled_extensions", extension);
-   if (value) {
-      ALLEGRO_WARN("%s found in [opengl_disabled_extensions].\n",
-         extension);
-      return false;
+   cfg = al_get_system_config();
+   if (cfg) {
+      value = al_get_config_value(cfg,
+         "opengl_disabled_extensions", extension);
+      if (value) {
+         ALLEGRO_WARN("%s found in [opengl_disabled_extensions].\n",
+            extension);
+         return false;
+      }
    }
 
    /* If the extension is included in the OpenGL version, there is no
@@ -613,7 +594,7 @@ void *al_get_opengl_proc_address(const char *name)
        * address. Unfortunately glXGetProcAddress is an extension
        * and may not be available on all platforms
        */
-#if defined ALLEGRO_RASPBERRYPI
+#ifdef ALLEGRO_GP2XWIZ
       symbol = alXGetProcAddress(name);
 #else
       symbol = alXGetProcAddress((const GLubyte *)name);
@@ -680,9 +661,6 @@ static void fill_in_info_struct(const GLubyte *rendereru, OPENGL_INFO *info)
    else if (strstr(renderer, "Mesa DRI R200")) {
       info->is_ati_r200_chip = 1;
    }
-   else if (strstr(renderer, "Intel HD Graphics 3000 OpenGL Engine")) {
-      info->is_intel_hd_graphics_3000 = 1;
-   }
 
    if ((strncmp(renderer, "3Dfx/Voodoo3 ", 13) == 0)
        || (strncmp(renderer, "3Dfx/Voodoo2 ", 13) == 0)
@@ -716,14 +694,11 @@ void _al_ogl_manage_extensions(ALLEGRO_DISPLAY *gl_disp)
    ALLEGRO_OGL_EXT_API *ext_api;
    ALLEGRO_OGL_EXT_LIST *ext_list;
 
-   /* Some functions depend on knowing the version of opengl in use */
-   fill_in_info_struct(glGetString(GL_RENDERER), &(gl_disp->ogl_extras->ogl_info));
-
    /* Print out OpenGL extensions
     * We should use glGetStringi(GL_EXTENSIONS, i) for OpenGL 3.0+
     * but it doesn't seem to work until later.
     */
-   if (gl_disp->ogl_extras->ogl_info.version < _ALLEGRO_OPENGL_VERSION_3_0) {
+   if (!_al_ogl_version_3_only(gl_disp->flags)) {
       ALLEGRO_DEBUG("OpenGL Extensions:\n");
       print_extensions((char const *)glGetString(GL_EXTENSIONS));
    }
@@ -782,20 +757,20 @@ void _al_ogl_manage_extensions(ALLEGRO_DISPLAY *gl_disp)
    print_extensions(ext);
 #endif
 
+   fill_in_info_struct(glGetString(GL_RENDERER), &(gl_disp->ogl_extras->ogl_info));
+
    /* Create & load extension API table */
    ext_api = create_extension_api_table();
    load_extensions(ext_api);
    gl_disp->ogl_extras->extension_api = ext_api;
    
-#if !defined ALLEGRO_CFG_OPENGLES
    /* Need that symbol already so can't wait until it is assigned later. */
    glGetStringi = ext_api->GetStringi;
 
-   if (gl_disp->ogl_extras->ogl_info.version >= _ALLEGRO_OPENGL_VERSION_3_0) {
+   if (_al_ogl_version_3_only(gl_disp->flags)) {
       ALLEGRO_DEBUG("OpenGL Extensions:\n");
       print_extensions_3_0();
    }
-#endif
 
    /* Create the list of supported extensions. */
    ext_list = create_extension_list();
@@ -828,6 +803,17 @@ void _al_ogl_manage_extensions(ALLEGRO_DISPLAY *gl_disp)
 #undef AGL_EXT
 #endif
 
+    /* TODO: use these somewhere */
+#if 0
+   for (i = 0; i < 5; i++) {
+      __allegro_gl_texture_read_format[i] = -1;
+      __allegro_gl_texture_components[i] = GL_RGB;
+   }
+   __allegro_gl_texture_read_format[3] = GL_UNSIGNED_BYTE;
+   __allegro_gl_texture_read_format[4] = GL_UNSIGNED_BYTE;
+   __allegro_gl_texture_components[4] = GL_RGBA;
+#endif /* #if 0 */
+
    /* Get max texture size */
    glGetIntegerv(GL_MAX_TEXTURE_SIZE,
                  (GLint *) & gl_disp->ogl_extras->ogl_info.max_texture_size);
@@ -866,7 +852,7 @@ void _al_ogl_manage_extensions(ALLEGRO_DISPLAY *gl_disp)
          }
       }
       else if (strstr(vendor, "ATI Technologies")) {
-         if (gl_disp->ogl_extras->ogl_info.version >= _ALLEGRO_OPENGL_VERSION_3_0) {
+         if (_al_ogl_version_3_only(gl_disp->flags)) {
             /* Assume okay. */
          }
          else if (!strstr((const char *)glGetString(GL_EXTENSIONS),
@@ -885,18 +871,12 @@ void _al_ogl_manage_extensions(ALLEGRO_DISPLAY *gl_disp)
          s[ALLEGRO_SUPPORT_SEPARATE_ALPHA] = 1;
 
       s[ALLEGRO_SUPPORT_NPOT_BITMAP] =
-         ext_list->ALLEGRO_GL_ARB_texture_non_power_of_two ||
-         ext_list->ALLEGRO_GL_OES_texture_npot;
+         ext_list->ALLEGRO_GL_ARB_texture_non_power_of_two;
    ALLEGRO_INFO("Use of non-power-of-two textures %s.\n",
       s[ALLEGRO_SUPPORT_NPOT_BITMAP] ? "enabled" : "disabled");
-#if defined ALLEGRO_CFG_OPENGLES
-   if (gl_disp->flags & ALLEGRO_PROGRAMMABLE_PIPELINE) {
-      s[ALLEGRO_CAN_DRAW_INTO_BITMAP] = true;
-   }
-   else {
-      s[ALLEGRO_CAN_DRAW_INTO_BITMAP] =
-         ext_list->ALLEGRO_GL_OES_framebuffer_object;
-   }
+#ifdef ALLEGRO_IPHONE
+   s[ALLEGRO_CAN_DRAW_INTO_BITMAP] =
+      ext_list->ALLEGRO_GL_OES_framebuffer_object;
    ALLEGRO_INFO("Use of FBO to draw to textures %s.\n",
       s[ALLEGRO_CAN_DRAW_INTO_BITMAP] ? "enabled" :
       "disabled");
