@@ -20,26 +20,19 @@
 
 #include "allegro5/allegro.h"
 #include "allegro5/internal/aintern.h"
-#ifdef ALLEGRO_CFG_SHADER_GLSL
-#include "allegro5/allegro_opengl.h"
-#include "allegro5/internal/aintern_opengl.h"
-#endif
 #include ALLEGRO_INTERNAL_HEADER
 #include "allegro5/internal/aintern_bitmap.h"
-#include "allegro5/internal/aintern_debug.h"
 #include "allegro5/internal/aintern_dtor.h"
-#include "allegro5/internal/aintern_exitfunc.h"
-#include "allegro5/internal/aintern_pixels.h"
 #include "allegro5/internal/aintern_system.h"
+#include "allegro5/internal/aintern_vector.h"
+#include "allegro5/internal/aintern_pixels.h"
 #include "allegro5/internal/aintern_thread.h"
 #include "allegro5/internal/aintern_timer.h"
 #include "allegro5/internal/aintern_tls.h"
-#include "allegro5/internal/aintern_vector.h"
 
 ALLEGRO_DEBUG_CHANNEL("system")
 
 static ALLEGRO_SYSTEM *active_sysdrv = NULL;
-static ALLEGRO_CONFIG *sys_config = NULL;
 
 _AL_VECTOR _al_system_interfaces;
 static _AL_VECTOR _user_system_interfaces = _AL_VECTOR_INITIALIZER(ALLEGRO_SYSTEM_INTERFACE *);
@@ -51,6 +44,26 @@ static bool atexit_virgin = true;
 static char _al_app_name[256] = "";
 static char _al_org_name[256] = "";
 
+#if 0
+bool al_register_system_driver(ALLEGRO_SYSTEM_INTERFACE *sys_interface)
+{
+   ALLEGRO_SYSTEM_INTERFACE **add = _al_vector_alloc_back(&_user_system_interfaces);
+   if (!add)
+      return false;
+   *add = sys_interface;
+   return true;
+}
+#endif
+
+
+
+#if 0
+void al_unregister_system_driver(ALLEGRO_SYSTEM_INTERFACE *sys_interface)
+{
+   _al_vector_find_and_delete(&_user_system_interfaces, &sys_interface);
+}
+#endif
+
 
 
 static ALLEGRO_SYSTEM *find_system(_AL_VECTOR *vector)
@@ -75,19 +88,20 @@ static ALLEGRO_SYSTEM *find_system(_AL_VECTOR *vector)
 static void shutdown_system_driver(void)
 {
    if (active_sysdrv) {
+      ALLEGRO_CONFIG *temp = active_sysdrv->config;
       if (active_sysdrv->user_exe_path)
          al_destroy_path(active_sysdrv->user_exe_path);
       if (active_sysdrv->vt && active_sysdrv->vt->shutdown_system)
          active_sysdrv->vt->shutdown_system();
       active_sysdrv = NULL;
+      /* active_sysdrv is not accessible here so we copied it */
+      al_destroy_config(temp);
  
       while (!_al_vector_is_empty(&_al_system_interfaces))
          _al_vector_delete_at(&_al_system_interfaces, _al_vector_size(&_al_system_interfaces)-1);
       _al_vector_free(&_al_system_interfaces);
       _al_vector_init(&_al_system_interfaces, sizeof(ALLEGRO_SYSTEM_INTERFACE *));
    }
-   al_destroy_config(sys_config);
-   sys_config = NULL;
 }
 
 
@@ -102,17 +116,10 @@ static ALLEGRO_PATH *early_get_exename_path(void)
    return _al_win_get_path(ALLEGRO_EXENAME_PATH);
 #elif defined(ALLEGRO_MACOSX)
    return _al_osx_get_path(ALLEGRO_EXENAME_PATH);
-#elif defined(ALLEGRO_IPHONE)
-   return _al_iphone_get_path(ALLEGRO_EXENAME_PATH);
 #elif defined(ALLEGRO_UNIX)
    return _al_unix_get_path(ALLEGRO_EXENAME_PATH);
-#elif defined(ALLEGRO_ANDROID)
-   return _al_android_get_path(ALLEGRO_EXENAME_PATH);
-#elif defined(ALLEGRO_SDL)
-   char* p = SDL_GetBasePath();
-   ALLEGRO_PATH *path = al_create_path_for_directory(p);
-   SDL_free(p);
-   return path;
+#elif defined(ALLEGRO_IPHONE)
+   return _al_iphone_get_path(ALLEGRO_EXENAME_PATH);
 #else
    #error early_get_exename_path not implemented
 #endif
@@ -122,34 +129,33 @@ static ALLEGRO_PATH *early_get_exename_path(void)
 
 static void read_allegro_cfg(void)
 {
+   /* We cannot use any logging in this function as it would cause the
+    * logging system to be initialised before all the relevant config files
+    * have been read in.
+    */
+
    /* We assume that the stdio file interface is in effect. */
 
    ALLEGRO_PATH *path;
    ALLEGRO_CONFIG *temp;
 
-   if (!sys_config)
-      sys_config = al_create_config();
+   active_sysdrv->config = NULL;
 
-#if defined(ALLEGRO_UNIX) && !defined(ALLEGRO_IPHONE)
-   temp = al_load_config_file("/etc/allegro5rc");
-   if (temp) {
-      al_merge_config_into(sys_config, temp);
-      al_destroy_config(temp);
-   }
+#if defined(ALLEGRO_UNIX) && !defined(ALLEGRO_GP2XWIZ) && !defined(ALLEGRO_IPHONE)
+   active_sysdrv->config = al_load_config_file("/etc/allegro5rc");
 
    path = _al_unix_get_path(ALLEGRO_USER_HOME_PATH);
    if (path) {
       al_set_path_filename(path, "allegro5rc");
       temp = al_load_config_file(al_path_cstr(path, '/'));
       if (temp) {
-         al_merge_config_into(sys_config, temp);
-         al_destroy_config(temp);
-      }
-      al_set_path_filename(path, ".allegro5rc");
-      temp = al_load_config_file(al_path_cstr(path, '/'));
-      if (temp) {
-         al_merge_config_into(sys_config, temp);
-         al_destroy_config(temp);
+         if (active_sysdrv->config) {
+            al_merge_config_into(active_sysdrv->config, temp);
+            al_destroy_config(temp);
+         }
+         else {
+            active_sysdrv->config = temp;
+         }
       }
       al_destroy_path(path);
    }
@@ -160,51 +166,45 @@ static void read_allegro_cfg(void)
       al_set_path_filename(path, "allegro5.cfg");
       temp = al_load_config_file(al_path_cstr(path, ALLEGRO_NATIVE_PATH_SEP));
       if (temp) {
-         al_merge_config_into(sys_config, temp);
-         al_destroy_config(temp);
+         if (active_sysdrv->config) {
+            al_merge_config_into(active_sysdrv->config, temp);
+            al_destroy_config(temp);
+         }
+         else {
+            active_sysdrv->config = temp;
+         }
       }
       al_destroy_path(path);
    }
-   /* Reconfigure logging in case something changed. */
-   _al_configure_logging();
+
+   /* Always have a configuration available whether or not a config file
+    * exists.
+    */
+   if (!active_sysdrv->config)
+      active_sysdrv->config = al_create_config();
 }
 
 
 
 /*
- * Can a binary with version a use a library with version b?
- *
  * Let a = xa.ya.za.*
  * Let b = xb.yb.zb.*
  *
- * When a has the unstable bit, a is compatible with b if xa.ya.za = xb.yb.zb.
- * Otherwise, a is compatible with b if xa.ya = xb.yb and zb >= za.
+ * When ya is odd, a is compatible with b if xa.ya.za = xb.yb.zb.
+ * When ya is even, a is compatible with b if xa.ya = xb.yb.
  *
  * Otherwise a and b are incompatible.
  */
 static bool compatible_versions(int a, int b)
 {
-   int a_unstable = a & (1 << 31);
-
-   int a_major = (a & 0x7f000000) >> 24;
-   int a_sub   = (a & 0x00ff0000) >> 16;
-   int a_wip   = (a & 0x0000ff00) >> 8;
-
-   int b_major = (b & 0x7f000000) >> 24;
-   int b_sub   = (b & 0x00ff0000) >> 16;
-   int b_wip   = (b & 0x0000ff00) >> 8;
-
-   if (a_major != b_major) {
+   if ((a & 0xffff0000) != (b & 0xffff0000)) {
       return false;
    }
-   if (a_unstable && a_wip != b_wip) {
-      return false;
-   }
-   if (a_sub != b_sub) {
-      return false;
-   }
-   if (a_wip > b_wip) {
-      return false;
+   if (((a & 0x00ff0000) >> 16) & 1) {
+      
+      if ((a & 0x0000ff00) != (b & 0x0000ff00)) {
+         return false;
+      }
    }
    return true;
 }
@@ -218,7 +218,7 @@ bool al_install_system(int version, int (*atexit_ptr)(void (*)(void)))
    ALLEGRO_SYSTEM bootstrap;
    ALLEGRO_SYSTEM *real_system;
    int library_version = al_get_allegro_version();
-
+   
    if (active_sysdrv) {
       return true;
    }
@@ -230,12 +230,17 @@ bool al_install_system(int version, int (*atexit_ptr)(void (*)(void)))
    if (!compatible_versions(version, library_version))
       return false;
 
-   _al_tls_init_once();
-   _al_reinitialize_tls_values();
-
+#ifdef ALLEGRO_CFG_PTHREADS_TLS
+   _al_pthreads_tls_init();
+#endif
+   
    _al_vector_init(&_al_system_interfaces, sizeof(ALLEGRO_SYSTEM_INTERFACE *));
 
-   /* Set up a bootstrap system so the calls expecting it don't freak out */
+   /* We want active_sysdrv->config to be available as soon as
+    * possible - for example what if a system driver need to read
+    * settings out of there. Hence we use a dummy ALLEGRO_SYSTEM
+    * here to load the initial config.
+    */
    memset(&bootstrap, 0, sizeof(bootstrap));
    active_sysdrv = &bootstrap;
    read_allegro_cfg();
@@ -262,7 +267,7 @@ bool al_install_system(int version, int (*atexit_ptr)(void (*)(void)))
    }
    
    active_sysdrv = real_system;
-   active_sysdrv->mouse_wheel_precision = 1;
+   active_sysdrv->config = bootstrap.config;
 
    ALLEGRO_INFO("Allegro version: %s\n", ALLEGRO_VERSION_STR);
 
@@ -279,22 +284,11 @@ bool al_install_system(int version, int (*atexit_ptr)(void (*)(void)))
    _al_init_pixels();
 
    _al_init_iio_table();
-   
-   _al_init_convert_bitmap_list();
 
    _al_init_timers();
 
-#ifdef ALLEGRO_CFG_SHADER_GLSL
-   _al_glsl_init_shaders();
-#endif
-
-   if (active_sysdrv->vt->heartbeat_init)
-      active_sysdrv->vt->heartbeat_init();
-
    if (atexit_ptr && atexit_virgin) {
-#ifndef ALLEGRO_ANDROID
       atexit_ptr(al_uninstall_system);
-#endif
       atexit_virgin = false;
    }
 
@@ -303,31 +297,20 @@ bool al_install_system(int version, int (*atexit_ptr)(void (*)(void)))
 
    active_sysdrv->installed = true;
 
-   _al_srand(time(NULL));
-
    return true;
 }
 
+
+
 /* Function: al_uninstall_system
  */
 void al_uninstall_system(void)
 {
-   /* Note: al_uninstall_system may get called multiple times without an
-    * al_install_system in between. For example if the user manually
-    * calls it at the end of the program it is called right again
-    * because it's installed as an atexit function by al_init.
-    */
-
    _al_run_destructors(_al_dtor_list);
    _al_run_exit_funcs();
    _al_shutdown_destructors(_al_dtor_list);
    _al_dtor_list = NULL;
 
-#ifdef ALLEGRO_CFG_SHADER_GLSL
-   _al_glsl_shutdown_shaders();
-#endif
-
-   _al_shutdown_logging();
 
    /* shutdown_system_driver is registered as an exit func so we don't need
     * to do any more here.
@@ -355,21 +338,12 @@ ALLEGRO_SYSTEM *al_get_system_driver(void)
    return active_sysdrv;
 }
 
-/* Function: al_get_system_id
- */
-ALLEGRO_SYSTEM_ID al_get_system_id(void)
-{
-   ASSERT(active_sysdrv);
-   return active_sysdrv->vt->id;
-}
 
 /* Function: al_get_system_config
  */
 ALLEGRO_CONFIG *al_get_system_config(void)
 {
-   if (!sys_config)
-      sys_config = al_create_config();
-   return sys_config;
+   return (active_sysdrv) ? active_sysdrv->config : NULL;
 }
 
 
