@@ -42,13 +42,8 @@
 #include "allegro5/internal/aintern_display.h"
 #include "allegro5/internal/aintern_file.h"
 #include "allegro5/internal/aintern_fshook.h"
-#include "allegro5/internal/aintern_shader.h"
 #include "allegro5/internal/aintern_tls.h"
 
-#ifdef ALLEGRO_ANDROID
-#include "allegro5/internal/aintern_android.h"
-#endif
-
 #if defined(ALLEGRO_MINGW32) && !defined(ALLEGRO_CFG_DLL_TLS)
    /*
     * MinGW < 4.2.1 doesn't have builtin thread local storage, so we
@@ -94,15 +89,10 @@ typedef struct thread_local_state {
    /* Error code */
    int allegro_errno;
 
-#ifdef ALLEGRO_ANDROID
-   JNIEnv *jnienv;
-#endif
-
    /* Destructor ownership count */
    int dtor_owner_count;
 } thread_local_state;
 
-
 typedef struct INTERNAL_STATE {
    thread_local_state tls;
    ALLEGRO_BLENDER stored_blender;
@@ -110,7 +100,8 @@ typedef struct INTERNAL_STATE {
    int flags;
 } INTERNAL_STATE;
 
-ALLEGRO_STATIC_ASSERT(tls, sizeof(ALLEGRO_STATE) > sizeof(INTERNAL_STATE));
+ALLEGRO_STATIC_ASSERT(tls,
+   sizeof(ALLEGRO_STATE) > sizeof(INTERNAL_STATE));
 
 
 static void initialize_blender(ALLEGRO_BLENDER *b)
@@ -133,7 +124,7 @@ static void initialize_tls_values(thread_local_state *tls)
    tls->new_window_y = INT_MAX;
 
    initialize_blender(&tls->current_blender);
-   tls->new_bitmap_flags = ALLEGRO_CONVERT_BITMAP;
+   tls->new_bitmap_flags = ALLEGRO_VIDEO_BITMAP;
    tls->new_bitmap_format = ALLEGRO_PIXEL_FORMAT_ANY_WITH_ALPHA;
    tls->new_file_interface = &_al_file_interface_stdio;
    tls->fs_interface = &_al_fs_interface_stdio;
@@ -141,7 +132,6 @@ static void initialize_tls_values(thread_local_state *tls)
    _al_fill_display_settings(&tls->new_display_settings);
 }
 
-
 // FIXME: The TLS implementation below only works for dynamic linking
 // right now - instead of using DllMain we should simply initialize
 // on first request.
@@ -150,12 +140,147 @@ static void initialize_tls_values(thread_local_state *tls)
 #endif
 
 #if defined(ALLEGRO_CFG_DLL_TLS)
-   #include "tls_dll.inc"
-#elif defined(ALLEGRO_MACOSX) || defined(ALLEGRO_IPHONE) || defined(ALLEGRO_ANDROID) || defined(ALLEGRO_RASPBERRYPI)
-   #include "tls_pthread.inc"
-#else
-   #include "tls_native.inc"
-#endif
+
+#include <windows.h>
+
+
+/* Forward declaration to bypass strict warning. */
+BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved);
+
+
+static DWORD tls_index;
+
+
+static thread_local_state *tls_get(void)
+{
+   thread_local_state *t = TlsGetValue(tls_index);
+   return t;
+}
+
+
+BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
+{ 
+   thread_local_state *data;
+
+   (void)hinstDLL;
+   (void)fdwReason;
+   (void)lpvReserved;
+ 
+   switch (fdwReason) { 
+      case DLL_PROCESS_ATTACH: 
+         if ((tls_index = TlsAlloc()) == TLS_OUT_OF_INDEXES) {
+            return false;
+	 }
+         // No break: Initialize the index for first thread.
+         // The attached process creates a new thread. 
+
+      case DLL_THREAD_ATTACH: 
+          // Initialize the TLS index for this thread.
+          data = al_malloc(sizeof(*data));
+          if (data != NULL) {
+             TlsSetValue(tls_index, data);
+             initialize_tls_values(data);
+          }
+          break; 
+ 
+        // The thread of the attached process terminates.
+      case DLL_THREAD_DETACH: 
+         // Release the allocated memory for this thread.
+         data = TlsGetValue(tls_index); 
+         if (data != NULL) 
+            al_free(data);
+ 
+         break; 
+ 
+      // DLL unload due to process termination or FreeLibrary. 
+      case DLL_PROCESS_DETACH: 
+         // Release the allocated memory for this thread.
+         data = TlsGetValue(tls_index); 
+         if (data != NULL) 
+            al_free(data);
+         // Release the TLS index.
+         TlsFree(tls_index); 
+         break; 
+ 
+      default: 
+         break; 
+   } 
+ 
+   return true; 
+}
+
+
+#else /* not ALLEGRO_CFG_DLL_TLS */
+
+#if defined(ALLEGRO_MSVC) || defined(ALLEGRO_BCC32)
+
+#define THREAD_LOCAL __declspec(thread)
+#define HAVE_NATIVE_TLS
+
+#elif defined ALLEGRO_MACOSX || defined ALLEGRO_GP2XWIZ || defined ALLEGRO_IPHONE
+
+#define THREAD_LOCAL
+
+static pthread_key_t tls_key = 0;
+
+static void pthreads_thread_destroy(void* ptr)
+{
+   al_free(ptr);
+}
+
+
+void _al_pthreads_tls_init(void)
+{
+   pthread_key_create(&tls_key, pthreads_thread_destroy);
+}
+
+static thread_local_state _tls;
+
+static thread_local_state* pthreads_thread_init(void)
+{
+   /* Allocate and copy the 'template' object */
+   thread_local_state* ptr = (thread_local_state*)al_malloc(sizeof(thread_local_state));
+   memcpy(ptr, &_tls, sizeof(thread_local_state));
+   pthread_setspecific(tls_key, ptr);
+   return ptr;
+}
+
+/* This function is short so it can hopefully be inlined. */
+static thread_local_state* tls_get(void)
+{
+   thread_local_state* ptr = (thread_local_state*)pthread_getspecific(tls_key);
+   if (ptr == NULL)
+   {
+      /* Must create object */
+      ptr = pthreads_thread_init();
+      initialize_tls_values(ptr);
+   }
+   return ptr;
+}
+
+#else /* not MSVC/BCC32, not OSX */
+
+#define THREAD_LOCAL __thread
+#define HAVE_NATIVE_TLS
+
+#endif /* end not MSVC/BCC32, not OSX */
+
+static THREAD_LOCAL thread_local_state _tls;
+
+#ifdef HAVE_NATIVE_TLS
+static thread_local_state *tls_get(void)
+{
+   static THREAD_LOCAL thread_local_state *ptr = NULL;
+   if (!ptr) {
+      ptr = &_tls;
+      initialize_tls_values(ptr);
+   }
+   return ptr;
+}
+#endif /* end HAVE_NATIVE_TLS */
+
+
+#endif /* end not ALLEGRO_CFG_DLL_TLS */
 
 
 
@@ -347,51 +472,24 @@ void al_set_target_bitmap(ALLEGRO_BITMAP *bitmap)
    thread_local_state *tls;
    ALLEGRO_DISPLAY *old_display;
    ALLEGRO_DISPLAY *new_display;
-   ALLEGRO_SHADER *old_shader;
-   ALLEGRO_SHADER *new_shader;
-   bool same_shader;
 
    ASSERT(!al_is_bitmap_drawing_held());
 
-   if (bitmap) {
-      if (bitmap->parent) {
-         bitmap->parent->dirty = true;
-      }
-      else {
-         bitmap->dirty = true;
-      }
-   }
-
    if ((tls = tls_get()) == NULL)
       return;
 
    old_display = tls->current_display;
 
-   if (tls->target_bitmap)
-      old_shader = tls->target_bitmap->shader;
-   else
-      old_shader = NULL;
-
    if (bitmap == NULL) {
       /* Explicitly releasing the current rendering context. */
       new_display = NULL;
-      new_shader = NULL;
    }
    else if (bitmap->flags & ALLEGRO_MEMORY_BITMAP) {
       /* Setting a memory bitmap doesn't change the rendering context. */
       new_display = old_display;
-      new_shader = NULL;
    }
    else {
       new_display = bitmap->display;
-      new_shader = bitmap->shader;
-   }
-
-   same_shader = (old_shader == new_shader && old_display == new_display);
-
-   /* Unset the old shader if necessary. */
-   if (old_shader && !same_shader) {
-      old_shader->vt->unuse_shader(old_shader, old_display);
    }
 
    /* Change the rendering context if necessary. */
@@ -418,18 +516,9 @@ void al_set_target_bitmap(ALLEGRO_BITMAP *bitmap)
          !(bitmap->flags & ALLEGRO_MEMORY_BITMAP) &&
          new_display &&
          new_display->vt &&
-         new_display->vt->set_target_bitmap)
-   {
+         new_display->vt->set_target_bitmap) {
       new_display->vt->set_target_bitmap(new_display, bitmap);
 
-      /* Set the new shader if necessary.  This should done before the
-       * update_transformation call, which will also update the shader's
-       * al_projview_matrix variable.
-       */
-      if (!same_shader || !new_shader) {
-         al_use_shader(new_shader);
-      }
-
       new_display->vt->update_transformation(new_display, bitmap);
    }
 }
@@ -475,13 +564,6 @@ void al_set_separate_blender(int op, int src, int dst,
    thread_local_state *tls;
    ALLEGRO_BLENDER *b;
 
-   ASSERT(op >= 0 && op < ALLEGRO_NUM_BLEND_OPERATIONS);
-   ASSERT(src >= 0 && src < ALLEGRO_NUM_BLEND_MODES);
-   ASSERT(dst >= 0 && src < ALLEGRO_NUM_BLEND_MODES);
-   ASSERT(alpha_op >= 0 && alpha_op < ALLEGRO_NUM_BLEND_OPERATIONS);
-   ASSERT(alpha_src >= 0 && alpha_src < ALLEGRO_NUM_BLEND_MODES);
-   ASSERT(alpha_dst >= 0 && alpha_dst < ALLEGRO_NUM_BLEND_MODES);
-
    if ((tls = tls_get()) == NULL)
       return;
 
@@ -562,6 +644,10 @@ void al_set_new_bitmap_flags(int flags)
    if ((tls = tls_get()) == NULL)
       return;
 
+   /* Assume ALLEGRO_VIDEO_BITMAP if ALLEGRO_MEMORY_BITMAP is not set. */
+   if (!(flags & ALLEGRO_MEMORY_BITMAP))
+      flags |= ALLEGRO_VIDEO_BITMAP;
+
    tls->new_bitmap_flags = flags;
 }
 
@@ -606,6 +692,7 @@ int al_get_new_bitmap_flags(void)
 
 
 
+#define _STORE(x) stored->tls.x = tls->x;
 /* Function: al_store_state
  */
 void al_store_state(ALLEGRO_STATE *state, int flags)
@@ -619,8 +706,6 @@ void al_store_state(ALLEGRO_STATE *state, int flags)
    stored = (void *)state;
    stored->flags = flags;
 
-#define _STORE(x) (stored->tls.x = tls->x)
-
    if (flags & ALLEGRO_STATE_NEW_DISPLAY_PARAMETERS) {
       _STORE(new_display_flags);
       _STORE(new_display_refresh_rate);
@@ -629,12 +714,12 @@ void al_store_state(ALLEGRO_STATE *state, int flags)
       _STORE(new_window_y);
       _STORE(new_display_settings);
    }
-
+   
    if (flags & ALLEGRO_STATE_NEW_BITMAP_PARAMETERS) {
       _STORE(new_bitmap_format);
       _STORE(new_bitmap_flags);
    }
-
+   
    if (flags & ALLEGRO_STATE_DISPLAY) {
       _STORE(current_display);
    }
@@ -651,7 +736,7 @@ void al_store_state(ALLEGRO_STATE *state, int flags)
       _STORE(new_file_interface);
       _STORE(fs_interface);
    }
-
+   
    if (flags & ALLEGRO_STATE_TRANSFORM) {
       ALLEGRO_BITMAP *target = al_get_target_bitmap();
       if (!target)
@@ -659,12 +744,12 @@ void al_store_state(ALLEGRO_STATE *state, int flags)
       else
          stored->stored_transform = target->transform;
    }
-
-#undef _STORE
 }
+#undef _STORE
 
 
 
+#define _STORE(x) tls->x = stored->tls.x;
 /* Function: al_restore_state
  */
 void al_restore_state(ALLEGRO_STATE const *state)
@@ -679,53 +764,46 @@ void al_restore_state(ALLEGRO_STATE const *state)
    stored = (void *)state;
    flags = stored->flags;
 
-#define _RESTORE(x) (tls->x = stored->tls.x)
-
    if (flags & ALLEGRO_STATE_NEW_DISPLAY_PARAMETERS) {
-      _RESTORE(new_display_flags);
-      _RESTORE(new_display_refresh_rate);
-      _RESTORE(new_display_adapter);
-      _RESTORE(new_window_x);
-      _RESTORE(new_window_y);
-      _RESTORE(new_display_settings);
+      _STORE(new_display_flags);
+      _STORE(new_display_refresh_rate);
+      _STORE(new_display_adapter);
+      _STORE(new_window_x);
+      _STORE(new_window_y);
+      _STORE(new_display_settings);
    }
-
+   
    if (flags & ALLEGRO_STATE_NEW_BITMAP_PARAMETERS) {
-      _RESTORE(new_bitmap_format);
-      _RESTORE(new_bitmap_flags);
+      _STORE(new_bitmap_format);
+      _STORE(new_bitmap_flags);
    }
-
+   
    if (flags & ALLEGRO_STATE_DISPLAY) {
-      if (tls->current_display != stored->tls.current_display) {
-         _al_set_current_display_only(stored->tls.current_display);
-         ASSERT(tls->current_display == stored->tls.current_display);
-      }
+      _STORE(current_display);
+      _al_set_current_display_only(tls->current_display);
    }
 
    if (flags & ALLEGRO_STATE_TARGET_BITMAP) {
-      if (tls->target_bitmap != stored->tls.target_bitmap) {
-         al_set_target_bitmap(stored->tls.target_bitmap);
-         ASSERT(tls->target_bitmap == stored->tls.target_bitmap);
-      }
+      _STORE(target_bitmap);
+      al_set_target_bitmap(tls->target_bitmap);
    }
-
+   
    if (flags & ALLEGRO_STATE_BLENDER) {
       tls->current_blender = stored->stored_blender;
    }
 
    if (flags & ALLEGRO_STATE_NEW_FILE_INTERFACE) {
-      _RESTORE(new_file_interface);
-      _RESTORE(fs_interface);
+      _STORE(new_file_interface);
+      _STORE(fs_interface);
    }
-
+   
    if (flags & ALLEGRO_STATE_TRANSFORM) {
       ALLEGRO_BITMAP *bitmap = al_get_target_bitmap();
       if (bitmap)
          al_use_transform(&stored->stored_transform);
    }
-
-#undef _RESTORE
 }
+#undef _STORE
 
 
 
@@ -797,15 +875,6 @@ void al_set_fs_interface(const ALLEGRO_FS_INTERFACE *fs_interface)
 
 
 
-/* Function: al_set_standard_fs_interface
- */
-void al_set_standard_fs_interface(void)
-{
-   al_set_fs_interface(&_al_fs_interface_stdio);
-}
-
-
-
 /* Function: al_get_errno
  */
 int al_get_errno(void)
@@ -831,32 +900,6 @@ void al_set_errno(int errnum)
 }
 
 
-#ifdef ALLEGRO_ANDROID
-/* Function: _al_android_get_jnienv
- */
-JNIEnv *_al_android_get_jnienv(void)
-{
-   thread_local_state *tls;
-
-   if ((tls = tls_get()) == NULL)
-      return 0;
-   return tls->jnienv;
-}
-
-
-
-/* Function: _al_android_set_jnienv
- */
-void _al_android_set_jnienv(JNIEnv *jnienv)
-{
-   thread_local_state *tls;
-
-   if ((tls = tls_get()) == NULL)
-      return;
-   tls->jnienv = jnienv;
-}
-#endif
-
 
 int *_al_tls_get_dtor_owner_count(void)
 {
