@@ -16,12 +16,12 @@
  */
 
 
-#include "allegro.h"
+#include "allegro5/allegro.h"
 
 
 
 /* list the available drivers */
-_DRIVER_INFO _keyboard_driver_list[] =
+_AL_DRIVER_INFO _al_keyboard_driver_list[] =
 {
    {  0,                NULL,              0     }
 };
