@@ -23,54 +23,33 @@
 #include "allegro5/internal/aintern.h"
 #include "allegro5/internal/aintern_bitmap.h"
 #include "allegro5/internal/aintern_display.h"
-#include "allegro5/internal/aintern_memblit.h"
-#include "allegro5/internal/aintern_pixels.h"
 #include "allegro5/internal/aintern_system.h"
-#include "allegro5/internal/aintern_tri_soft.h" // For ALLEGRO_VERTEX
 #include "allegro5/platform/aintwin.h"
 
 #include "d3d.h"
 
+extern "C" {
+
 ALLEGRO_DEBUG_CHANNEL("d3d")
 
 
 static ALLEGRO_BITMAP_INTERFACE *vt;
+static _AL_VECTOR created_bitmaps;
 
-// C++ needs to cast void pointers
-#define get_extra(b) ((ALLEGRO_BITMAP_EXTRA_D3D *)\
-   (b->parent ? b->parent->extra : b->extra))
-
-
-/* Function: al_get_d3d_texture_size
- */
-bool al_get_d3d_texture_size(ALLEGRO_BITMAP *bitmap, int *width, int *height)
-{
-   ASSERT(bitmap);
-   ASSERT(width);
-   ASSERT(height);
-
-   if (!(bitmap->flags & _ALLEGRO_INTERNAL_OPENGL) &&
-         !(bitmap->flags & ALLEGRO_MEMORY_BITMAP)) {
-      ALLEGRO_BITMAP_EXTRA_D3D *d3d_bmp = get_extra(bitmap);
-      *width = d3d_bmp->texture_w;
-      *height = d3d_bmp->texture_h;
-      return true;
-   }
-   else {
-      *width = 0;
-      *height = 0;
-      return false;
-   }
-}
 
 void _al_d3d_bmp_init(void)
 {
-  
+   _al_vector_init(&created_bitmaps, sizeof(ALLEGRO_BITMAP_D3D *));
 }
 
 
 void _al_d3d_bmp_destroy(void)
 {
+   while (!_al_vector_is_empty(&created_bitmaps))
+      _al_vector_delete_at(&created_bitmaps, _al_vector_size(&created_bitmaps)-1);
+   _al_vector_free(&created_bitmaps);
+   _al_vector_init(&created_bitmaps, sizeof(ALLEGRO_SYSTEM_INTERFACE *));
+
    al_free(vt);
    vt = NULL;
 }
@@ -84,7 +63,7 @@ static INLINE void transform_vertex(float* x, float* y)
  * Draw a textured quad
  */
 static void d3d_draw_textured_quad(
-   ALLEGRO_DISPLAY_D3D *disp, ALLEGRO_BITMAP *bmp, ALLEGRO_COLOR tint,
+   ALLEGRO_DISPLAY_D3D *disp, ALLEGRO_BITMAP_D3D *bmp, ALLEGRO_COLOR tint,
    float sx, float sy, float sw, float sh, int flags)
 {
    float right;
@@ -95,7 +74,7 @@ static void d3d_draw_textured_quad(
    float tv_end;
    int texture_w;
    int texture_h;
-   ALLEGRO_BITMAP_EXTRA_D3D *extra = get_extra(bmp);
+   DWORD d3d_color;
 
    const float z = 0.0f;
    
@@ -106,11 +85,13 @@ static void d3d_draw_textured_quad(
    }
    aldisp->cache_texture = (uintptr_t)bmp;
 
+   D3D_TL_VERTEX* vertices = (D3D_TL_VERTEX*)aldisp->vt->prepare_vertex_cache(aldisp, 6);
+
    right  = sw;
    bottom = sh;
 
-   texture_w = extra->texture_w;
-   texture_h = extra->texture_h;
+   texture_w = bmp->texture_w;
+   texture_h = bmp->texture_h;
    tu_start = sx / texture_w;
    tv_start = sy / texture_h;
    tu_end = sw / texture_w + tu_start;
@@ -127,59 +108,47 @@ static void d3d_draw_textured_quad(
       tv_end = temp;
    }
 
-#define ALLEGRO_COLOR_TO_D3D(c) D3DCOLOR_COLORVALUE(c.r, c.g, c.b, c.a)
+   d3d_color = D3DCOLOR_COLORVALUE(tint.r, tint.g, tint.b, tint.a);
 
-#define SET(f) \
-   vertices[0].x = 0; \
-   vertices[0].y = 0; \
-   vertices[0].z = z; \
-   vertices[0].color = f(tint); \
-   vertices[0].u = tu_start; \
-   vertices[0].v = tv_start; \
- \
-   vertices[1].x = right; \
-   vertices[1].y = 0; \
-   vertices[1].z = z; \
-   vertices[1].color = f(tint); \
-   vertices[1].u = tu_end; \
-   vertices[1].v = tv_start; \
- \
-   vertices[2].x = right; \
-   vertices[2].y = bottom; \
-   vertices[2].z = z; \
-   vertices[2].color = f(tint); \
-   vertices[2].u = tu_end; \
-   vertices[2].v = tv_end; \
- \
-   vertices[5].x = 0; \
-   vertices[5].y = bottom; \
-   vertices[5].z = z; \
-   vertices[5].color = f(tint); \
-   vertices[5].u = tu_start; \
-   vertices[5].v = tv_end; \
-\
-   if (aldisp->cache_enabled) { \
-      transform_vertex(&vertices[0].x, &vertices[0].y); \
-      transform_vertex(&vertices[1].x, &vertices[1].y); \
-      transform_vertex(&vertices[2].x, &vertices[2].y); \
-      transform_vertex(&vertices[5].x, &vertices[5].y); \
-   } \
-    \
-   vertices[3] = vertices[0]; \
+   vertices[0].x = 0;
+   vertices[0].y = 0;
+   vertices[0].z = z;
+   vertices[0].diffuse = d3d_color;
+   vertices[0].tu = tu_start;
+   vertices[0].tv = tv_start;
+
+   vertices[1].x = right;
+   vertices[1].y = 0;
+   vertices[1].z = z;
+   vertices[1].diffuse = d3d_color;
+   vertices[1].tu = tu_end;
+   vertices[1].tv = tv_start;
+
+   vertices[2].x = right;
+   vertices[2].y = bottom;
+   vertices[2].z = z;
+   vertices[2].diffuse = d3d_color;
+   vertices[2].tu = tu_end;
+   vertices[2].tv = tv_end;
+
+   vertices[5].x = 0;
+   vertices[5].y = bottom;
+   vertices[5].z = z;
+   vertices[5].diffuse = d3d_color;
+   vertices[5].tu = tu_start;
+   vertices[5].tv = tv_end;
+
+   if (aldisp->cache_enabled) {
+      transform_vertex(&vertices[0].x, &vertices[0].y);
+      transform_vertex(&vertices[1].x, &vertices[1].y);
+      transform_vertex(&vertices[2].x, &vertices[2].y);
+      transform_vertex(&vertices[5].x, &vertices[5].y);
+   }
+   
+   vertices[3] = vertices[0];
    vertices[4] = vertices[2];
 
-   bool pp = (aldisp->flags & ALLEGRO_PROGRAMMABLE_PIPELINE) != 0;
-
-   if (pp) {
-   	ALLEGRO_VERTEX *vertices = (ALLEGRO_VERTEX *)aldisp->vt->prepare_vertex_cache(aldisp, 6);
-	SET(ALLEGRO_COLOR)
-   }
-   else {
-   	D3D_FIXED_VERTEX *vertices = (D3D_FIXED_VERTEX *)aldisp->vt->prepare_vertex_cache(aldisp, 6);
-	SET(ALLEGRO_COLOR_TO_D3D)
-   }
-
-   if (!aldisp->cache_enabled)
+   if(!aldisp->cache_enabled)
       aldisp->vt->flush_vertex_cache(aldisp);
 }
 
@@ -187,7 +156,7 @@ static void d3d_draw_textured_quad(
 static void d3d_sync_bitmap_memory(ALLEGRO_BITMAP *bitmap)
 {
    D3DLOCKED_RECT locked_rect;
-   ALLEGRO_BITMAP_EXTRA_D3D *d3d_bmp = get_extra(bitmap);
+   ALLEGRO_BITMAP_D3D *d3d_bmp = (ALLEGRO_BITMAP_D3D *)bitmap;
    LPDIRECT3DTEXTURE9 texture;
 
    if (_al_d3d_render_to_texture_supported())
@@ -212,11 +181,9 @@ static void d3d_sync_bitmap_texture(ALLEGRO_BITMAP *bitmap,
 {
    D3DLOCKED_RECT locked_rect;
    RECT rect;
-   ALLEGRO_BITMAP_EXTRA_D3D *d3d_bmp = get_extra(bitmap);
+   ALLEGRO_BITMAP_D3D *d3d_bmp = (ALLEGRO_BITMAP_D3D *)bitmap;
    LPDIRECT3DTEXTURE9 texture;
 
-   if (bitmap->parent) return;
-
    rect.left = x;
    rect.top = y;
    rect.right = x + width;
@@ -269,10 +236,10 @@ static void d3d_sync_bitmap_texture(ALLEGRO_BITMAP *bitmap,
    }
 }
 
-static void d3d_do_upload(ALLEGRO_BITMAP *bmp, int x, int y, int width,
+static void d3d_do_upload(ALLEGRO_BITMAP_D3D *d3d_bmp, int x, int y, int width,
    int height, bool sync_from_memory)
 {
-   ALLEGRO_BITMAP_EXTRA_D3D *d3d_bmp = get_extra(bmp);
+   ALLEGRO_BITMAP *bmp = (ALLEGRO_BITMAP *)d3d_bmp;
 
    if (sync_from_memory) {
       d3d_sync_bitmap_texture(bmp, x, y, width, height);
@@ -292,19 +259,17 @@ static void d3d_do_upload(ALLEGRO_BITMAP *bmp, int x, int y, int width,
  * Release all default pool textures. This must be done before
  * resetting the device.
  */
-void _al_d3d_release_default_pool_textures(ALLEGRO_DISPLAY *disp)
+void _al_d3d_release_default_pool_textures(void)
 {
    unsigned int i;
 
-   for (i = 0; i < disp->bitmaps._size; i++) {
-   	ALLEGRO_BITMAP **bptr = (ALLEGRO_BITMAP **)_al_vector_ref(&disp->bitmaps, i);
+   for (i = 0; i < created_bitmaps._size; i++) {
+   	ALLEGRO_BITMAP **bptr = (ALLEGRO_BITMAP **)_al_vector_ref(&created_bitmaps, i);
 	ALLEGRO_BITMAP *albmp = *bptr;
-	_al_set_bitmap_shader_field(albmp, NULL);
-
+	ALLEGRO_BITMAP_D3D *d3d_bmp;
 	if ((albmp->flags & ALLEGRO_MEMORY_BITMAP) || (albmp->parent))
 	   continue;
-
-	ALLEGRO_BITMAP_EXTRA_D3D *d3d_bmp = get_extra(albmp);
+	d3d_bmp = (ALLEGRO_BITMAP_D3D *)albmp;
 	if (!d3d_bmp->is_backbuffer && d3d_bmp->render_target) {
 		d3d_bmp->render_target->Release();
 		d3d_bmp->render_target = NULL;
@@ -339,7 +304,7 @@ static bool d3d_create_textures(ALLEGRO_DISPLAY_D3D *disp, int w, int h,
       if (video_texture) {
          err = disp->device->CreateTexture(w, h, levels,
             D3DUSAGE_RENDERTARGET | autogenmipmap,
-            (D3DFORMAT)_al_pixel_format_to_d3d(format), D3DPOOL_DEFAULT,
+            (D3DFORMAT)_al_format_to_d3d(format), D3DPOOL_DEFAULT,
             video_texture, NULL);
          if (err != D3D_OK && err != D3DOK_NOAUTOGEN) {
             ALLEGRO_ERROR("d3d_create_textures: Unable to create video texture.\n");
@@ -349,7 +314,7 @@ static bool d3d_create_textures(ALLEGRO_DISPLAY_D3D *disp, int w, int h,
 
       if (system_texture) {
          err = disp->device->CreateTexture(w, h, 1,
-            0, (D3DFORMAT)_al_pixel_format_to_d3d(format), D3DPOOL_SYSTEMMEM,
+            0, (D3DFORMAT)_al_format_to_d3d(format), D3DPOOL_SYSTEMMEM,
             system_texture, NULL);
          if (err != D3D_OK) {
             ALLEGRO_ERROR("d3d_create_textures: Unable to create system texture.\n");
@@ -366,7 +331,7 @@ static bool d3d_create_textures(ALLEGRO_DISPLAY_D3D *disp, int w, int h,
    else {
       if (video_texture) {
          err = disp->device->CreateTexture(w, h, 1,
-            0, (D3DFORMAT)_al_pixel_format_to_d3d(format), D3DPOOL_MANAGED,
+            0, (D3DFORMAT)_al_format_to_d3d(format), D3DPOOL_MANAGED,
             video_texture, NULL);
          if (err != D3D_OK) {
             ALLEGRO_ERROR("d3d_create_textures: Unable to create video texture (no render-to-texture).\n");
@@ -383,10 +348,11 @@ static ALLEGRO_BITMAP
    int format, int flags)
 {
    ALLEGRO_BITMAP *bitmap;
-   ALLEGRO_BITMAP_EXTRA_D3D *extra;
+   ALLEGRO_BITMAP_D3D *d3d_bmp;
    D3DSURFACE_DESC desc;
    D3DLOCKED_RECT surf_locked_rect;
    D3DLOCKED_RECT sys_locked_rect;
+   ALLEGRO_STATE backup;
    unsigned int y;
 
    if (surface->GetDesc(&desc) != D3D_OK) {
@@ -399,16 +365,22 @@ static ALLEGRO_BITMAP
       return NULL;
    }
 
-   bitmap = _al_create_bitmap_params(al_get_current_display(),
-      desc.Width, desc.Height, format, flags);
+   al_store_state(&backup, ALLEGRO_STATE_NEW_BITMAP_PARAMETERS);
+
+   al_set_new_bitmap_format(format);
+   al_set_new_bitmap_flags(flags);
+
+   bitmap = al_create_bitmap(desc.Width, desc.Height);
+   d3d_bmp = (ALLEGRO_BITMAP_D3D *)bitmap;
+
+   al_restore_state(&backup);
+
    if (!bitmap) {
       surface->UnlockRect();
       return NULL;
    }
 
-   extra = get_extra(bitmap);
-
-   if (extra->system_texture->LockRect(0, &sys_locked_rect, 0, 0) != D3D_OK) {
+   if (d3d_bmp->system_texture->LockRect(0, &sys_locked_rect, 0, 0) != D3D_OK) {
       surface->UnlockRect();
       al_destroy_bitmap(bitmap);
       ALLEGRO_ERROR("d3d_create_bitmap_from_surface: Lock system texture failed.\n");
@@ -424,11 +396,11 @@ static ALLEGRO_BITMAP
    }
 
    surface->UnlockRect();
-   extra->system_texture->UnlockRect(0);
+   d3d_bmp->system_texture->UnlockRect(0);
 
-   if (extra->display->device->UpdateTexture(
-         (IDirect3DBaseTexture9 *)extra->system_texture,
-         (IDirect3DBaseTexture9 *)extra->video_texture) != D3D_OK) {
+   if (d3d_bmp->display->device->UpdateTexture(
+         (IDirect3DBaseTexture9 *)d3d_bmp->system_texture,
+         (IDirect3DBaseTexture9 *)d3d_bmp->video_texture) != D3D_OK) {
       ALLEGRO_ERROR("d3d_create_bitmap_from_texture: Couldn't update texture.\n");
    }
 
@@ -436,75 +408,59 @@ static ALLEGRO_BITMAP
 }
 
 /* Copies video texture to system texture and bitmap->memory */
-static bool _al_d3d_sync_bitmap(ALLEGRO_BITMAP *dest)
+static void _al_d3d_sync_bitmap(ALLEGRO_BITMAP *dest)
 {
-   ALLEGRO_BITMAP_EXTRA_D3D *d3d_dest;
+   ALLEGRO_BITMAP_D3D *d3d_dest;
    LPDIRECT3DSURFACE9 system_texture_surface;
    LPDIRECT3DSURFACE9 video_texture_surface;
-   bool ok;
    UINT i;
 
-   if (!_al_d3d_render_to_texture_supported()) {
-      ALLEGRO_ERROR("Render-to-texture not supported.\n");
-      return false;
-   }
+   if (!_al_d3d_render_to_texture_supported())
+      return;
 
    if (dest->locked) {
-      ALLEGRO_ERROR("Already locked.\n");
-      return false;
+      return;
    }
 
-   d3d_dest = get_extra(dest);
+   d3d_dest = (ALLEGRO_BITMAP_D3D *)dest;
 
    if (d3d_dest->system_texture == NULL || d3d_dest->video_texture == NULL) {
-      ALLEGRO_ERROR("A texture is null.\n");
-      return false;
+      return;
    }
 
    if (dest->parent) {
       dest = dest->parent;
    }
 
-   ok = true;
-   system_texture_surface = NULL;
-   video_texture_surface = NULL;
-
    if (d3d_dest->system_texture->GetSurfaceLevel(
          0, &system_texture_surface) != D3D_OK) {
       ALLEGRO_ERROR("_al_d3d_sync_bitmap: GetSurfaceLevel failed while updating video texture.\n");
-      ok = false;
+      return;
    }
 
-   if (ok && d3d_dest->video_texture->GetSurfaceLevel(
+   if (d3d_dest->video_texture->GetSurfaceLevel(
          0, &video_texture_surface) != D3D_OK) {
       ALLEGRO_ERROR("_al_d3d_sync_bitmap: GetSurfaceLevel failed while updating video texture.\n");
-      ok = false;
+      return;
    }
 
-   if (ok && d3d_dest->display->device->GetRenderTargetData(
+   if (d3d_dest->display->device->GetRenderTargetData(
          video_texture_surface,
          system_texture_surface) != D3D_OK) {
       ALLEGRO_ERROR("_al_d3d_sync_bitmap: GetRenderTargetData failed.\n");
-      ok = false;
+      return;
    }
 
-   if (system_texture_surface) {
-       if ((i = system_texture_surface->Release()) != 0) {
-	  ALLEGRO_DEBUG("_al_d3d_sync_bitmap (system) ref count == %d\n", i);
-       }
-   }
-   if (video_texture_surface) {
-       if ((i = video_texture_surface->Release()) != 0) {
-	  // This can be non-zero
-	  ALLEGRO_DEBUG("_al_d3d_sync_bitmap (video) ref count == %d\n", i);
-       }
+   if ((i = system_texture_surface->Release()) != 0) {
+      ALLEGRO_DEBUG("_al_d3d_sync_bitmap (system) ref count == %d\n", i);
    }
 
-   if (ok) {
-      d3d_sync_bitmap_memory(dest);
+   if ((i = video_texture_surface->Release()) != 0) {
+      // This can be non-zero
+      ALLEGRO_DEBUG("_al_d3d_sync_bitmap (video) ref count == %d\n", i);
    }
 
-   return ok;
+   d3d_sync_bitmap_memory(dest);
 }
 
 /*
@@ -515,7 +471,6 @@ static bool _al_d3d_sync_bitmap(ALLEGRO_BITMAP *dest)
 void _al_d3d_prepare_bitmaps_for_reset(ALLEGRO_DISPLAY_D3D *disp)
 {
    unsigned int i;
-   ALLEGRO_DISPLAY *display = (ALLEGRO_DISPLAY *)disp;
 
    if (disp->device_lost)
       return;
@@ -525,22 +480,22 @@ void _al_d3d_prepare_bitmaps_for_reset(ALLEGRO_DISPLAY_D3D *disp)
 
    al_lock_mutex(_al_d3d_lost_device_mutex);
 
-   for (i = 0; i < display->bitmaps._size; i++) {
-      ALLEGRO_BITMAP **bptr = (ALLEGRO_BITMAP **)_al_vector_ref(&display->bitmaps, i);
-      ALLEGRO_BITMAP *bmp = *bptr;
-      ALLEGRO_BITMAP_EXTRA_D3D *extra = get_extra(bmp);
-      if ((void *)bmp->display == (void *)disp) {
-         if ((bmp->flags & ALLEGRO_MEMORY_BITMAP) ||
-            (bmp->flags & ALLEGRO_NO_PRESERVE_TEXTURE) ||
-   	    !bmp->dirty ||
-   	    extra->is_backbuffer ||
-	    bmp->parent)
-	    continue;
-         _al_d3d_sync_bitmap(bmp);
-         bmp->dirty = false;
+   for (i = 0; i < created_bitmaps._size; i++) {
+      ALLEGRO_BITMAP_D3D **bptr = (ALLEGRO_BITMAP_D3D **)_al_vector_ref(&created_bitmaps, i);
+      ALLEGRO_BITMAP_D3D *bmp = *bptr;
+      ALLEGRO_BITMAP *al_bmp = (ALLEGRO_BITMAP *)bmp;
+      if (bmp->display == disp) {
+         //d3d_sync_bitmap_memory(al_bmp);
+         if (!al_bmp->preserve_texture) {
+            bmp->modified = false;
+         }
+         else if (!bmp->is_backbuffer && bmp->modified && !(al_bmp->flags & ALLEGRO_MEMORY_BITMAP)) {
+            _al_d3d_sync_bitmap(al_bmp);
+            bmp->modified = false;
+         }
       }
    }
-     
+
    al_unlock_mutex(_al_d3d_lost_device_mutex);
 }
 
@@ -550,22 +505,21 @@ void _al_d3d_prepare_bitmaps_for_reset(ALLEGRO_DISPLAY_D3D *disp)
 bool _al_d3d_recreate_bitmap_textures(ALLEGRO_DISPLAY_D3D *disp)
 {
    unsigned int i;
-   ALLEGRO_DISPLAY *display = (ALLEGRO_DISPLAY *)disp;
 
-   for (i = 0; i < display->bitmaps._size; i++) {
-      ALLEGRO_BITMAP **bptr = (ALLEGRO_BITMAP **)_al_vector_ref(&display->bitmaps, i);
-      ALLEGRO_BITMAP *bmp = *bptr;
-      ALLEGRO_BITMAP_EXTRA_D3D *extra = get_extra(bmp);
+   for (i = 0; i < created_bitmaps._size; i++) {
+      ALLEGRO_BITMAP_D3D **bptr = (ALLEGRO_BITMAP_D3D **)_al_vector_ref(&created_bitmaps, i);
+      ALLEGRO_BITMAP_D3D *bmp = *bptr;
+      ALLEGRO_BITMAP *al_bmp = (ALLEGRO_BITMAP *)bmp;
 
-      if ((void *)bmp->display == (void *)disp) {
-	      if (!d3d_create_textures(disp, extra->texture_w,
-            extra->texture_h,
-            bmp->flags,
-            &extra->video_texture,
-            &extra->system_texture,
-            bmp->format))
-            return false;
-	      d3d_do_upload(bmp, 0, 0, bmp->w, bmp->h, true);
+      if (bmp->display == disp) {
+	      if (!d3d_create_textures(disp, bmp->texture_w,
+		    bmp->texture_h,
+		    al_bmp->flags,
+		    &bmp->video_texture,
+		    &bmp->system_texture,
+		    al_bmp->format))
+		 return false;
+	      d3d_do_upload(bmp, 0, 0, al_bmp->w, al_bmp->h, true);
       }
    }
 
@@ -576,57 +530,36 @@ bool _al_d3d_recreate_bitmap_textures(ALLEGRO_DISPLAY_D3D *disp)
  * Refresh the texture memory. This must be done after a device is
  * lost or after it is reset.
  */
-void _al_d3d_refresh_texture_memory(ALLEGRO_DISPLAY *display)
+void _al_d3d_refresh_texture_memory(void)
 {
    unsigned int i;
 
-   /* Refresh video hardware textures */
-   for (i = 0; i < display->bitmaps._size; i++) {
-      ALLEGRO_BITMAP **bptr = (ALLEGRO_BITMAP **)_al_vector_ref(&display->bitmaps, i);
-      ALLEGRO_BITMAP *bmp = *bptr;
-      ALLEGRO_BITMAP_EXTRA_D3D *extra = get_extra(bmp);
-      ALLEGRO_DISPLAY_D3D *bmps_display = (ALLEGRO_DISPLAY_D3D *)bmp->display;
+   for (i = 0; i < created_bitmaps._size; i++) {
+      ALLEGRO_BITMAP_D3D **bptr = (ALLEGRO_BITMAP_D3D **)_al_vector_ref(&created_bitmaps, i);
+      ALLEGRO_BITMAP_D3D *bmp = *bptr;
+      ALLEGRO_BITMAP *al_bmp = (ALLEGRO_BITMAP *)bmp;
+      ALLEGRO_DISPLAY_D3D *bmps_display = (ALLEGRO_DISPLAY_D3D *)al_bmp->display;
 
-      if ((bmp->flags & ALLEGRO_MEMORY_BITMAP) || bmp->parent) {
+      if ((al_bmp->flags & ALLEGRO_MEMORY_BITMAP) || (al_bmp->parent)) {
          continue;
       }
 
-      d3d_create_textures(bmps_display, extra->texture_w,
-         extra->texture_h, bmp->flags,
-	 &extra->video_texture, /*&bmp->system_texture*/0, bmp->format);
-      if (!(bmp->flags & ALLEGRO_NO_PRESERVE_TEXTURE)) {
-         d3d_sync_bitmap_texture(bmp,
-	    0, 0, bmp->w, bmp->h);
-         if (_al_d3d_render_to_texture_supported()) {
-	    bmps_display->device->UpdateTexture(
-	       (IDirect3DBaseTexture9 *)extra->system_texture,
-               (IDirect3DBaseTexture9 *)extra->video_texture);
-         }
-      }
-   }
-
-   /* Reset sub-bitmaps */
-   for (i = 0; i < display->bitmaps._size; i++) {
-      ALLEGRO_BITMAP **bptr = (ALLEGRO_BITMAP **)_al_vector_ref(&display->bitmaps, i);
-      ALLEGRO_BITMAP *bmp = *bptr;
-      ALLEGRO_BITMAP_EXTRA_D3D *extra = get_extra(bmp);
-
-      if (bmp->flags & ALLEGRO_MEMORY_BITMAP) {
-         continue;
-      }
-
-      if (bmp->parent) {
-         ALLEGRO_BITMAP_EXTRA_D3D *pextra = get_extra(bmp->parent);
-	 extra->system_texture = pextra->system_texture;
-	 extra->video_texture = pextra->video_texture;
-	 extra->render_target = pextra->render_target;
+      d3d_create_textures(bmps_display, bmp->texture_w, bmp->texture_h,
+         al_bmp->flags,
+	 &bmp->video_texture, /*&bmp->system_texture*/0, al_bmp->format);
+      d3d_sync_bitmap_texture(al_bmp,
+	 0, 0, al_bmp->w, al_bmp->h);
+      if (_al_d3d_render_to_texture_supported()) {
+	 bmps_display->device->UpdateTexture(
+	    (IDirect3DBaseTexture9 *)bmp->system_texture,
+	    (IDirect3DBaseTexture9 *)bmp->video_texture);
       }
    }
 }
 
 static bool d3d_upload_bitmap(ALLEGRO_BITMAP *bitmap)
 {
-   ALLEGRO_BITMAP_EXTRA_D3D *d3d_bmp = get_extra(bitmap);
+   ALLEGRO_BITMAP_D3D *d3d_bmp = (ALLEGRO_BITMAP_D3D *)bitmap;
    int w = bitmap->w;
    int h = bitmap->h;
 
@@ -640,11 +573,11 @@ static bool d3d_upload_bitmap(ALLEGRO_BITMAP *bitmap)
       if (non_pow2 && non_square) {
          // Any shape and size
          d3d_bmp->texture_w = w;
-         d3d_bmp->texture_h = h;
+	 d3d_bmp->texture_h = h;
       }
       else if (non_pow2) {
-         // Must be square
-         int max = _ALLEGRO_MAX(w, h);
+         // Must be sqaure
+         int max = _ALLEGRO_MAX(w,  h);
          d3d_bmp->texture_w = max;
          d3d_bmp->texture_h = max;
       }
@@ -661,17 +594,23 @@ static bool d3d_upload_bitmap(ALLEGRO_BITMAP *bitmap)
       if (d3d_bmp->video_texture == 0)
          if (!d3d_create_textures(d3d_bmp->display, d3d_bmp->texture_w,
                d3d_bmp->texture_h,
-               bitmap->flags,
+               d3d_bmp->bitmap.flags,
                &d3d_bmp->video_texture,
                &d3d_bmp->system_texture,
                bitmap->format)) {
             return false;
          }
 
+      /*
+       * Keep track of created bitmaps, in case the display is lost
+       * or resized.
+       */
+      *(ALLEGRO_BITMAP_D3D **)_al_vector_alloc_back(&created_bitmaps) = d3d_bmp;
+
       d3d_bmp->initialized = true;
    }
 
-   d3d_do_upload(bitmap, 0, 0, w, h, false);
+   d3d_do_upload(d3d_bmp, 0, 0, w, h, false);
 
    return true;
 }
@@ -682,10 +621,8 @@ static void d3d_draw_bitmap_region(
    float sx, float sy, float sw, float sh, int flags)
 {
    ALLEGRO_BITMAP *dest = al_get_target_bitmap();
-   ALLEGRO_BITMAP_EXTRA_D3D *d3d_dest = get_extra(dest);
-   ALLEGRO_BITMAP_EXTRA_D3D *d3d_src = get_extra(src);
-   
-   ASSERT(src->parent == NULL);
+   ALLEGRO_BITMAP_D3D *d3d_dest = (ALLEGRO_BITMAP_D3D *)dest;
+   ALLEGRO_BITMAP_D3D *d3d_src = (ALLEGRO_BITMAP_D3D *)src;
 
    if (!_al_d3d_render_to_texture_supported()) {
       _al_draw_bitmap_region_memory(src, tint,
@@ -700,9 +637,15 @@ static void d3d_draw_bitmap_region(
    _al_d3d_set_bitmap_clip(dest);
 
    /* For sub-bitmaps */
+   if (src->parent) {
+      sx += src->xofs;
+      sy += src->yofs;
+      src = src->parent;
+      d3d_src = (ALLEGRO_BITMAP_D3D *)src;
+   }
    if (dest->parent) {
       dest = dest->parent;
-      d3d_dest = get_extra(dest);
+      d3d_dest = (ALLEGRO_BITMAP_D3D *)dest;
    }
 
    if (d3d_src->is_backbuffer) {
@@ -753,7 +696,7 @@ static void d3d_draw_bitmap_region(
          src->flags
       );
       if (tmp_bmp) {
-         d3d_draw_bitmap_region(tmp_bmp, tint,
+         d3d_draw_bitmap_region((ALLEGRO_BITMAP *)tmp_bmp, tint,
             sx, sy, sw, sh, flags);
          al_destroy_bitmap(tmp_bmp);
 	 if (desc.MultiSampleType != D3DMULTISAMPLE_NONE) {
@@ -766,16 +709,44 @@ static void d3d_draw_bitmap_region(
    _al_d3d_set_blender(d3d_dest->display);
 
    d3d_draw_textured_quad(
-      d3d_dest->display, src, tint,
+      d3d_dest->display, d3d_src, tint,
       sx, sy, sw, sh, flags);
+
+   d3d_dest->modified = true;
+}
+
+static void d3d_destroy_bitmap(ALLEGRO_BITMAP *bitmap)
+{
+   ALLEGRO_BITMAP_D3D *d3d_bmp = (ALLEGRO_BITMAP_D3D *)bitmap;
+
+   if (!al_is_sub_bitmap(bitmap)) {
+      if (d3d_bmp->video_texture) {
+         if (d3d_bmp->video_texture->Release() != 0) {
+            ALLEGRO_WARN("d3d_destroy_bitmap: Release video texture failed.\n");
+         }
+      }
+      if (d3d_bmp->system_texture) {
+         if (d3d_bmp->system_texture->Release() != 0) {
+            ALLEGRO_WARN("d3d_destroy_bitmap: Release system texture failed.\n");
+         }
+      }
+
+      if (d3d_bmp->render_target) {
+         if (d3d_bmp->render_target->Release() != 0) {
+            ALLEGRO_WARN("d3d_destroy_bitmap: Release render target failed.\n");
+         }
+      }
+   }
+
+   _al_vector_find_and_delete(&created_bitmaps, &d3d_bmp);
 }
 
 static ALLEGRO_LOCKED_REGION *d3d_lock_region(ALLEGRO_BITMAP *bitmap,
    int x, int y, int w, int h, int format,
    int flags)
 {
-   ALLEGRO_BITMAP_EXTRA_D3D *d3d_bmp = get_extra(bitmap);
-   
+   ALLEGRO_BITMAP_D3D *d3d_bmp = (ALLEGRO_BITMAP_D3D *)bitmap;
+
    if (d3d_bmp->display->device_lost)
       return NULL;
 
@@ -805,9 +776,7 @@ static ALLEGRO_LOCKED_REGION *d3d_lock_region(ALLEGRO_BITMAP *bitmap,
 	  * Sync bitmap->memory with texture
           */
 	 bitmap->locked = false;
-         if (!_al_d3d_sync_bitmap(bitmap)) {
-            return NULL;
-         }
+         _al_d3d_sync_bitmap(bitmap);
 	 bitmap->locked = true;
          texture = d3d_bmp->system_texture;
       }
@@ -844,7 +813,9 @@ static ALLEGRO_LOCKED_REGION *d3d_lock_region(ALLEGRO_BITMAP *bitmap,
 
 static void d3d_unlock_region(ALLEGRO_BITMAP *bitmap)
 {
-   ALLEGRO_BITMAP_EXTRA_D3D *d3d_bmp = get_extra(bitmap);
+   ALLEGRO_BITMAP_D3D *d3d_bmp = (ALLEGRO_BITMAP_D3D *)bitmap;
+
+   d3d_bmp->modified = true;
 
    if (bitmap->locked_region.format != 0 && bitmap->locked_region.format != bitmap->format) {
       if (!(bitmap->lock_flags & ALLEGRO_LOCK_READONLY)) {
@@ -869,7 +840,7 @@ static void d3d_unlock_region(ALLEGRO_BITMAP *bitmap)
       texture->UnlockRect(0);
       if (bitmap->lock_flags & ALLEGRO_LOCK_READONLY)
          return;
-      d3d_do_upload(bitmap, bitmap->lock_x, bitmap->lock_y,
+      d3d_do_upload(d3d_bmp, bitmap->lock_x, bitmap->lock_y,
          bitmap->lock_w, bitmap->lock_h, false);
    }
 }
@@ -893,7 +864,7 @@ ALLEGRO_BITMAP_INTERFACE *_al_bitmap_d3d_driver(void)
    vt->draw_bitmap_region = d3d_draw_bitmap_region;
    vt->upload_bitmap = d3d_upload_bitmap;
    vt->update_clipping_rectangle = NULL;
-   vt->destroy_bitmap = _al_d3d_destroy_bitmap;
+   vt->destroy_bitmap = d3d_destroy_bitmap;
    vt->lock_region = d3d_lock_region;
    vt->unlock_region = d3d_unlock_region;
    vt->update_clipping_rectangle = d3d_update_clipping_rectangle;
@@ -901,3 +872,4 @@ ALLEGRO_BITMAP_INTERFACE *_al_bitmap_d3d_driver(void)
    return vt;
 }
 
+} // end extern "C"
