@@ -1,5 +1,5 @@
 /*         ______   ___    ___ 
- *        /\  _  \ /\_ \  /\_ \
+ *        /\  _  \ /\_ \  /\_ \ 
  *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
  *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
  *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
@@ -88,6 +88,19 @@ static char* get_error_desc(DWORD err)
    return err_msg;
 }
 
+/* Helper to set up GL state as we want it. */
+static void setup_gl(ALLEGRO_DISPLAY *d)
+{
+   glViewport(0, 0, d->w, d->h);
+
+   glMatrixMode(GL_PROJECTION);
+   glLoadIdentity();
+   glOrtho(0, d->w, d->h, 0, -1, 1);
+
+   glMatrixMode(GL_MODELVIEW);
+   glLoadIdentity();
+}
+
 static bool is_wgl_extension_supported(const char *extension, HDC dc)
 {
    _ALLEGRO_wglGetExtensionsStringARB_t _wglGetExtensionsStringARB;
@@ -989,6 +1002,15 @@ static bool create_display_internals(ALLEGRO_DISPLAY_WGL *wgl_disp)
       disp->extra_settings.settings[ALLEGRO_COMPATIBLE_DISPLAY] = 0;
    }
 
+   disp->ogl_extras->backbuffer = _al_ogl_create_backbuffer(disp);
+   if (!disp->ogl_extras->backbuffer) {
+      ALLEGRO_ERROR("Failed to create a backbuffer.\n");
+      destroy_display_internals(wgl_disp);
+      return false;
+   }
+
+   al_identity_transform(&al_get_backbuffer(disp)->transform);
+
    /* Try to enable or disable vsync as requested */
    /* NOTE: my drivers claim I don't have WGL_EXT_swap_control
     * (according to al_have_opengl_extension), but wglSwapIntervalEXT
@@ -1010,7 +1032,7 @@ static bool create_display_internals(ALLEGRO_DISPLAY_WGL *wgl_disp)
    _al_win_grab_input(win_disp);
 
    if (disp->extra_settings.settings[ALLEGRO_COMPATIBLE_DISPLAY])
-      _al_ogl_setup_gl(disp);
+      setup_gl(disp);
 
    return true;
 }
@@ -1080,27 +1102,8 @@ static void destroy_display_internals(ALLEGRO_DISPLAY_WGL *wgl_disp)
    _al_ogl_unmanage_extensions(disp);
 
    PostMessage(win_disp->window, _al_win_msg_suicide, (WPARAM)win_disp, 0);
-
    while (!win_disp->thread_ended)
       al_rest(0.001);
-
-   if (wgl_disp->glrc) {
-      wglDeleteContext(wgl_disp->glrc);
-      wgl_disp->glrc = NULL;
-   }
-   if (wgl_disp->dc) {
-      ReleaseDC(win_disp->window, wgl_disp->dc);
-      wgl_disp->dc = NULL;
-   }
-
-   if (disp->flags & ALLEGRO_FULLSCREEN && !_wgl_do_not_change_display_mode) {
-      ChangeDisplaySettings(NULL, 0);
-   }
-
-   if (win_disp->window) {
-      DestroyWindow(win_disp->window);
-      win_disp->window = NULL;
-   }
 }
 
 
@@ -1121,7 +1124,6 @@ static void wgl_destroy_display(ALLEGRO_DISPLAY *disp)
    _al_vector_find_and_delete(&system->system.displays, &disp);
 
    _al_vector_free(&disp->bitmaps);
-   _al_vector_free(&((ALLEGRO_DISPLAY_WIN*) disp)->msg_callbacks);
    al_free(disp->ogl_extras);
 
    if (old_disp != disp)
@@ -1150,8 +1152,6 @@ static bool wgl_set_current_display(ALLEGRO_DISPLAY *d)
       _al_ogl_set_extensions(d->ogl_extras->extension_api);
    }
 
-   _al_ogl_update_render_state(d);
-
    return true;
 }
 
@@ -1308,6 +1308,24 @@ static void display_thread_proc(void *arg)
          break;                 /* WM_QUIT received or error (GetMessage returned -1)  */
    }
 
+   if (wgl_disp->glrc) {
+      wglDeleteContext(wgl_disp->glrc);
+      wgl_disp->glrc = NULL;
+   }
+   if (wgl_disp->dc) {
+      ReleaseDC(win_disp->window, wgl_disp->dc);
+      wgl_disp->dc = NULL;
+   }
+
+   if (disp->flags & ALLEGRO_FULLSCREEN && !_wgl_do_not_change_display_mode) {
+      ChangeDisplaySettings(NULL, 0);
+   }
+
+   if (win_disp->window) {
+      DestroyWindow(win_disp->window);
+      win_disp->window = NULL;
+   }
+
    ALLEGRO_INFO("wgl display thread exits\n");
    win_disp->thread_ended = true;
 }
@@ -1316,8 +1334,9 @@ static void display_thread_proc(void *arg)
 static void wgl_flip_display(ALLEGRO_DISPLAY *d)
 {
    ALLEGRO_DISPLAY_WGL* disp = (ALLEGRO_DISPLAY_WGL*)d;
-   glFlush();
-   if (!d->extra_settings.settings[ALLEGRO_SINGLE_BUFFER])
+   if (d->extra_settings.settings[ALLEGRO_SINGLE_BUFFER])
+      glFlush();
+   else
       SwapBuffers(disp->dc);
 }
 
@@ -1367,13 +1386,10 @@ static bool wgl_resize_helper(ALLEGRO_DISPLAY *d, int width, int height)
       size_t i;
 
       target_bmp = al_get_target_bitmap();
-      if (target_bmp && target_bmp->vt) {
-         ALLEGRO_BITMAP_EXTRA_OPENGL *extra = target_bmp->parent ?
-            target_bmp->parent->extra : target_bmp->extra;
-         was_backbuffer = extra->is_backbuffer;
-      }
+      if (target_bmp && target_bmp->vt)
+         was_backbuffer = ((ALLEGRO_BITMAP_OGL*)target_bmp)->is_backbuffer;
 
-      /* Remember display bitmaps. */
+      /* Remeber display bitmaps. */
       _al_vector_init(&disp_bmps, sizeof(ALLEGRO_BITMAP*));
       for (i = 0; i < _al_vector_size(&d->bitmaps); i++) {
          ALLEGRO_BITMAP **dis = _al_vector_ref(&d->bitmaps, i);
@@ -1460,8 +1476,8 @@ static bool wgl_acknowledge_resize(ALLEGRO_DISPLAY *d)
 {
    WINDOWINFO wi;
    ALLEGRO_DISPLAY_WIN *win_disp = (ALLEGRO_DISPLAY_WIN *)d;
+   ALLEGRO_DISPLAY *ogl_disp = (ALLEGRO_DISPLAY *)d;
    int w, h;
-   ALLEGRO_STATE state;
 
    wi.cbSize = sizeof(WINDOWINFO);
    GetWindowInfo(win_disp->window, &wi);
@@ -1471,13 +1487,8 @@ static bool wgl_acknowledge_resize(ALLEGRO_DISPLAY *d)
    d->w = w;
    d->h = h;
 
-   _al_ogl_setup_gl(d);
-   _al_ogl_update_render_state(d);
-
-   al_store_state(&state, ALLEGRO_STATE_DISPLAY | ALLEGRO_STATE_TARGET_BITMAP);
-   al_set_target_backbuffer(d);
-   al_set_clipping_rectangle(0, 0, w, h);
-   al_restore_state(&state);
+   setup_gl(d);
+   _al_ogl_resize_backbuffer(ogl_disp->ogl_extras->backbuffer, w, h);
 
    return true;
 }
@@ -1539,6 +1550,7 @@ ALLEGRO_DISPLAY_INTERFACE *_al_display_wgl_driver(void)
    vt.update_display_region = wgl_update_display_region;
    vt.acknowledge_resize = wgl_acknowledge_resize;
    vt.create_bitmap = _al_ogl_create_bitmap;
+   vt.create_sub_bitmap = _al_ogl_create_sub_bitmap;
    vt.get_backbuffer = _al_ogl_get_backbuffer;
    vt.set_target_bitmap = _al_ogl_set_target_bitmap;
    vt.is_compatible_bitmap = wgl_is_compatible_bitmap;
@@ -1550,15 +1562,11 @@ ALLEGRO_DISPLAY_INTERFACE *_al_display_wgl_driver(void)
    vt.show_mouse_cursor = _al_win_show_mouse_cursor;
    vt.hide_mouse_cursor = _al_win_hide_mouse_cursor;
 
-   vt.set_icons = _al_win_set_display_icons;
+   vt.set_icon = _al_win_set_display_icon;
    vt.set_window_position = wgl_set_window_position;
    vt.get_window_position = wgl_get_window_position;
-   vt.set_window_constraints = _al_win_set_window_constraints;
-   vt.get_window_constraints = _al_win_get_window_constraints;
    vt.set_display_flag = _al_win_set_display_flag;
    vt.set_window_title = _al_win_set_window_title;
-
-   vt.update_render_state = _al_ogl_update_render_state;
    _al_ogl_add_drawing_functions(&vt);
 
    return &vt;
@@ -1611,26 +1619,6 @@ ALLEGRO_DISPLAY_MODE *_al_wgl_get_display_mode(int index, int format,
    mode->width = dm.dmPelsWidth;
    mode->height = dm.dmPelsHeight;
    mode->refresh_rate = dm.dmDisplayFrequency;
-   mode->format = format;
-   switch (dm.dmBitsPerPel) {
-      case 32:
-         if (format == ALLEGRO_PIXEL_FORMAT_ANY)
-            mode->format = ALLEGRO_PIXEL_FORMAT_ANY_32_WITH_ALPHA;
-         else if (format == ALLEGRO_PIXEL_FORMAT_ANY_NO_ALPHA)
-            mode->format = ALLEGRO_PIXEL_FORMAT_ANY_32_NO_ALPHA;
-         break;
-      case 24:
-         mode->format = ALLEGRO_PIXEL_FORMAT_ANY_24_NO_ALPHA;
-         break;
-      case 16:
-         if (format == ALLEGRO_PIXEL_FORMAT_ANY)
-            mode->format = ALLEGRO_PIXEL_FORMAT_ANY_16_WITH_ALPHA;
-         else if(format == ALLEGRO_PIXEL_FORMAT_ANY_NO_ALPHA)
-            mode->format = ALLEGRO_PIXEL_FORMAT_ANY_16_NO_ALPHA;
-         break;
-      default:
-         break;
-   }
 
    return mode;
 }
