@@ -18,7 +18,6 @@
 
 #include "allegro5/allegro.h"
 #include "allegro5/internal/aintern.h"
-#include "allegro5/internal/aintern_driver.h"
 #include "allegro5/internal/aintern_events.h"
 #include "allegro5/internal/aintern_keyboard.h"
 #include "allegro5/platform/aintwin.h"
@@ -388,7 +387,7 @@ void _al_win_kbd_handle_key_press(int scode, int vcode, bool extended,
    /* Toggle mouse grab key. */
    if (my_code && !repeated) {
       ALLEGRO_SYSTEM_WIN *system = (void *)al_get_system_driver();
-      if (system->toggle_mouse_grab_keycode && my_code == system->toggle_mouse_grab_keycode &&
+      if (my_code == system->toggle_mouse_grab_keycode &&
          (modifiers & system->toggle_mouse_grab_modifiers) == system->toggle_mouse_grab_modifiers)
       {
          if (system->mouse_grab_display == display) {
