@@ -1,6 +1,6 @@
-/*         ______   ___    ___
- *        /\  _  \ /\_ \  /\_ \
- *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
+/*         ______   ___    ___ 
+ *        /\  _  \ /\_ \  /\_ \ 
+ *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
  *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
  *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
  *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
@@ -29,6 +29,9 @@
 #if defined ALLEGRO_CFG_OPENGL
    #include "allegro5/allegro_opengl.h"
 #endif
+#if defined ALLEGRO_CFG_D3D
+   #include "allegro5/allegro_direct3d.h"
+#endif
 
 #include <windows.h>
 #include <mmsystem.h>
@@ -140,20 +143,14 @@ int _WinMain(void *_main, void *hInst, void *hPrev, char *Cmd, int nShow)
 }
 
 
-static bool maybe_d3d_init_display(void)
-{
-#ifdef ALLEGRO_CFG_D3D
-   return _al_d3d_init_display();
-#else
-   return false;
-#endif
-}
 
 
 /* Create a new system object. */
 static ALLEGRO_SYSTEM *win_initialize(int flags)
 {
+   bool result;
    (void)flags;
+   (void)result;
 
    _al_win_system = al_calloc(1, sizeof *_al_win_system);
 
@@ -169,20 +166,25 @@ static ALLEGRO_SYSTEM *win_initialize(int flags)
 
    _al_win_system->system.vt = vt;
 
-   d3d_available = maybe_d3d_init_display();
-
+#if defined ALLEGRO_CFG_D3D
+   result = _al_d3d_init_display();
+#ifndef ALLEGRO_CFG_OPENGL
+   if (result != true)
+      return NULL;
+#else
+   if (result != true)
+      d3d_available = false;
+#endif
+#endif
+   
    return &_al_win_system->system;
 }
 
 
 static void win_shutdown(void)
 {
-   ALLEGRO_SYSTEM *s;
-   ALLEGRO_DISPLAY_INTERFACE *display_driver;
-   ASSERT(vt);
-
    /* Close all open displays. */
-   s = al_get_system_driver();
+   ALLEGRO_SYSTEM *s = al_get_system_driver();
    while (_al_vector_size(&s->displays) > 0) {
       ALLEGRO_DISPLAY **dptr = _al_vector_ref(&s->displays, 0);
       ALLEGRO_DISPLAY *d = *dptr;
@@ -192,9 +194,8 @@ static void win_shutdown(void)
 
    _al_vector_free(&s->displays);
 
-   display_driver = vt->get_display_driver();
-   if (display_driver && display_driver->shutdown) {
-      display_driver->shutdown();
+   if (vt->get_display_driver && vt->get_display_driver()->shutdown) {
+      vt->get_display_driver()->shutdown();
    }
 
    _al_win_shutdown_time();
@@ -203,6 +204,7 @@ static void win_shutdown(void)
       timeEndPeriod(1);
    }
 
+   ASSERT(vt);
    al_free(vt);
    vt = NULL;
 
@@ -213,9 +215,11 @@ static void win_shutdown(void)
 
 static ALLEGRO_DISPLAY_INTERFACE *win_get_display_driver(void)
 {
-   const int flags = al_get_new_display_flags();
+   int flags = al_get_new_display_flags();
    ALLEGRO_SYSTEM *sys = al_get_system_driver();
    ALLEGRO_SYSTEM_WIN *syswin = (ALLEGRO_SYSTEM_WIN *)sys;
+   const char *s;
+   ALLEGRO_DISPLAY_INTERFACE *ret = NULL;
 
    /* Look up the toggle_mouse_grab_key binding.  This isn't such a great place
     * to do it, but the config file is not available in win_initialize,
@@ -236,65 +240,61 @@ static ALLEGRO_DISPLAY_INTERFACE *win_get_display_driver(void)
       }
    }
 
-   /* Programmatic selection. */
-#ifdef ALLEGRO_CFG_D3D
-   if (flags & ALLEGRO_DIRECT3D_INTERNAL) {
-      if (d3d_available) {
-         return _al_display_d3d_driver();
-      }
-      ALLEGRO_WARN("Direct3D graphics driver not available.\n");
-      return NULL;
+#if defined ALLEGRO_CFG_D3D
+   if (flags & ALLEGRO_DIRECT3D) {
+      return _al_display_d3d_driver();
    }
 #endif
-#ifdef ALLEGRO_CFG_OPENGL
+#if defined ALLEGRO_CFG_OPENGL
    if (flags & ALLEGRO_OPENGL) {
       return _al_display_wgl_driver();
    }
 #endif
 
-   /* Selection by configuration file.  The configuration value is non-binding.
-    * The user may unknowingly set a value which was configured out at compile
-    * time.  The value should have no effect instead of causing a failure.
-    */
    if (sys->config) {
-      const char *s = al_get_config_value(sys->config, "graphics", "driver");
+      s = al_get_config_value(sys->config, "graphics", "driver");
       if (s) {
-         ALLEGRO_DEBUG("Configuration value graphics.driver = %s\n", s);
-         if (0 == _al_stricmp(s, "DIRECT3D") || 0 == _al_stricmp(s, "D3D")) {
-#ifdef ALLEGRO_CFG_D3D
-            if (d3d_available) {
-               al_set_new_display_flags(flags | ALLEGRO_DIRECT3D_INTERNAL);
-               return _al_display_d3d_driver();
-            }
-#endif
-         }
-         else if (0 == _al_stricmp(s, "OPENGL")) {
-#ifdef ALLEGRO_CFG_OPENGL
-            al_set_new_display_flags(flags | ALLEGRO_OPENGL);
+         if (!_al_stricmp(s, "OPENGL")) {
+#if defined ALLEGRO_CFG_OPENGL
+            flags |= ALLEGRO_OPENGL;
+            al_set_new_display_flags(flags);
             return _al_display_wgl_driver();
+#else
+            return NULL;
 #endif
          }
-         else if (0 != _al_stricmp(s, "DEFAULT")) {
-            ALLEGRO_WARN("Graphics driver selection unrecognised: %s\n", s);
+         else if (!_al_stricmp(s, "DIRECT3D") || !_al_stricmp(s, "D3D")) {
+#if defined ALLEGRO_CFG_D3D
+            flags |= ALLEGRO_DIRECT3D;
+            al_set_new_display_flags(flags);
+            return _al_display_d3d_driver();
+#else
+            return NULL;
+#endif
          }
       }
    }
 
-   /* Automatic graphics driver selection. */
-   /* XXX is implicitly setting new_display_flags the desired behaviour? */
-#ifdef ALLEGRO_CFG_D3D
+#if defined ALLEGRO_CFG_D3D
+#ifdef ALLEGRO_CFG_OPENGL
    if (d3d_available) {
-      al_set_new_display_flags(flags | ALLEGRO_DIRECT3D_INTERNAL);
-      return _al_display_d3d_driver();
-   }
 #endif
+      flags |= ALLEGRO_DIRECT3D;
+      al_set_new_display_flags(flags);
+      ret = _al_display_d3d_driver();
+      if (ret != NULL)
+         return ret;
+      flags &= ~ALLEGRO_DIRECT3D;
 #ifdef ALLEGRO_CFG_OPENGL
-   {
-      al_set_new_display_flags(flags | ALLEGRO_OPENGL);
-      return _al_display_wgl_driver();
    }
 #endif
-   ALLEGRO_WARN("No graphics driver available.\n");
+#endif
+#if defined ALLEGRO_CFG_OPENGL
+   flags |= ALLEGRO_OPENGL;
+   al_set_new_display_flags(flags);
+   return _al_display_wgl_driver();
+#endif
+
    return NULL;
 }
 
@@ -432,11 +432,6 @@ static ALLEGRO_MOUSE_DRIVER *win_get_mouse_driver(void)
    return _al_mouse_driver_list[0].driver;
 }
 
-static ALLEGRO_TOUCH_INPUT_DRIVER *win_get_touch_input_driver(void)
-{
-   return _al_touch_input_driver_list[0].driver;
-}
-
 /* _al_win_get_path:
  *  Returns full path to various system and user diretories
  */
@@ -484,7 +479,7 @@ ALLEGRO_PATH *_al_win_get_path(int id)
       case ALLEGRO_USER_HOME_PATH: /* CSIDL_PROFILE */
          csidl = CSIDL_PROFILE;
          break;
-
+         
       case ALLEGRO_USER_DOCUMENTS_PATH: /* CSIDL_PERSONAL */
          csidl = CSIDL_PERSONAL;
          break;
@@ -495,7 +490,7 @@ ALLEGRO_PATH *_al_win_get_path(int id)
 
          return al_create_path(path);
       } break;
-
+      
       default:
          return NULL;
    }
@@ -604,7 +599,7 @@ HMODULE _al_win_safe_load_library(const char *filename)
    const char *other_dirs[3];
    HMODULE lib = NULL;
    bool msvc_only = false;
-
+   
    /* MSVC only: if the executable is in the build configuration directory,
     * which is also just under the current directory, then also try to load the
     * library from the current directory.  This leads to less surprises when
@@ -675,7 +670,6 @@ static ALLEGRO_SYSTEM_INTERFACE *_al_system_win_driver(void)
    vt->get_display_driver = win_get_display_driver;
    vt->get_keyboard_driver = win_get_keyboard_driver;
    vt->get_mouse_driver = win_get_mouse_driver;
-   vt->get_touch_input_driver = win_get_touch_input_driver;
    vt->get_joystick_driver = win_get_joystick_driver;
    vt->get_num_display_modes = win_get_num_display_modes;
    vt->get_display_mode = win_get_display_mode;
@@ -700,8 +694,10 @@ void _al_register_system_interfaces()
 {
    ALLEGRO_SYSTEM_INTERFACE **add;
 
+#if defined ALLEGRO_CFG_D3D || defined ALLEGRO_CFG_OPENGL
    add = _al_vector_alloc_back(&_al_system_interfaces);
    *add = _al_system_win_driver();
+#endif
 }
 
 /* vim: set sts=3 sw=3 et: */
