@@ -1,16 +1,7 @@
-#include "allegro5/allegro.h"
-#include "allegro5/allegro_opengl.h"
-#include "allegro5/opengl/gl_ext.h"
-#include "allegro5/internal/aintern.h"
-#include "allegro5/internal/aintern_display.h"
-#include "allegro5/internal/aintern_opengl.h"
+#include "allegro5/internal/aintern_xglx.h"
 #include "allegro5/internal/aintern_pixels.h"
-#include "allegro5/internal/aintern_x.h"
-#include "allegro5/internal/aintern_xdisplay.h"
-#include "allegro5/internal/aintern_xglx_config.h"
-#include "allegro5/internal/aintern_xsystem.h"
 
-ALLEGRO_DEBUG_CHANNEL("xglx_config")
+ALLEGRO_DEBUG_CHANNEL("display")
 
 #ifdef DEBUGMODE
 static void display_pixel_format(ALLEGRO_EXTRA_DISPLAY_SETTINGS *eds)
@@ -536,13 +527,7 @@ bool _al_xglx_config_create_context(ALLEGRO_DISPLAY_XGLX *glx)
          bool forward_compat = (disp->flags & ALLEGRO_OPENGL_FORWARD_COMPATIBLE) != 0;
          glx->context = create_context_new(glx->glx_version,
             system->gfxdisplay, *glx->fbc, existing_ctx, forward_compat, 3, 0);
-         /* TODO: Right now Allegro's own OpenGL driver only works with a 3.0+
-          * context when using the programmable pipeline, for some reason. All
-          * that's missing is probably a default shader though.
-          */
-         disp->extra_settings.settings[ALLEGRO_COMPATIBLE_DISPLAY] = 1;
-         if (forward_compat && !(disp->flags & ALLEGRO_PROGRAMMABLE_PIPELINE))
-            disp->extra_settings.settings[ALLEGRO_COMPATIBLE_DISPLAY] = 0;
+         disp->extra_settings.settings[ALLEGRO_COMPATIBLE_DISPLAY] = !forward_compat;
       }
       else {
          glx->context = glXCreateNewContext(system->gfxdisplay, *glx->fbc,
