@@ -35,15 +35,7 @@
 #include "allegro5/internal/aintern.h"
 #include "allegro5/internal/aintern_events.h"
 #include "allegro5/internal/aintern_keyboard.h"
-#include "allegro5/internal/aintern_x.h"
-#include "allegro5/internal/aintern_xdisplay.h"
-#include "allegro5/internal/aintern_xkeyboard.h"
-#include "allegro5/internal/aintern_xsystem.h"
-
-#ifdef ALLEGRO_RASPBERRYPI
-#include "allegro5/internal/aintern_raspberrypi.h"
-#define ALLEGRO_SYSTEM_XGLX ALLEGRO_SYSTEM_RASPBERRYPI
-#endif
+#include "allegro5/internal/aintern_xglx.h"
 
 ALLEGRO_DEBUG_CHANNEL("keyboard")
 
@@ -454,21 +446,24 @@ void _al_xwin_keyboard_handler(XKeyEvent *event, ALLEGRO_DISPLAY *display)
 /* _al_xwin_keyboard_switch_handler:
  *  Handle a focus switch event.
  */
-void _al_xwin_keyboard_switch_handler(ALLEGRO_DISPLAY *display, bool focus_in)
+void _al_xwin_keyboard_switch_handler(ALLEGRO_DISPLAY *display,
+   const XFocusChangeEvent *event)
 {
    _al_event_source_lock(&the_keyboard.parent.es);
 
-   if (focus_in) {
-      the_keyboard.state.display = display;
+   switch (event->type) {
+      case FocusIn:
+         the_keyboard.state.display = display;
 #ifdef ALLEGRO_XWINDOWS_WITH_XIM
-      if (xic) {
-         ALLEGRO_DISPLAY_XGLX *display_glx = (void *)display;
-         XSetICValues(xic, XNClientWindow, display_glx->window, NULL);
-      }
+         if (xic) {
+            ALLEGRO_DISPLAY_XGLX *display_glx = (void *)display;
+            XSetICValues(xic, XNClientWindow, display_glx->window, NULL);
+         }
 #endif
-   }
-   else {
-      the_keyboard.state.display = NULL;
+         break;
+      case FocusOut:
+         the_keyboard.state.display = NULL;
+         break;
    }
 
    _al_event_source_unlock(&the_keyboard.parent.es);
@@ -520,12 +515,11 @@ static const char *x_scancode_to_name(int scancode)
  *  KeySyms to ALLEGRO_KEY_* codes.
  */
 
-static bool _al_xwin_get_keyboard_mapping(void)
+static void _al_xwin_get_keyboard_mapping(void)
 {
    int i;
    int count;
    int missing = 0;
-
    ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
 
    memset(used, 0, sizeof used);
@@ -542,10 +536,6 @@ static bool _al_xwin_get_keyboard_mapping(void)
 
    ALLEGRO_INFO("%i keys, %i symbols per key.\n", count, sym_per_key);
 
-   if (sym_per_key <= 0) {
-      return false;
-   }
-
    missing = 0;
 
    for (i = min_keycode; i <= max_keycode; i++) {
@@ -647,8 +637,6 @@ static bool _al_xwin_get_keyboard_mapping(void)
       }
       key = al_get_next_config_entry(&it);
    }
-
-   return true;
 }
 
 
@@ -719,13 +707,7 @@ static int x_keyboard_init(void)
    ALLEGRO_INFO("Using X Input Method.\n");
 
    old_locale = setlocale(LC_CTYPE, NULL);
-   ALLEGRO_DEBUG("Old locale: %s\n", old_locale ? old_locale : "(null)");
-   if (old_locale) {
-      /* The return value of setlocale() may be clobbered by the next call
-       * to setlocale() so we must copy it.
-       */
-      old_locale = strdup(old_locale);
-   }
+   ALLEGRO_DEBUG("Old locale: %s\n", old_locale);
 
    /* Otherwise we are restricted to ISO-8859-1 characters. */
    if (setlocale(LC_CTYPE, "") == NULL) {
@@ -747,7 +729,6 @@ static int x_keyboard_init(void)
    if (old_locale) {
       ALLEGRO_DEBUG("Restoring old locale: %s\n", old_locale);
       setlocale(LC_CTYPE, old_locale);
-      free(old_locale);
    }
 
    if (xim) {
@@ -797,9 +778,7 @@ static int x_keyboard_init(void)
    }
 #endif
 
-   if (!_al_xwin_get_keyboard_mapping()) {
-      return 1;
-   }
+   _al_xwin_get_keyboard_mapping();
 
    _al_mutex_unlock(&s->lock);
 
@@ -1026,8 +1005,6 @@ static void handle_key_press(int mycode, int unichar, int filtered,
    }
    _al_event_source_unlock(&the_keyboard.parent.es);
 
-// FIXME?
-#ifndef ALLEGRO_RASPBERRYPI
    /* Toggle mouse grab key.  The system driver should not be locked here. */
    if (last_press_code && !is_repeat) {
       ALLEGRO_SYSTEM_XGLX *system = (void *)al_get_system_driver();
@@ -1041,7 +1018,6 @@ static void handle_key_press(int mycode, int unichar, int filtered,
             al_grab_mouse(display);
       }
    }
-#endif
 
    /* Exit by Ctrl-Alt-End.  */
    if ((_al_three_finger_flag)
@@ -1086,5 +1062,4 @@ static void handle_key_release(int mycode, ALLEGRO_DISPLAY *display)
    _al_event_source_unlock(&the_keyboard.parent.es);
 }
 
-
 /* vim: set sts=3 sw=3 et: */
