@@ -19,7 +19,6 @@
 #define MAX_TRANS    8
 #define MAX_FONTS    16
 #define MAX_VERTICES 100
-#define MAX_POLYGONS 8
 
 typedef struct {
    ALLEGRO_USTR   *name;
@@ -47,7 +46,7 @@ typedef struct {
 typedef struct {
    ALLEGRO_USTR   *name;
    ALLEGRO_FONT   *font;
-} NamedFont;
+} Font;
 
 int               argc;
 char              **argv;
@@ -56,11 +55,8 @@ ALLEGRO_BITMAP    *membuf;
 Bitmap            bitmaps[MAX_BITMAPS];
 LockRegion        lock_region;
 Transform         transforms[MAX_TRANS];
-NamedFont         fonts[MAX_FONTS];
+Font              fonts[MAX_FONTS];
 ALLEGRO_VERTEX    vertices[MAX_VERTICES];
-float             simple_vertices[2 * MAX_VERTICES];
-int               num_simple_vertices;
-int               vertex_counts[MAX_POLYGONS];
 int               num_global_bitmaps;
 float             delay = 0.0;
 bool              save_outputs = false;
@@ -124,7 +120,6 @@ static void error(char const *msg, ...)
    fprintf(stderr, "test_driver: ");
    vfprintf(stderr, msg, ap);
    fprintf(stderr, "\n");
-   fprintf(stderr, "See --help for usage\n");
    va_end(ap);
    exit(EXIT_FAILURE);
 }
@@ -138,11 +133,11 @@ static char const *bmp_type_to_string(BmpType bmp_type)
    return "error";
 }
 
-static ALLEGRO_BITMAP *load_relative_bitmap(char const *filename, int flags)
+static ALLEGRO_BITMAP *load_relative_bitmap(char const *filename)
 {
    ALLEGRO_BITMAP *bmp;
 
-   bmp = al_load_bitmap_flags(filename, flags);
+   bmp = al_load_bitmap(filename);
    if (!bmp) {
       error("failed to load %s", filename);
    }
@@ -150,7 +145,7 @@ static ALLEGRO_BITMAP *load_relative_bitmap(char const *filename, int flags)
 }
 
 static void load_bitmaps(ALLEGRO_CONFIG const *cfg, const char *section,
-   BmpType bmp_type, int flags)
+   BmpType bmp_type)
 {
    int i = 0;
    ALLEGRO_CONFIG_ENTRY *iter;
@@ -162,7 +157,7 @@ static void load_bitmaps(ALLEGRO_CONFIG const *cfg, const char *section,
       value = al_get_config_value(cfg, section, key);
 
       bitmaps[i].name = al_ustr_new(key);
-      bitmaps[i].bitmap[bmp_type] = load_relative_bitmap(value, flags);
+      bitmaps[i].bitmap[bmp_type] = load_relative_bitmap(value);
 
       key = al_get_next_config_entry(&iter);
       i++;
@@ -266,15 +261,6 @@ static ALLEGRO_BITMAP *get_bitmap(char const *value, BmpType bmp_type,
    return NULL;
 }
 
-static int get_load_bitmap_flag(char const *value)
-{
-   if (streq(value, "ALLEGRO_NO_PREMULTIPLIED_ALPHA"))
-      return ALLEGRO_NO_PREMULTIPLIED_ALPHA;
-   if (streq(value, "ALLEGRO_KEEP_INDEX"))
-      return ALLEGRO_KEEP_INDEX;
-   return atoi(value);
-}
-
 static int get_draw_bitmap_flag(char const *value)
 {
    if (streq(value, "ALLEGRO_FLIP_HORIZONTAL"))
@@ -302,10 +288,6 @@ static int get_blend_factor(char const *value)
       : streq(value, "ALLEGRO_ONE") ? ALLEGRO_ONE
       : streq(value, "ALLEGRO_ALPHA") ? ALLEGRO_ALPHA
       : streq(value, "ALLEGRO_INVERSE_ALPHA") ? ALLEGRO_INVERSE_ALPHA
-      : streq(value, "ALLEGRO_SRC_COLOR") ? ALLEGRO_SRC_COLOR
-      : streq(value, "ALLEGRO_DEST_COLOR") ? ALLEGRO_DEST_COLOR
-      : streq(value, "ALLEGRO_INVERSE_SRC_COLOR") ? ALLEGRO_INVERSE_SRC_COLOR
-      : streq(value, "ALLEGRO_INVERSE_DEST_COLOR") ? ALLEGRO_INVERSE_DEST_COLOR
       : atoi(value);
 }
 
@@ -371,13 +353,6 @@ static int get_lock_bitmap_flags(char const *v)
       : atoi(v);
 }
 
-static int get_bitmap_flags(char const *v)
-{
-   return streq(v, "ALLEGRO_MEMORY_BITMAP") ? ALLEGRO_MEMORY_BITMAP
-      : streq(v, "ALLEGRO_VIDEO_BITMAP") ? ALLEGRO_VIDEO_BITMAP
-      : atoi(v);
-}
-
 static void fill_lock_region(LockRegion *lr, float alphafactor, bool blended)
 {
    int x, y;
@@ -523,59 +498,6 @@ static void fill_vertices(ALLEGRO_CONFIG const *cfg, char const *name)
 #undef MAXBUF
 }
 
-static void fill_simple_vertices(ALLEGRO_CONFIG const *cfg, char const *name)
-{
-#define MAXBUF    80
-
-   char const *value;
-   char buf[MAXBUF];
-   float x, y;
-   int i;
-
-   memset(simple_vertices, 0, sizeof(simple_vertices));
-
-   for (i = 0; i < MAX_VERTICES; i++) {
-      sprintf(buf, "v%d", i);
-      value = al_get_config_value(cfg, name, buf);
-      if (!value)
-         break;
-
-      if (sscanf(value, " %f , %f", &x, &y) == 2) {
-         simple_vertices[2*i + 0] = x;
-         simple_vertices[2*i + 1] = y;
-      }
-   }
-
-   num_simple_vertices = i;
-
-#undef MAXBUF
-}
-
-static void fill_vertex_counts(ALLEGRO_CONFIG const *cfg, char const *name)
-{
-#define MAXBUF    80
-
-   char const *value;
-   char buf[MAXBUF];
-   int count;
-   int i;
-
-   memset(vertex_counts, 0, sizeof(vertex_counts));
-
-   for (i = 0; i < MAX_POLYGONS; i++) {
-      sprintf(buf, "p%d", i);
-      value = al_get_config_value(cfg, name, buf);
-      if (!value)
-         break;
-
-      if (sscanf(value, " %d ", &count) == 1) {
-         vertex_counts[i] = count;
-      }
-   }
-
-#undef MAXBUF
-}
-
 static int get_prim_type(char const *value)
 {
    return streq(value, "ALLEGRO_PRIM_POINT_LIST") ? ALLEGRO_PRIM_POINT_LIST
@@ -588,25 +510,6 @@ static int get_prim_type(char const *value)
       : atoi(value);
 }
 
-static int get_line_join(char const *value)
-{
-   return streq(value, "ALLEGRO_LINE_JOIN_NONE") ? ALLEGRO_LINE_JOIN_NONE
-      : streq(value, "ALLEGRO_LINE_JOIN_BEVEL") ? ALLEGRO_LINE_JOIN_BEVEL
-      : streq(value, "ALLEGRO_LINE_JOIN_ROUND") ? ALLEGRO_LINE_JOIN_ROUND
-      : streq(value, "ALLEGRO_LINE_JOIN_MITER") ? ALLEGRO_LINE_JOIN_MITER
-      : atoi(value);
-}
-
-static int get_line_cap(char const *value)
-{
-   return streq(value, "ALLEGRO_LINE_CAP_NONE") ? ALLEGRO_LINE_CAP_NONE
-      : streq(value, "ALLEGRO_LINE_CAP_SQUARE") ? ALLEGRO_LINE_CAP_SQUARE
-      : streq(value, "ALLEGRO_LINE_CAP_ROUND") ? ALLEGRO_LINE_CAP_ROUND
-      : streq(value, "ALLEGRO_LINE_CAP_TRIANGLE") ? ALLEGRO_LINE_CAP_TRIANGLE
-      : streq(value, "ALLEGRO_LINE_CAP_CLOSED") ? ALLEGRO_LINE_CAP_CLOSED
-      : atoi(value);
-}
-
 /* FNV-1a algorithm, parameters from:
  * http://www.isthe.com/chongo/tech/comp/fnv/index.html
  */
@@ -827,28 +730,13 @@ static void check_similarity(ALLEGRO_CONFIG const *cfg,
    
    if (bmp_type == HW) {
       char const *exp = al_get_config_value(cfg, testname, "hash_hw");
-      char const *sigexp = al_get_config_value(cfg, testname, "sig_hw");
       char hash[16];
-      char sig[SIG_LENZ];
       sprintf(hash, "%08x", hash_bitmap(bmp1));
       if (exp && streq(hash, exp)) {
          printf("OK   %s [%s]\n", testname, bt);
          passed_tests++;
          return;
       }
-
-      if (sigexp && strlen(sigexp) != SIG_LEN) {
-         printf("WARNING: ignoring bad signature: %s\n", sigexp);
-         sigexp = NULL;
-      }
-
-      compute_signature(bmp1, sig);
-
-      if (sigexp && similar_signatures(sig, sigexp)) {
-         printf("OK   %s [%s] - by signature\n", testname, bt);
-         passed_tests++;
-         return;
-      }
    }
 
    /* The default cutoff is "empirically determined" only. */
@@ -953,16 +841,6 @@ static void do_test(ALLEGRO_CONFIG *cfg, char const *testname,
          continue;
       }
 
-      if (SCAN("al_set_new_bitmap_flags", 1)) {
-         al_set_new_bitmap_flags(get_bitmap_flags(V(0)));
-         continue;
-      }
-
-      if (SCAN("al_set_new_bitmap_format", 1)) {
-         al_set_new_bitmap_format(get_pixel_format(V(0)));
-         continue;
-      }
-
       if (SCAN("al_clear_to_color", 1)) {
          al_clear_to_color(C(0));
          continue;
@@ -1072,12 +950,7 @@ static void do_test(ALLEGRO_CONFIG *cfg, char const *testname,
 
       if (SCANLVAL("al_load_bitmap", 1)) {
          ALLEGRO_BITMAP **bmp = reserve_local_bitmap(lval, bmp_type);
-         (*bmp) = load_relative_bitmap(V(0), 0);
-         continue;
-      }
-      if (SCANLVAL("al_load_bitmap_flags", 2)) {
-         ALLEGRO_BITMAP **bmp = reserve_local_bitmap(lval, bmp_type);
-         (*bmp) = load_relative_bitmap(V(0), get_load_bitmap_flag(V(1)));
+         (*bmp) = load_relative_bitmap(V(0));
          continue;
       }
       if (SCAN("al_save_bitmap", 2)) {
@@ -1122,13 +995,6 @@ static void do_test(ALLEGRO_CONFIG *cfg, char const *testname,
          continue;
       }
 
-      /* Conversion */
-      if (SCAN("al_convert_bitmap", 1)) {
-         ALLEGRO_BITMAP *bmp = B(0);
-         al_convert_bitmap(bmp);
-         continue;
-      }
-
       /* Locking */
       if (SCAN("al_lock_bitmap", 3)) {
          ALLEGRO_BITMAP *bmp = B(0);
@@ -1284,54 +1150,6 @@ static void do_test(ALLEGRO_CONFIG *cfg, char const *testname,
          continue;
       }
 
-      /* Keep 5.0 and 5.1 functions separate for easier merging. */
-
-      /* Primitives (5.1) */
-      if (SCAN("al_draw_polyline", 6)) {
-         fill_simple_vertices(cfg, V(0));
-         al_draw_polyline(simple_vertices, 2 * sizeof(float),
-            num_simple_vertices, get_line_join(V(1)), get_line_cap(V(2)),
-            C(3), F(4), F(5));
-         continue;
-      }
-      if (SCAN("al_draw_polygon", 5)) {
-         fill_simple_vertices(cfg, V(0));
-         al_draw_polygon(simple_vertices, num_simple_vertices,
-            get_line_join(V(1)), C(2), F(3), F(4));
-         continue;
-      }
-      if (SCAN("al_draw_filled_polygon", 2)) {
-         fill_simple_vertices(cfg, V(0));
-         al_draw_filled_polygon(simple_vertices, num_simple_vertices, C(1));
-         continue;
-      }
-      if (SCAN("al_draw_filled_polygon_with_holes", 3)) {
-         fill_simple_vertices(cfg, V(0));
-         fill_vertex_counts(cfg, V(1));
-         al_draw_filled_polygon_with_holes(simple_vertices, vertex_counts, C(2));
-         continue;
-      }
-
-      /* Transformations (5.1) */
-      if (SCAN("al_horizontal_shear_transform", 2)) {
-         al_horizontal_shear_transform(get_transform(V(0)), F(1));
-         continue;
-      }
-      if (SCAN("al_vertical_shear_transform", 2)) {
-         al_vertical_shear_transform(get_transform(V(0)), F(1));
-         continue;
-      }
-      if (SCAN("al_orthographic_transform", 7)) {
-         al_orthographic_transform(get_transform(V(0)), F(1), F(2), F(3), F(4), F(5), F(6));
-         continue;
-      }
-      if (SCAN("al_set_projection_transform", 1)) {
-         if (!display)
-            error("cannot call al_set_projection_transform without creating a display");
-         al_set_projection_transform(display, get_transform(V(0)));
-         continue;
-      }
-
       error("statement didn't scan: %s", stmt);
    }
 
@@ -1365,9 +1183,6 @@ static void do_test(ALLEGRO_CONFIG *cfg, char const *testname,
       al_rest(delay);
    }
 
-   /* Ensure we don't target a bitmap which is about to be destroyed. */
-   al_set_target_bitmap(display ? al_get_backbuffer(display) : NULL);
-
    /* Destroy local bitmaps. */
    for (i = num_global_bitmaps; i < MAX_BITMAPS; i++) {
       if (bitmaps[i].name) {
@@ -1392,13 +1207,13 @@ static void sw_hw_test(ALLEGRO_CONFIG *cfg, char const *testname)
    int old_failed_tests = failed_tests;
    bool reliable;
 
-   al_set_new_bitmap_flags(ALLEGRO_MEMORY_BITMAP);
+   al_set_new_bitmap_flags(ALLEGRO_MEMORY_BITMAP | ALLEGRO_NO_PREMULTIPLIED_ALPHA);
    do_test(cfg, testname, membuf, SW, true);
 
    reliable = (failed_tests == old_failed_tests);
 
    if (display) {
-      al_set_new_bitmap_flags(ALLEGRO_VIDEO_BITMAP);
+      al_set_new_bitmap_flags(ALLEGRO_VIDEO_BITMAP | ALLEGRO_NO_PREMULTIPLIED_ALPHA);
       do_test(cfg, testname, al_get_backbuffer(display), HW, reliable);
    }
 }
@@ -1514,7 +1329,7 @@ static void process_ini_files(void)
 
    while (argc > 0) {
       if (!has_suffix(argv[0], ".ini"))
-         error("expected .ini argument: %s\n", argv[0]);
+         error("expected .ini arument: %s\n", argv[0]);
       cfg = al_load_config_file(argv[0]);
       if (!cfg)
          error("failed to load config file %s", argv[0]);
@@ -1525,12 +1340,12 @@ static void process_ini_files(void)
       argc--;
       argv++;
 
-      al_set_new_bitmap_flags(ALLEGRO_MEMORY_BITMAP);
-      load_bitmaps(cfg, "bitmaps", SW, ALLEGRO_NO_PREMULTIPLIED_ALPHA);
+      al_set_new_bitmap_flags(ALLEGRO_MEMORY_BITMAP | ALLEGRO_NO_PREMULTIPLIED_ALPHA);
+      load_bitmaps(cfg, "bitmaps", SW);
 
       if (display) {
-         al_set_new_bitmap_flags(ALLEGRO_VIDEO_BITMAP);
-         load_bitmaps(cfg, "bitmaps", HW, ALLEGRO_NO_PREMULTIPLIED_ALPHA);
+         al_set_new_bitmap_flags(ALLEGRO_VIDEO_BITMAP | ALLEGRO_NO_PREMULTIPLIED_ALPHA);
+         load_bitmaps(cfg, "bitmaps", HW);
       }
 
       load_fonts(cfg, "fonts");
@@ -1551,30 +1366,8 @@ static void process_ini_files(void)
    }
 }
 
-const char* help_str =
-" [OPTION] CONFIG_FILE [TEST_NAME]... [CONFIG_FILE [TEST_NAME]...]...\n"
-"\n"
-"Run Allegro graphical output tests within one or more CONFIG_FILEs (each\n"
-"having an .ini extension). By default this program runs all the tests in a\n"
-"file, but individual TEST_NAMEs can be specified after each CONFIG_FILE.\n"
-"\n"
-"Options:\n"
-" -d, --delay        duration (in sec) to wait between tests\n"
-" --force-d3d        force using D3D (Windows only)\n"
-" --force-opengl-1.2 force using OpenGL 1.2\n"
-" --force-opengl-2.0 force using OpenGL 2.0\n"
-" --force-opengl     force using OpenGL\n"
-" -h, --help         display this message\n"
-" -n, --no-display   do not create a display (hardware drawing is disabled)\n"
-" -s, --save         save the output of each test in the current directory\n"
-" --use-shaders      use the programmable pipeline for drawing\n"
-" -v, --verbose      show additional information after each test\n"
-" -q, --quiet        do not draw test output to the display\n";
-
 int main(int _argc, char *_argv[])
 {
-   int display_flags = 0;
-
    argc = _argc;
    argv = _argv;
 
@@ -1613,26 +1406,19 @@ int main(int _argc, char *_argv[])
       else if (streq(opt, "--force-opengl-1.2")) {
          ALLEGRO_CONFIG *cfg = al_get_system_config();
          al_set_config_value(cfg, "opengl", "force_opengl_version", "1.2");
-         display_flags |= ALLEGRO_OPENGL;
+         al_set_new_display_flags(ALLEGRO_OPENGL);
       }
       else if (streq(opt, "--force-opengl-2.0")) {
          ALLEGRO_CONFIG *cfg = al_get_system_config();
          al_set_config_value(cfg, "opengl", "force_opengl_version", "2.0");
-         display_flags |= ALLEGRO_OPENGL;
+         al_set_new_display_flags(ALLEGRO_OPENGL);
       }
       else if (streq(opt, "--force-opengl")) {
-         display_flags |= ALLEGRO_OPENGL;
+         al_set_new_display_flags(ALLEGRO_OPENGL);
       }
       else if (streq(opt, "--force-d3d")) {
          /* Don't try this at home. */
-         display_flags |= ALLEGRO_DIRECT3D_INTERNAL;
-      }
-      else if (streq(opt, "--use-shaders")) {
-         display_flags |= ALLEGRO_PROGRAMMABLE_PIPELINE;
-      }
-      else if (streq(opt, "-h") || streq(opt, "--help")) {
-         printf("Usage:\n%s%s", _argv[0], help_str);
-         return 0;
+         al_set_new_display_flags(ALLEGRO_DIRECT3D_INTERNAL);
       }
       else {
          break;
@@ -1640,7 +1426,6 @@ int main(int _argc, char *_argv[])
    }
 
    if (want_display) {
-      al_set_new_display_flags(display_flags);
       display = al_create_display(640, 480);
       if (!display) {
          error("failed to create display");
